/**
 * Seed - a generic javascript library file created by Jialiang (Cain) 
 * 
 */

if (!this.seed) {
	// namespace objects
	var seed = {
		resourceRoot: '/$/',
		imgPathAttrName: '_src',
		linkPathAttrName: '_href',
		
		isEmptyString: function(str) {
			if (!str) {
				return true;
			} else {
				if (str.length == 0) {
					return true;
				}
			}
			
			return false;
		},
		
		getAbsPath: function(src) {
			var colonIndex = src.indexOf(seed.resourceRoot);
			if (colonIndex != -1) {
				var url = seed.path + src.substring(colonIndex + seed.resourceRoot.length);
				return url;
			}
			
			return src;
		},
		
		addOnLoad: function(func) {
			if (!this.onloads) {
				this.onloads = [];
			} 
			
			this.onloads[this.onloads.length] = func;
		},
		
		importJs: function(src) {
			var scriptNode = document.createElement('script');
			scriptNode.type = "text/javascript";
			scriptNode.src = this.getAbsPath(src);
			var head = document.getElementsByTagName('head')[0];
			head.appendChild(scriptNode);
		},
		
		importCss: function(src) {
			var styleNode = document.createElement('link');
			styleNode.rel = "stylesheet";
			styleNode.type = "text/css";
			styleNode.href = this.getAbsPath(src);
			var head = document.getElementsByTagName('head')[0];
			head.appendChild(styleNode);
		}
	};
	
	// gets the dom node by id
	var $ = function(id) {
		return document.getElementById(id);
	}
	
	// gets the dom node value by id
	var $val = function(id) {
		return $(id).value;
	}
	
	// event utilities
	seed.event = {
		getEvent: function(e) {
			return e || window.event;
		},
		
		getTarget: function(e) {
			e = this.getEvent(e);
			var target;
			
			target = e.target || e.srcElement;
			
			if (target.nodeType == 3) {
				target = target.parentNode;
			}
			
			return target;
		},
		
		getKeyCode: function(e) {
			e = this.getEvent(e);
			return e.keyCode || e.which;			
		},
		
		getKeyChar: function(e) {
			var keyCode = this.getKeyCode(e);
			var character = String.fromCharCode(keyCode).toLowerCase();			
			return character;
		}
	};
	
	seed.dom = {
		getAttribute: function(node, name) {
			var attrs = node.attributes;
			for (var i in attrs) {
				var attr = attrs[i];
				if (attr&& attr.nodeName && attr.nodeName == name) {
					return attr.nodeValue;
				}
			}
		}
	}
	
	seed.ajax = {
		get: function(url, params, onload, onerror) {
			var req = new seed.ajax.Request(url, params, onload, onerror, 'GET');
			req.send();
		},
		
		post: function(url, params, onload, onerror) {
			var req = new seed.ajax.Request(url, params, onload, onerror, 'POST');
			req.send();
		},
		
		getSync: function(url, params, onerror) {
			var req = new seed.ajax.Request(url, params, null, onerror, 'GET', null, false);
			req.send();
			return req.request;
		},
		
		postSync: function(url, params, onerror) {
			var req = new seed.ajax.Request(url, params, null, onerror, 'POST', null, false);
			req.send();
			return req.request;
		},
		
		getXML: function(url, params, onload, onerror) {
			var req = new seed.ajax.Request(url, params, onload, onerror, 'GET', 'XML');
			req.send();
		},
		
		postXML: function(url, params, onload, onerror) {
			var req = new seed.ajax.Request(url, params, onload, onerror, 'POST', 'XML');
			req.send();
		},
		
		getXMLSync: function(url, params, onerror) {
			var req = this.getSync(url, params, onerror);
			return req.responseXML;
		},
		
		postXMLSync: function(url, params, onerror) {
			var req = this.postSync(url, params, onerror);
			return req.responseXML;
		},
		
		getJson: function(url, params, onload, onerror) {
			var req = new seed.ajax.Request(url, params, onload, onerror, 'GET', 'JSON');
			req.send();
		},
		
		postJson: function(url, params, onload, onerror) {
			var req = new seed.ajax.Request(url, params, onload, onerror, 'POST', 'JSON');
			req.send();
		},
		
		getJsonSync: function(url, params, onerror) {
			var text = this.getTextSync(url, params, onerror);
			var json = eval('(' + text + ')');
			return json;
		},
		
		postJsonSync: function(url, params, onerror) {
			var text = this.postTextSync(url, params, onerror);
			var json = eval('(' + text + ')');
			return json;
		},
		
		getText: function(url, params, onload, onerror) {
			var req = new seed.ajax.Request(url, params, onload, onerror, 'GET', 'TEXT');
			req.send();
		},
		
		postText: function(url, params, onload, onerror) {
			var req = new seed.ajax.Request(url, params, onload, onerror, 'POST', 'TEXT');
			req.send();
		},
		
		getTextSync: function(url, params, onerror) {
			var req = this.getSync(url, params, onerror);
			return req.responseText;
		},
		
		postTextSync: function(url, params, onerror) {
			var req = this.postSync(url, params, onerror);
			return req.responseText;
		},
		
		postTextSync: function(url, params, onerror) {
			var req = this.postSync(url, params, onerror);
			return req.responseText;
		},
		
		Request: function(url, params, onload, onerror, method, parsingMethod, async) {
			this.url = url;
			this.onload = onload;
			this.onerror = onerror;
			this.params = params;			
			if (!method) {
				method = 'GET';
			}
			this.method = method;			
			this.parsingMethod = parsingMethod;			
			if (async == null || typeof async == 'undefined') {
				async = true;
			}
			this.sync = async;
			
			this.createXMLHttpRequest = function() {
				if (window.XMLHttpRequest) {
					this.request = new XMLHttpRequest();
				} else if (window.ActiveXObject) {
					this.request = new ActiveXObject("Microsoft.XMLHTTP");
				}
				
				return this.request;
			};
			
			this.send = function() {
				if (!this.request) {
					this.createXMLHttpRequest();
				}
				
				var url = this.url;
				url = seed.getAbsPath(url);
				if (this.request && !seed.isEmptyString(url)) {
					var req = this.request;
					var sender = this;
					req.onreadystatechange = function() {
						sender.onReadyStateChange.call(sender);
					};
					
					if ('GET' == this.method) {
						if (!seed.isEmptyString(this.params)) {
							url += '?' + params;
						}
						req.open('GET', url, async);
						req.send(null);
					} else if ('POST' == this.method) {
						req.open('POST', url, async);
						req.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
						req.send(this.params);
					}
				}
			};
			
			this.onReadyStateChange = function() {
				var req = this.request;
				var readyState = req.readyState;
				
				if (readyState == 4) {
					var httpStatus = req.status;
					
					if (httpStatus == 200 || httpStatus == 0) {
						if (this.onload) {
							if ('XML' == this.parsingMethod) {
								this.onload.call(this, req.responseXML);
							} else if ('JSON' == this.parsingMethod) {
								var json = eval('(' + req.responseText + ')');
								this.onload.call(this, json);
							} else if ('TEXT' == this.parsingMethod) {
								this.onload.call(this, req.responseText);
							} else {
								this.onload.call(this, req);
							}
						}
					} else {
						if (this.onerror) {
							this.onerror.call(this, req);
						}
					}
				}
			};
		}
	};
	
	(function() {
		// gets the absoluate path of seed.js
		var jsName = 'seed.js';
		
		var scriptNodes = document.getElementsByTagName('script');
		for (var i = 0; i < scriptNodes.length; i ++) {
			var scriptNode = scriptNodes.item(i);
			var src = scriptNode.src;
			if (src.substring(src.length - jsName.length, src.length) == jsName) {
				seed.path = src.substring(0, src.length - jsName.length);
				break;
			}
		}
	})();
	
	window.onload = function() {
		if (seed.onloads) {
			for (var i in seed.onloads) {
				seed.onloads[i].call();
			}
		}
		
		// parse absoluate path of images
		var imgNodes = document.getElementsByTagName('img');
		for (var i = 0; i < imgNodes.length; i ++) {
			var imgNode = imgNodes.item(i);
			var src = seed.dom.getAttribute(imgNode, seed.imgPathAttrName);
			if (src) {
				var colonIndex = src.indexOf(seed.resourceRoot);
				if (colonIndex != -1) {
					var url = seed.path + src.substring(colonIndex + seed.resourceRoot.length);
					imgNode.src = url;
				}
			}
		}
		
		// parse absoluate path of links
		var linkNodes = document.getElementsByTagName('a');
		for (var i = 0; i < linkNodes.length; i ++) {
			var linkNode = linkNodes.item(i);
			var href = seed.dom.getAttribute(linkNode, seed.linkPathAttrName);
			if (href) {
				var colonIndex = href.indexOf(seed.resourceRoot);
				if (colonIndex != -1) {
					var url = seed.path + href.substring(colonIndex + seed.resourceRoot.length);
					linkNode.href = url;
				}
			}
		}
	};
}
