/**
 *
 */
package edu.cmu.radar.dynaweb.actions;

import java.util.LinkedList;
import java.util.List;

import edu.cmu.radar.dynaweb.entities.Location;

/**
 * @author Jialiang
 *
 */
public class UCRoomAction extends RoomAction {
	/**
	 * the number of rooms in each group
	 */
	private final static int groupSize = 5;

	/**
	 * the formatted capacity names
	 */
	private List<String> fmtCapNames = new LinkedList<String>();

	/**
	 * the room capacity formatter
	 * */
	private static CapacityFormat format = new UCCapacityFormat();

	/**
	 * the groups of rooms
	 */
	private List<List<Room>> roomGroups = new LinkedList<List<Room>>();

	/*
	 * (non-Javadoc)
	 *
	 * @see com.opensymphony.xwork2.ActionSupport#execute()
	 */
	public String execute() throws Exception {
		List<Location> locations = services.getLocation("UC");
		int size = locations.size();

		int groupCount = size / groupSize;
		if (size % groupSize != 0) {
			groupCount++;
		}

		List<String> capNames = getCapacityNames(locations);
		fmtCapNames = getFormattedCapacityNames(capNames);
		List<Room> ucRooms = getRooms(locations, capNames, format);
		for (int i = 0; i < size; i++) {
			int groupIndex = i / groupSize;

			List<Room> rooms = null;
			if (groupIndex > roomGroups.size() - 1) {
				rooms = new LinkedList<Room>();
				roomGroups.add(rooms);
			} else {
				rooms = roomGroups.get(groupIndex);
			}

			rooms.add(ucRooms.get(i));
		}

		return SUCCESS;
	}

	/**
	 * gets the number of room groups
	 */
	public int getRoomGroupCount() {
		return roomGroups.size();
	}

	/**
	 * @return the fmtCapNames
	 */
	public List<String> getFmtCapNames() {
		return fmtCapNames;
	}

	/**
	 * @return the roomGroups
	 */
	public List<List<Room>> getRoomGroups() {
		return roomGroups;
	}

}

class UCCapacityFormat implements CapacityFormat {

	/*
	 * (non-Javadoc)
	 *
	 * @see edu.cmu.radar.dynaweb.actions.CapacityFormat#format(java.lang.String,
	 *      java.lang.String)
	 */
	public String format(String configuration, String capacity) {
		if (configuration == null || capacity == null) {
			return "n/a";
		}
		try {
		if (!"0".equals(capacity) && (Integer.parseInt(capacity)==0)) {
			return "n/a";
		}
		} catch (Exception e) {
			return "n/a";
		}



		return capacity;

	}

}
