package edu.cmu.radar.dynaweb.utils;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

/**
 * @author Jialiang (Cain)
 * 
 * creates a thread safe hibernate session
 */
public class HibernateUtils {
	/**
	 * the hibernate configuration file path
	 */
	public final static String HIBERNATE_CFG_PATH = "dynaweb.hibernate.xml";

	/**
	 * thread local object to contain the hibernate session associated with
	 * current thread
	 */
	private static ThreadLocal<Session> threadLocal = new ThreadLocal<Session>();

	/**
	 * the unique session factory
	 */
	private static SessionFactory sessionFactory;

	static {
		Configuration config = new Configuration();
		config.configure(HIBERNATE_CFG_PATH);
		sessionFactory = config.buildSessionFactory();
	}

	/**
	 * gets the session associated with current thread; creates one if none
	 * exists
	 */
	public static Session getSession() {
		Session session = threadLocal.get();

		if (session == null) {
			session = sessionFactory.openSession();
			// put into thread local object, so that this session could be
			// reused before closed in the thread
			threadLocal.set(session);
			return session;
		} else {
			return session;
		}
	}

	/**
	 * closes the session and clear it from the current thread
	 * */
	public static void close(Session session) {
		threadLocal.set(null);
		session.close();
	}
}
