-- STP ontology
-- revised by mlahut Jan 06

DROP SCHEMA stp CASCADE;
CREATE SCHEMA stp;

-- do we want some sort of constant string table to hold the header? 
-- this shouldn't change regularly, but it might occasionally need an extra few lines
-- it could also be owned by the STP team


CREATE TABLE stp.roomdata (
	id SERIAL PRIMARY KEY,
	location_id INTEGER, -- base location table (radar.location)
	markedusable TEXT, -- "True" iff part of the initial configuration of usable rooms
	starttime INTEGER, -- UNIX time
	duration INTEGER, -- seconds
	inherit TEXT, 
	-- inherit is possibly unnecessary: the STP allows selective omission of properties, 
	-- and any property not specified gets assigned to the property of whatever other 
	-- room it inherits from. This doesn't need to be an integer ID because the schema 
	-- would never use it, but it might not even be necessary if we're automatically 
	-- filling in every field of every room.
	
	stationspud TEXT, 
	microphonespud TEXT,
	projectorpud TEXT,

	availabilityplf TEXT, -- availability of the room in PLF string format
	
	-- the following are added from http://www.cs.cmu.edu/~eugene/Radar/Algs/props.txt
	-- all are TEXT because they may end up being a PUD
	floor TEXT,
	len TEXT,
	width TEXT,
	area TEXT,
	height TEXT,
	microphones TEXT,
	projectors TEXT,
	computers TEXT,
	stations TEXT
	
);

CREATE TABLE stp.roomcapacity (
	id SERIAL PRIMARY KEY, -- one-to-one with radar.locationcapacity (this means
						   -- that the id values are mirrored)
	capacitypud TEXT -- for -L
	-- potential issue: some +L files still use PUDs for capacity. 
	-- also attempting to find the maximum PUD is not really a straightforward math problem, however in the past we've had no qualms about bullshitting our way past ambiguity, since it doesn't matter much
);

CREATE TABLE stp.persondata (
	id SERIAL PRIMARY KEY,
	weight INTEGER
);
	
CREATE TABLE stp.eventdata (
	id SERIAL PRIMARY KEY,
	gathering_id INTEGER, -- base gathering table
	speaker_id INTEGER, -- STP person table
	room_id INTEGER, -- STP room table
	weight INTEGER,
	startweight INTEGER,
	durationweight INTEGER,
	projectorweight INTEGER,
	microphonesweight INTEGER,
	stationsweight INTEGER,
	movingpenalty FLOAT, -- this is a widget that the STP team frequently uses to hack the optimizer; it tends to be changed in bulk across all events
	assignedstart INTEGER, -- UNIX time
	assignedduration INTEGER, -- seconds
	lockedroom INTEGER,
	lockedduration INTEGER,
	lockedstart INTEGER,
	eventtype TEXT,
	inherit TEXT, -- see footnote in room table

	startprefplf TEXT,
	durationprefplf TEXT,
	projectorprefplf TEXT,
	microphonesprefplf TEXT,
	stationsprefplf TEXT,

	temporalconstraintplf TEXT, -- such as "must be after" or "non-overlap"; perhaps leave this field as STP code to be inserted

	-- the following are added from http://www.cs.cmu.edu/~eugene/Radar/Algs/props.txt
	-- all are TEXT because they may end up being a PUD
	len TEXT,
	width TEXT,
	area TEXT,
	height TEXT,
	attendance TEXT,
	arrangement TEXT,
	duration TEXT,
	microphones TEXT, --
	projectors TEXT,  -- these may obsolete some previous entries
	computers TEXT,   -- but keeping both for compatibility now
	stations TEXT,    -- 
	catering TEXT,
	vegetarian TEXT,
	kosher TEXT,
	vegan TEXT,
	atkins TEXT,
	alcohol TEXT,
	flowers TEXT,
	guards TEXT,
	flipcharts TEXT,
	laserpointers TEXT,
	easels TEXT,
	videoplayers TEXT
	
);

CREATE TABLE stp.eventcapacityweight (
	id SERIAL PRIMARY KEY,
	event_id INTEGER,
	configuration TEXT, -- classroom, conference, ushape, square, theater, banquet, reception, max?, other
	capacityweight INTEGER,
	capacitypref TEXT 
	-- same footnotes apply as in stp.roomcapacity
);
