-- RADAR Common Ontology Concepts
-- written for postgres
-- Anthony Tomasic 1/12/2005
-- Last Revised 6/29/2006 by Robert McGuire

-- load this file before loading any other .sql file


DROP SCHEMA base CASCADE;
CREATE SCHEMA base;

-- person
-- person is any reference to any person, either inside or outside the 
-- base system
CREATE TABLE base.person (
	id SERIAL PRIMARY KEY,
	first TEXT,	-- first name -- see aka_first below
	middle TEXT,	-- middle name or initial
	last TEXT,	-- last name
	eid TEXT,       -- FirstLast (identifier for STP and other things)
	officephone TEXT,	-- (xxx) xxx-xxxx US format
	homephone TEXT,
	cellphone TEXT,
	labphone TEXT,
	faxphone TEXT,
	email TEXT,		-- xx@xx.xx format
--	cryptmail TEXT,		-- cyptographic representation of email
	organization TEXT,  -- organization affiliation
	personalhref TEXT,	-- personal web page
	bio TEXT,		-- one line description of interests
	picturehref TEXT,	-- url of image

-- ardra specific
	akafirst TEXT,		-- 'a'lso 'k'nown 'a's first name
--	akamiddle TEXT, 	-- also known as middle name or initial
--	akalast TEXT,		-- also known as last name
-- 
	vip TEXT,			-- vip status, used for the corpus generator and
						-- (eventually) scoring?.
	type TEXT,	-- 'www' if the person appears on the website, 
				-- null otherwise
	weight INTEGER -- this person's weight for STP
);

-- gathering_person_speaker (a many-many link table to assign speakers (in the
--							 person table) to gatherings)
CREATE TABLE base.gathering_person_speaker (
	gathering_id INTEGER, 	-- gathering
	person_id INTEGER 		-- speaker
);


-- location
-- room on cmu campus
-- room in fictional ardra site
-- room in fictional ardra flagstaf hotel
CREATE TABLE base.location (
	id SERIAL PRIMARY KEY,
	caption TEXT, 	-- e.g. WEAN 5301
	eid TEXT,	-- NameWithSpacesRemoved (identifier for STP and other things)
	x INTEGER,
	y INTEGER,
	z INTEGER, -- x,y,z coords listed in CMRadar example
	building TEXT,	-- building name, e.g. WEAN, Flagstaf Hotel
	floor TEXT,	-- floor of building 'CL', 'G', '1', etc.
				-- spacetime converts to an integer
	footprint INTEGER,	-- square feet
	width INTEGER,		-- feet
	length INTEGER,		-- feet
	height INTEGER,		-- feet
	type TEXT,		-- classroom, auditorium, conference, reception
					-- this determines which vague stats are
					-- provided to STP
	cost INTEGER,		-- dollars to rent per hour
						-- spacetime converts to non-positive int
						-- or to a pud?
	microphones INTEGER,
	podiums INTEGER,
	projector INTEGER,
	stations INTEGER,
	computers INTEGER,
	cmflag INTEGER
);

-- storage for various room configurations/capacities for locations
CREATE TABLE base.locationcapacity (
	id SERIAL PRIMARY KEY,
	location_id INTEGER,
	configuration TEXT, -- values of classroom, conference, ushape, square,
						-- theater, banquet, reception, min, max, etc...
	capacity TEXT -- capacity and/or PUD for STP
);

-- sponsor table
CREATE TABLE base.sponsor (
	id SERIAL PRIMARY KEY,
	name TEXT 	-- sponsor's name
);

CREATE TABLE base.radarlog (
	id		SERIAL PRIMARY KEY,
	time		TIMESTAMP NOT NULL,
	component	TEXT NOT NULL,
	level		TEXT NOT NULL,
	message		TEXT NOT NULL
);