-- Populates the pim.vendororder and pim.quote tables from existing data in the
-- vendor.* tables.
--
-- Author: Robert McGuire <rmcguire@cs.cmu.edu>
-- Updated: 2006-10-15

-- Clear out the pim.vendororder table
TRUNCATE TABLE pim.vendororder;

-- Copy data from vendor.* tables to pim.vendororder
INSERT INTO pim.vendororder (
		vendor, 
		title, 
		location, 
		eventdate, 
		starttime, 
		endtime, 
		quotedprice, 
		externaltransaction_id,
		status
	)
	SELECT 
		upper(vendor.vendorinfo.name), 
		title, 
		base.location.caption, 
		startdate, 
		TO_TIMESTAMP(setupstart, 'HH24:MI'), 
		TO_TIMESTAMP(setupcomplete, 'HH24:MI'), 
		total, 
		vendor.order.id,
		0
	FROM 
		vendor.order, 
		vendor.vendorinfo, 
		base.location, 
		vendor.vendor_order_total 
	WHERE 
		vendor.order.quoteaccepted = vendor.vendorinfo.typeid 
		AND base.location.id = CAST(vendor.order.location_id as INTEGER)
		AND vendor.vendor_order_total.vendor_typeid = vendor.vendorinfo.typeid 
		AND vendor.vendor_order_total.order_id=vendor.order.id;

-- Set the URLs for each vendor order.
-- URLs differ by 
--  * vendor type (AV, Catering, Floral, Rental, Security)
--  * vendor name
--  * external transaction ID

-- Set URLs for Security orders
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/Security/SecurityReviewQuote.jsp?uniqueID='
			|| externaltransaction_id 
	WHERE vendor=upper('CMU Security Services');

-- Catering Services
-- Set URLs for Breakfast Break
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/Catering/BBreakQuote.jsp?eventSource=CateringForm.htm'
			|| '&uniqueID=' || externaltransaction_id 
	WHERE vendor=upper('Breakfast Break');

-- Set URLs for Maximal Foods
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/Catering/MaximalQuote.jsp?eventSource=CateringForm.htm'
			|| '&uniqueID=' || externaltransaction_id 
	WHERE vendor=upper('Maximal Foods');

-- Set URLs for Pittsburgh Catering
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/Catering/PghQuote.jsp?eventSource=CateringForm.htm'
			|| '&uniqueID=' || externaltransaction_id 
	WHERE vendor=upper('Pittsburgh Catering');

-- AV Services
-- Set URLs for CMU AV Services
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/AV/CMUAVQuote.jsp?eventSource=AVForm.htm'
			|| '&uniqueID=' || externaltransaction_id 
	WHERE vendor=upper('CMU AV Services');

-- Set URLs for Confer.net Services
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/AV/ConferQuote.jsp?eventSource=AVForm.htm'
			|| '&uniqueID=' || externaltransaction_id 
	WHERE vendor=upper('Confer.net Services');

-- Set URLs for Flagstaff Hotel
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/AV/FlagstaffAVQuote.jsp?eventSource=AVForm.htm'
			|| '&uniqueID=' || externaltransaction_id 
	WHERE vendor=upper('Flagstaff Hotel');

-- Floral Services
-- Set URLs for Adam's Floral
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/Floral/AdamsQuote.jsp?eventSource=FloralForm.htm'
			|| '&uniqueID=' || externaltransaction_id 
	WHERE vendor=upper('Adam''s Floral');

-- Set URLs for Canal Street Ltd.
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/Floral/CanalQuote.jsp?eventSource=FloralForm.htm'
			|| '&uniqueID=' || externaltransaction_id 
	WHERE vendor=upper('Canal Street Ltd.');

-- Set URLs for Lilli Festivale & Co.
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/Floral/LilliQuote.jsp?eventSource=FloralForm.htm'
			|| '&uniqueID=' || externaltransaction_id 
	WHERE vendor=upper('Lilli Festivale & Co.');

-- Rental Services
-- Set URLs for Classic Rentals, Inc
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/Rental/ClassicQuote.jsp?eventSource=RentalForm.htm'
			|| '&uniqueID=' || externaltransaction_id 
	WHERE vendor=upper('Classic Rentals, Inc');

-- Set URLs for Rent From Us!
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/Rental/RentFromUsQuote.jsp?eventSource=RentalForm.htm'
			|| '&uniqueID=' || externaltransaction_id 
	WHERE vendor=upper('Rent From Us!');

-- Set URLs for Speedy Delivery Rentals
UPDATE pim.vendororder SET 
	formurl='http://vendor.self.radar.cs.cmu.edu:8080/myapp/webForms'
			|| '/Rental/SpeedyQuote.jsp?eventSource=RentalForm.htm'
			|| '&uniqueID=' || externaltransaction_id 
	WHERE vendor=upper('Speedy Delivery Rentals');

-- Copy data from vendor.* table to pim.quote
-- We do not do this because the initial state should contain no outstanding
-- quotes.
-- INSERT INTO pim.quote (
-- 		vendor, 
-- 		quotedprice, 
-- 		externaltransaction_id
-- 	) 
-- 	SELECT 
-- 		vendor.vendorinfo.name, 
-- 		total, 
-- 		vendor.vendor_order_total.id 
-- 	FROM 
-- 		vendor.vendor_order_total, 
-- 		vendor.vendorinfo 
-- 	WHERE 
-- 		vendor.vendorinfo.typeid = vendor_order_total.vendor_typeid;
