package airspaceGUI.base;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;



/**
 * class to be able to un-do edits
 * 
 * @author srbennet
 * 
 */
class UndoAction extends AbstractAction
{
    /**
     * 
     */
    private transient UndoManager undo;
    
    private transient RedoAction redoAction;
    
    /*
     * this needs to be modified to reset between messages so that one isn't
     * changed with anothers data
     */
    private static final long serialVersionUID = 1L;

    /**
     * Constructs an undoAction
     * @param undo The manager for undos
     * 
     */
    public UndoAction(final UndoManager undo)
    {
        super("Undo");
        this.undo = undo;
        setEnabled(false);
    }

    /**
     * Doesn't do anything, only here so that the getRedoAction will work, 
     * since the two are close to one another.
     *
     */
    public UndoAction()
    {
        // TODO Auto-generated constructor stub
    }

    /**
     * Defines the action for the object to perform
     * 
     * @param e
     *            the action event
     */
    public void actionPerformed(final ActionEvent e) {
        try
        {
            if (undo.canUndo())
            {
                undo.undo();
            } else
            {
                Toolkit.getDefaultToolkit().beep();
            }
            updateUndoState();
            redoAction.updateRedoState();
        } catch (CannotUndoException ex)
        {
            ex.printStackTrace();
        }
    }

    /**
     * updates the undo dtack.
     *
     */
    protected void updateUndoState() {
        if (undo.canUndo())
        {
            setEnabled(true);
            putValue(Action.NAME, undo.getUndoPresentationName());
        } else
        {
            setEnabled(false);
            putValue(Action.NAME, "Undo");
        }
    }
    
    /**
     * sets this object's redo action to the parameter
     * @param redo - the redoAction that complements the undo action.
     */
    public void setRedoAction(final RedoAction redo)
    {
        redoAction = redo;
    }
    
    /**
     * Creates a new RedoAction whose manager is undo
     * @param undo the manager of undo/redo actions
     * @return the newly created redo action
     */
    public RedoAction getRedoAction(final UndoManager undo)
    {
        RedoAction redo = new RedoAction(undo);
        this.undo = undo;
        this.redoAction = redo;
        return redo;
    }
}