package airspaceGUI.base.corpus;

// import java.sql.*; // un-comment this line when a database has the people
// file
import java.util.ArrayList;

import javax.swing.JOptionPane;

import org.hibernate.Session;
import org.hibernate.criterion.Order;

import airspaceGUI.model.PeopleModel;


import edu.cmu.radar.base.Person;

/**
 * Class AddressBook
 * 
 * Implements an address book by reading out of a database all the people in the
 * world
 * 
 * @author Rachael Bennett (srbennet@andrew)
 * 
 */
public class AddressBook
{
    private final transient ArrayList idList; /*
                                                 * the id of the correcponding
                                                 * person who has name and
                                                 * e-mail addresses atLabel the same
                                                 * index in the lists below
                                                 */

    private final transient ArrayList VIPList; /*
                                                 * The VIP status of the person
                                                 * whose other information is in
                                                 * the other lists
                                                 */

    private final transient ArrayList nameList; /*
                                                 * a list of names of the people
                                                 * in this world, whose e-mail
                                                 * addresses are contained in
                                                 * emailList in the
                                                 * corresponding indexes
                                                 */

    private final transient ArrayList emailList; /*
                                                     * the list of emails for
                                                     * the people whose
                                                     * corresponding names are
                                                     * in the corresponding
                                                     * indexes of the nameList
                                                     * above
                                                     */

    // private static SessionFactory sessionFactory;

    private static Session session;
    
    private transient PeopleModel pm;

    /**
     * the number of people in the people file
     */
    // private final transient int numPeople;
    /**
     * Default Constructor, creates an empty AddressBook
     */
    public AddressBook()
    {
        // initializes the variables
        idList = new ArrayList();
        VIPList = new ArrayList();
        nameList = new ArrayList();
        emailList = new ArrayList();
        // numPeople = 0;
    }

    /**
     * Function to access the list of names in the AddressBook
     * 
     * @return the list of names in the AddressBook
     */
    public ArrayList getNames() // get the name of the person
    {
        return nameList;
    }

    /**
     * returns the VIP status of the person atLabel index emailIndex
     * 
     * @param emailIndex
     *            the index of the person
     * @return the VIP status of the person.
     */
    public boolean getVIP(final int emailIndex) {
        return Boolean.valueOf((String)VIPList.get(emailIndex)).booleanValue();
    }

    /**
     * Function to access the list of emails in the AddressBook
     * 
     * @return the list of emails in the AddressBook
     */
    public ArrayList getEmail() {
        return emailList;
    }

    /**
     * populates the people list from the database
     * 
     * @param session
     *            An initialized hibernate session to populate the address book
     *            with
     * 
     */
    public void populateBook(final Session session) {

        AddressBook.session = session;
        AddressBook.session.beginTransaction();

        ArrayList people = new ArrayList();
        try
        {
            people = (ArrayList)session.createCriteria(Person.class).addOrder(
                    Order.asc("last")).list();
        } catch (Exception e)
        {
            JOptionPane.showMessageDialog(null,
                    "The database cannot be accessed.  Please check that corpus.properties "
                            + "has the correct information", "Error",
                    JOptionPane.ERROR_MESSAGE);
        }
        Person workingOn;
        String name;

        for (int i = 0; i < people.size(); i++)
        {
            workingOn = (Person)people.get(i);
            idList.add(workingOn.getId());
            VIPList.add(i, workingOn.getVip());
            name = workingOn.getFirst() + " " + workingOn.getMiddle() + " "
                    + workingOn.getLast();
            nameList.add(name);
            emailList.add(i, workingOn.getEmail());
        }
    }

    /**
     * Populates the address book with the people in the PeopleModel
     * @param pm Contains and manages all people in the world.
     */
    public void populateBook(final PeopleModel pm) {
        final ArrayList people = pm;
        Person workingOn;
        String name;
        
        this.pm = pm;

        for (int i = 0; i < people.size(); i++)
        {
            workingOn = (Person)people.get(i);
            idList.add(workingOn.getId());
            VIPList.add(i, workingOn.getVip());
            if (workingOn.getMiddle() != null)
            {
                name = workingOn.getFirst() + " " + workingOn.getMiddle() + " "
                        + workingOn.getLast();
            } else
            {
                name = workingOn.getFirst() + " " + workingOn.getLast();
            }
            nameList.add(name);
            emailList.add(i, workingOn.getEmail());
        }
    }
    
    /**
     * Gets the list of Ids for the people in the address book
     * 
     * @return the list of ids.
     */
    public ArrayList getIds() {
        return idList;
    }

    /**
     * returns the id of the person with the specified email
     * 
     * @param email
     *            the email that we're looking for the id of
     * @return the id that corresponds to the holder of the email
     */
    public int getId(final String email) {
        final int index = emailList.indexOf(email);
        return ((Integer)idList.get(index)).intValue();
    }

    /**
     * Returns the person attached to the email passed in the parameter
     * 
     * @param text
     *            the email of the person we're looking for
     * @return the person with the specified email.
     */
    public Person getPerson(final String text) {
        int index = emailList.indexOf(text); 
        
        Person person;
        
        if (session == null) { 
            person = (Person)pm.get(index);
        } else
        {
            try{
                index = ((Integer)idList.get(index)).intValue();
            }catch(ArrayIndexOutOfBoundsException exc)
            {
                return null;
            }
            session.beginTransaction();
            person = (Person)session.get(Person.class, new Integer(index));
        }
        // session.close();

        return person;
    }


    /**
     * Reads the people out of the people file. The filename is hardcoded for
     * not will be irrelevant if/when the program is allowed to read from a sql
     * database
     * 
     * @throws IOException
     */
    /*USELESS SINCE WE"RE READING FROM A DATABASE
     * public void populateBook() throws IOException {
     * 
     * BufferedReader people = new BufferedReader(new FileReader(
     * "people.www.sql")); // open the people file String garbage, first,
     * middle, last, email; // strings to store the // lines of the file in
     * String test = " "; String blank = ""; nameList = new ArrayList();
     * emailList = new ArrayList();
     * 
     * while (test != null) { test = people.readLine(); if (test != null &&
     * test.endsWith("VALUES(")) { // if this line marks the start of the
     * definition of a person numPeople++; garbage = people.readLine();
     *//*
         * throw away the id, numPeople the index in the ArrayList serves as the
         * way to reference them in this program rather than their id
         */
    /*
     * if (garbage.compareTo(blank) == 0) // account for extra blank // lines
     * garbage = people.readLine(); // read in the first name of the person
     * first = people.readLine(); if (first.compareTo(blank) == 0) // account
     * for extra blank // lines first = people.readLine(); first =
     * first.substring(2, first.length() - 2); // read in the middle name of the
     * person (if one exists) middle = people.readLine(); if
     * (middle.compareTo(blank) == 0) // account for extra blank // lines middle =
     * people.readLine(); middle = middle.substring(2, middle.length() - 2); //
     * read in the last name of the person last = people.readLine(); if
     * (last.compareTo(blank) == 0) // account for extra blank lines last =
     * people.readLine(); last = last.substring(2, last.length() - 2); // read
     * in the e-mail address of the person email = people.readLine(); if
     * (email.compareTo(blank) == 0) // account for extra blank // lines email =
     * people.readLine(); email = email.substring(2, email.length() - 2);
     * 
     *//*
         * add the name to the ArrayList, accounting for the fact that some
         * people don't have middle names
         */
    /*
     * if (middle.compareTo("") == 0) // account for extra blank lines
     * nameList.add(first + " " + last); else nameList.add(first + " " + middle + " " +
     * last); // add the e-mail address to the ArrayList emailList.add(email); } } }
     */

}