package airspaceGUI.base.corpus;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.hibernate.Session;

import edu.cmu.radar.base.Person;
import edu.cmu.radar.base.corpus.FakeMessage;
import edu.cmu.radar.base.corpus.FakeMessageClass;
import edu.cmu.radar.base.corpus.Label;
import edu.cmu.radar.base.corpus.NormalFakeMessage;
import edu.cmu.radar.base.corpus.Recipient;
import edu.cmu.radar.base.corpus.Variable;
import edu.cmu.radar.base.corpus.VariableSenderFakeMessage;

/**
 * This class is used to transparently control for the differences between
 * NormalFakeMessages and VariableSenderFakeMessages. Becuase of this,
 * developers do not need to worry about which kid of message they actually
 * have.
 * 
 * @author srbennet
 * 
 */
public class CorpusGUIMessage implements MessageType {
	private NormalFakeMessage normalMessage;

	private VariableSenderFakeMessage variableSenderMessage;

	/**
	 * 
	 * @param nfm
	 *            the message that this class is a wrapper of
	 */
	public CorpusGUIMessage(NormalFakeMessage nfm) {
		normalMessage = nfm;
	}

	/**
	 * 
	 * @param vsfm
	 *            the message that this class is a wrapper of
	 */
	public CorpusGUIMessage(VariableSenderFakeMessage vsfm) {
		variableSenderMessage = vsfm;
	}

	/**
	 * 
	 * @param message
	 *            the message to wrap with this class
	 */
	public CorpusGUIMessage(FakeMessage message) {
		if (message instanceof NormalFakeMessage) {
			normalMessage = (NormalFakeMessage) message;
		} else {
			variableSenderMessage = (VariableSenderFakeMessage) message;
		}
	}

	/**
	 * default constructor: creates a new NormalFakeMessage
	 * 
	 */
	public CorpusGUIMessage() {
		normalMessage = new NormalFakeMessage();
	}

	/**
	 * Returns the id of this message
	 * 
	 * @return the id of this message
	 */
	public int getId() {
		if (getMessage().getId() != null) {
			return getMessage().getId().intValue();
		} else {
			return 0;
		}
	}

	/**
	 * 
	 * @return gets the messageId
	 */
	public Integer getMessageId() {
		return getMessage().getMessageId();
	}

	/**
	 * 
	 * @return the messageGroup that this message belongs to
	 */
	public String getMessageGroup() {
		return getMessage().getMessageGroup();
	}

	/**
	 * 
	 * @return the messageClass this message belongs to
	 */
	public Set getMessageClass() {
		return getMessage().getMessageClass();
	}

	/**
	 * @return the group that this message belongs to
	 */
	public String getGroup() {
		return getMessage().getMessageGroup();
	}

	/**
	 * @return true if this message comes from a VIP, false otherwise
	 */
	public boolean getVIP() {
		if (variableSenderMessage == null) {
			return normalMessage.getVIP();
		} else {
			return variableSenderMessage.getVIP();
		}
	}

	/**
	 * @return a string representation of the sender of the message
	 */
	public String getFrom() {
		if (variableSenderMessage == null) {
			final Person from = normalMessage.getFrom();
			String name;

			if (from.getMiddle() != null && !from.getMiddle().equals("")) {
				name = from.getFirst() + " " + from.getMiddle() + " "
						+ from.getLast();
			} else {
				name = from.getFirst() + " " + from.getLast();
			}

			return name;
		} else {
			return variableSenderMessage.getFrom().toVariableString();
		}
	}

	/**
	 * @return a string representation of the sender of the message
	 */
	public String getFromEmail() {
		if (variableSenderMessage == null) {
			final Person from = normalMessage.getFrom();

			return from.getEmail();
		} else {
			return variableSenderMessage.getFrom().toVariableString();
		}
	}

	/**
	 * 
	 * @return the xml representation of the sender of the message
	 */
	public void getXMLFrom(Element fromNode) {
		if (variableSenderMessage == null) {
			Person from = normalMessage.getFrom();
			String name;

			if (from.getMiddle() != null && !from.getMiddle().equals("")) {
				name = from.getFirst() + " " + from.getMiddle() + " "
						+ from.getLast();
			} else {
				name = from.getFirst() + " " + from.getLast();
			}

			addXMLText(fromNode.addElement("personal"), name);
			addXMLText(fromNode.addElement("addr"), from.getEmail());
		} else {
			// addXMLText(fromNode,
			// variableSenderMessage.getFrom().toVariableString());
			try {
				Document tempDoc = DocumentHelper
						.parseText(variableSenderMessage.getFrom()
								.toVariableString());
				fromNode.add(tempDoc.getRootElement());
			} catch (DocumentException e) {
				e.printStackTrace();
			}
		}

		/*
		 * if (variableSenderMessage == null) { final Person from =
		 * normalMessage.getFrom(); String name;
		 * 
		 * if (from.getMiddle() != null && !from.getMiddle().equals("")) { name
		 * = from.getFirst() + " " + from.getMiddle() + " " + from.getLast(); }
		 * else { name = from.getFirst() + " " + from.getLast(); }
		 * 
		 * return "\t\t<personal>" + name + "</personal>\n" + "\t\t<addr>" +
		 * from.getEmail() + "</addr>\n"; } else { return
		 * variableSenderMessage.getFrom().toVariableString(); }
		 */
	}

	/**
	 * @deprecated This function should never me used/needed here only because
	 *             there used to be the need for the index of the sender
	 * @return -1
	 */
	public int getIntFrom() {
		try {
			throw new Exception();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return -1;
	}

	/**
	 * This adds a new messageClass to the message
	 * 
	 * @param messageClass
	 *            the messageclass to add to this message
	 */
	public void addMessageClass(final FakeMessageClass messageClass) {
		getMessage().addMessageClass(messageClass);
	}

	/**
	 * 
	 * @param messageClassSet
	 *            the set of messages that should be associated with this
	 *            message
	 */
	public void setMessageClass(final Set messageClassSet) {
		getMessage().setMessageClass(messageClassSet);
	}

	/**
	 * @deprecated Integers should not be used to relate message classes to
	 *             messages
	 * @param messageClass
	 *            the new messageClass to be set for this message
	 */
	public void setMessageClass(final int messageClass) {
		// getMessage().addMessageClass(new FakeMessageClass(messageClass));
	}

	/**
	 * @param period
	 *            the period of the message
	 */
	public void setPeriod(final int period) {
		getMessage().setPeriod(new Integer(period));
	}

	/**
	 * @param group
	 *            the group that this message belongs to
	 */
	public void setGroup(String group) {
		getMessage().setMessageGroup(group);
	}

	/**
	 * @param thread
	 *            the thread that this message belongs to
	 */
	public void setThread(int thread) {
		getMessage().setThread(new Integer(thread));
	}

	/**
	 * @param noise
	 *            the noise value for this message
	 */
	public void setNoise(boolean noise) {
		getMessage().setNoise(Boolean.toString(noise));
	}

	/**
	 * @param response
	 *            true if this email is a response to another, false otherwise
	 */
	public void setResponse(boolean response) {
		getMessage().setResponse(Boolean.toString(response));
	}

	/**
	 * @param id
	 *            the id of this message in the database
	 */
	public void setId(int id) {
		getMessage().setId(new Integer(id));
	}

	/**
	 * 
	 * @param messageId
	 *            the id of this message, gathered by who wrote them, or other
	 *            factors
	 */
	public void setMessageId(Integer messageId) {
		getMessage().setMessageId(messageId);
	}

	/**
	 * 
	 * @param messageId
	 *            the id of this message, gathered by who wrote them, or other
	 *            factors
	 */
	public void setMessageId(int messageId) {
		getMessage().setMessageId(new Integer(messageId));
	}

	/**
	 * Sets the replyto field to point to the messageid of the message that this
	 * message is a reply to
	 * 
	 * @param replyto
	 *            the messageId that this message is a reply to
	 */
	public void setReplyTo(int replyto) {
		getMessage().setReplyTo(new Integer(replyto));
	}

	/**
	 * Sets the from field of this message to the person parameter. If this
	 * message isn't a NormalFakeMessage, then this throws a class cast
	 * exception
	 * 
	 * @param from
	 *            the person who sent this message
	 * @throws ClassCastException
	 *             if this message cannot have a person sender
	 */
	public void setFrom(Person from) {
		if (normalMessage != null) {
			normalMessage.setFrom(from);
		} else {
			throw new ClassCastException();
		}
	}

	/**
	 * Sets the from field of this message to the variable parameter. If this
	 * message isn't a VariableSenderFakeMessage, then this throws a class cast
	 * exception
	 * 
	 * @param from
	 *            the variable in which there is the email address of the sender
	 * @throws ClassCastException
	 *             if this message cannot have a variable sender
	 */
	public void setFrom(edu.cmu.radar.base.corpus.Variable from) {
		if (variableSenderMessage != null) {
			variableSenderMessage.setFrom(from);
		} else {
			throw new ClassCastException();
		}
	}

	/**
	 * @param to
	 *            the list of recipients that should recieve this message
	 */
	public void setTo(ArrayList to) {
		getMessage().getRecipients().addAll(to);
	}

	/**
	 * @param cc
	 *            the list of recipients that should recieve a carbon copy of
	 *            this message
	 */
	public void setCC(ArrayList cc) {
		getMessage().getRecipients().addAll(cc);
	}

	/**
	 * @param bcc
	 *            the list of recipients that should recieve a blind carbon copy
	 *            of this message
	 */
	public void setBCC(ArrayList bcc) {
		getMessage().getRecipients().addAll(bcc);
	}

	/**
	 * @param label
	 *            the new label that should be added to this message
	 */
	public void addLabel(String label) {
		getMessage().addLabel(new Label(label));
	}

	public Set getLabels() {
		return getMessage().getLabels();
	}

	/**
	 * @return the time at which this message was/should be sent
	 */
	public String getTime() {
		return getMessage().getTime();
	}

	/**
	 * @return the period in which this assignment was sent
	 */
	public int getPeriod() {
		return getMessage().getPeriod().intValue();
	}

	/**
	 * @return the outcome of this message
	 */
	public String getOutcome() {
		return getMessage().getOutcome();
	}

	/**
	 * @return the thread that this message is a part of
	 */
	public int getThread() {
		return getMessage().getThread().intValue();
	}

	/**
	 * @return the value of the noise field
	 */
	public boolean getNoise() {
		return new Boolean(getMessage().getNoise()).booleanValue();
	}

	/**
	 * @return true if this message is a response to another message, false
	 *         otherwise
	 */
	public boolean getResponse() {
		return new Boolean(getMessage().getNoise()).booleanValue();
	}

	/**
	 * @return true if this message is part of the backstory, false otherwise
	 */
	public boolean getBackstory() {
		return getMessage().getBackstory();
	}

	/**
	 * @return the messageid that this message is a reply to
	 */
	public int getReplyTo() {
		return getMessage().getReplyTo().intValue();
	}

	/**
	 * @return the list of all recipients
	 */
	public Set getRecipients() {
		return getMessage().getRecipients();
	}

	/**
	 * @return the list of "to" recipients
	 */
	public ArrayList getTo() {
		return getMessage().getTo();
	}

	/**
	 * @return the list of "cc" recipients
	 */
	public ArrayList getCC() {
		return getMessage().getCC();
	}

	/**
	 * @return the list of "bcc" recipients
	 */
	public ArrayList getBCC() {
		return getMessage().getBCC();
	}

	/**
	 * @return the subject of the email
	 */
	public String getSubject() {
		return getMessage().getSubject();
	}

	/**
	 * @return the body of the email
	 */
	public String getBody() {
		return getMessage().getBody();
	}

	/**
	 * sets the time of this message
	 * 
	 * @param time
	 *            the time that this message was/should be sent at
	 */
	public void setTime(String time) {
		getMessage().setTime(time);
	}

	/**
	 * sets the outcome of this message
	 * 
	 * @param outcome
	 *            the outcome of this message
	 */
	public void setOutcome(String outcome) {
		getMessage().setOutcome(outcome);
	}

	/**
	 * Sets the subject of this message
	 * 
	 * @param subject
	 *            the new subject of this message
	 */
	public void setSubject(String subject) {
		getMessage().setSubject(subject);
	}

	/**
	 * sets the body of this message
	 * 
	 * @param body
	 *            the body that this message should have
	 */
	public void setBody(String body) {
		getMessage().setBody(body);
	}

	/**
	 * 
	 * @return the message that is within this wrapper
	 */
	public FakeMessage getMessage() {
		if (normalMessage == null) {
			return variableSenderMessage;
		} else {
			return normalMessage;
		}
	}

	/**
	 * adds node text to the element; if text is null, adds an empty string
	 * */
	private void addXMLText(Element element, String text) {
		if (text == null) {
			text = "";
		}

		element.addText(text);
	}

	/**
	 * This returns this message in an broken xml representation
	 * 
	 * @param session
	 *            the session that any necessary data can be obtained from
	 * @return the broken injector representation
	 */
	public void getBrokenXML(Session session, Element root) {
		Element emailNode = root.addElement("email");

		getXMLBeforeEmails(getMessage(), emailNode);

		Element fromNode = emailNode.addElement("from");
		if (normalMessage == null) {
			getXMLFrom(fromNode);
		} else {
			getXMLFrom(fromNode);
			addXMLText(fromNode, "\t");
		}

		Element recNode = emailNode.addElement("recipients");

		ArrayList toRecipients = getTo();
		ArrayList ccRecipients = getCC();
		ArrayList bccRecipients = getBCC();

		toRecipientXML(session, toRecipients, "to", recNode);
		toRecipientXML(session, ccRecipients, "cc", recNode);
		toRecipientXML(session, bccRecipients, "bcc", recNode);

		addXMLText(emailNode.addElement("subject"), getSubject());
		addXMLText(emailNode.addElement("body"), getBody());

		/*
		 * String asString = getXMLBeforeEmails(this.getMessage());
		 * 
		 * if (normalMessage == null) { asString += ("\t<from>" + getXMLFrom() +
		 * "</from>\n"); } else { asString += ("\t<from>\n" + getXMLFrom() +
		 * "\t</from>\n");
		 * 
		 * }
		 * 
		 * asString += ("\t<recipients>\n");
		 * 
		 * final ArrayList toRecipients = getTo(); final ArrayList ccRecipients
		 * = getCC(); final ArrayList bccRecipients = getBCC();
		 * 
		 * asString += toRecipientXML(session, toRecipients, "to"); if
		 * (ccRecipients != null) { asString += toRecipientXML(session,
		 * ccRecipients, "cc"); } if (bccRecipients != null) { asString +=
		 * toRecipientXML(session, bccRecipients, "bcc"); } asString +=
		 * ("\t</recipients>\n"); asString += ("\t<subject>" +
		 * getSubject().replaceAll("&", "&amp;") + "</subject>\n"); asString +=
		 * ("\t<body>" + getBody().replaceAll("&", "&amp;") + "\n\t</body>\n");
		 * 
		 * asString += ("</email>\n"); asString += ("\n");
		 * 
		 * return asString;
		 */
	}

	/**
	 * @param session
	 *            An open session to possibly querry for variables
	 * @param recipients
	 *            The list of recipients to transform into xml
	 * @param type
	 *            the type of recipent that's in recipients (one of to, cc, or
	 *            bcc)
	 * @return The string representation of the recipients in the arrayList
	 */
	private void toRecipientXML(Session session, ArrayList recipients,
			String type, Element recipientsNode) {
		for (int i = 0; i < recipients.size(); i++) {
			Recipient toRecipient = (Recipient) recipients.get(i);

			if (Boolean.valueOf(toRecipient.getIsVariable())) {
				Element typeNode = recipientsNode.addElement(type);
				addXMLText(typeNode, ((Variable) session.load(Variable.class,
						toRecipient.getRecipientId())).toVariableString());
			} else {
				Person person = ((Person) session.load(Person.class,
						toRecipient.getRecipientId()));
				String fullname = person.getFirst() + " ";
				if (person.getMiddle() == null) {
					fullname += person.getLast();
				} else {
					fullname += person.getMiddle() + " " + person.getLast();
				}

				Element typeNode = recipientsNode.addElement(type);
				Element personNode = typeNode.addElement("personal");
				addXMLText(personNode, fullname);
				addXMLText(typeNode.addElement("addr"), person.getEmail());
			}
		}

		/*
		 * String asString = "";
		 * 
		 * for (int i = 0; i < recipients.size(); i++) { final Recipient
		 * toRecipient = (Recipient)recipients.get(i); if (new
		 * Boolean(toRecipient.getIsVariable()).booleanValue()) { asString +=
		 * ("\t\t<" + type + ">" + ((Variable)session.load(Variable.class,
		 * toRecipient .getRecipientId())).toVariableString() + "</" + type +
		 * ">\n"); } else { final Person person =
		 * ((Person)session.load(Person.class, toRecipient.getRecipientId()));
		 * if (person.getMiddle() == null) { asString += ("\t\t<" + type +
		 * "> \n \t\t\t<personal>" + person.getFirst() + " " + person.getLast()
		 * + "</personal>\n\t\t\t<addr>" + person.getEmail() + "</addr>\n\t\t</"
		 * + type + ">\n"); } else { asString += ("\t\t<" + type +
		 * "> \n \t\t\t<personal>" + person.getFirst() + " " +
		 * person.getMiddle() + " " + person.getLast() +
		 * "</personal>\n\t\t\t<addr>" + person.getEmail() + "</addr>\n\t\t</" +
		 * type + ">\n"); } } } return asString;
		 */
	}

	/**
	 * validates this sting is valid xml
	 * 
	 * @param s
	 *            the string to validate
	 * @return the string with all <, >, and &'s replaced with their codes
	 */
	private String revalidate(final String s) {
		String string;
		string = s.replaceAll("&", "&amp;");
		string = string.replaceAll("<", "&lt;");
		string = string.replaceAll(">", "&gt;");
		return string;
	}

	/**
	 * This is a helper function to get the first chunk of bad xml
	 * 
	 * @param message
	 *            the message to format
	 * @return the xml representation of all fields replyto and above
	 */
	protected void getXMLBeforeEmails(final FakeMessage message,
			Element emailNode) {
		Element timeNode = emailNode.addElement("time");
		addXMLText(timeNode, message.getTime());
		if (message.getBackstory()) {
			timeNode.addAttribute("mode", "backstory");
		}

		addXMLText(emailNode.addElement("period"), String.valueOf(message
				.getPeriod()));
		addXMLText(emailNode.addElement("group"), message.getMessageGroup());
		addXMLText(emailNode.addElement("outcome"), message.getOutcome());
		addXMLText(emailNode.addElement("thread"), String.valueOf(message
				.getThread()));

		getLabelXML(emailNode);

		Element vipNode = emailNode.addElement("VIP");
		if (message instanceof NormalFakeMessage) {
			addXMLText(vipNode, String.valueOf(((NormalFakeMessage) message)
					.getVIP()));
		} else {
			addXMLText(vipNode, "false");
		}

		addXMLText(emailNode.addElement("noise"), message.getNoise());
		addXMLText(emailNode.addElement("response"), message.getResponse());
		addXMLText(emailNode.addElement("messageid"), String.valueOf(message
				.getId()));
		addXMLText(emailNode.addElement("replyto"), String.valueOf(message
				.getReplyTo()));

		/*
		 * String asString = "<email>\n";
		 * 
		 * if (message.getBackstory()) { // add a special parameter in the time
		 * tag if this // message is backstory asString +=
		 * ("\t<time mode=\"backstory\">" + message.getTime() + "</time>\n"); }
		 * else { asString += ("\t<time>" + message.getTime() + "</time>\n"); }
		 * 
		 * asString += ("\t<period>" + message.getPeriod() + "</period>\n");
		 * asString += ("\t<group>" + message.getMessageGroup() + "</group>\n");
		 * asString += ("\t<outcome>" + message.getOutcome() + "</outcome>\n");
		 * asString += ("\t<thread>" + message.getThread() + "</thread>\n");
		 * 
		 * asString = getLabelXML(asString);
		 * 
		 * if (message instanceof NormalFakeMessage) { asString += ("\t<VIP>" +
		 * ((NormalFakeMessage)message).getVIP() + "</VIP>\n"); } else {
		 * asString += ("\t<VIP>" + false + "</VIP>\n"); } asString +=
		 * ("\t<noise>" + message.getNoise() + "</noise>\n"); asString +=
		 * ("\t<response>" + message.getResponse() + "</response>\n"); asString
		 * += ("\t<messageid>" + message.getId() + "</messageid>\n"); asString
		 * += ("\t<replyto>" + message.getReplyTo() + "</replyto>\n"); return
		 * asString;
		 */
	}

	/**
	 * This gets the xml representation of the labels
	 * 
	 * @param asString
	 *            the string to append the labels onto
	 * @return the string with labels in xml form attached
	 */
	private void getLabelXML(Element root) {
		Iterator labelList = getLabels().iterator();

		while (labelList.hasNext()) {
			addXMLText(root.addElement("label"), labelList.next().toString());
			// asString += "\t<label>" + labelList.next() + "</label>\n";
		}
	}

	/**
	 * sets the set of labels for the message
	 * 
	 * @param set
	 *            the set of labels for this email
	 */
	public void setLabels(Set set) {
		getMessage().setLabels(set);
	}

	/**
	 * Sets the recipients of this message
	 * 
	 * @param recipientSet
	 *            a set of recipients to recieve this email
	 */
	public void setRecipients(Set recipientSet) {
		getMessage().setRecipients(recipientSet);
	}

	/**
	 * This function is used to switch from a NormalFakeMessage to a
	 * VariableSenderFakeMessage and vice versa
	 * 
	 * @param from
	 *            The new sender of the email
	 */
	public void switchMessageType(Object from) {
		FakeMessage saveIn = getMessage();

		if (normalMessage == null) {
			normalMessage = new NormalFakeMessage(getMessageClass(),
					getMessageId(), getTime(), new Integer(getPeriod()),
					getMessageGroup(), getLabels(), getOutcome(), new Integer(
							getThread()), Boolean.toString(getNoise()), Boolean
							.toString(getResponse()),
					new Integer(getReplyTo()), getSubject(), getRecipients(),
					getBody(), (Person) from);
			variableSenderMessage = null;
		} else {
			variableSenderMessage = new VariableSenderFakeMessage(
					getMessageClass(), getMessageId(), getTime(), new Integer(
							getPeriod()), getMessageGroup(), getLabels(),
					getOutcome(), new Integer(getThread()), Boolean
							.toString(getNoise()), Boolean
							.toString(getResponse()),
					new Integer(getReplyTo()), getRecipients(), getSubject(),
					getBody(), (edu.cmu.radar.base.corpus.Variable) from);
			normalMessage = null;
		}
	}

	/**
	 * @return the string representation of this email
	 */
	public String toString() {
		return this.getMessage().toString();
	}

	/**
	 * @param object
	 *            the object to test this object against
	 * @return true if this object is equal to the parameter
	 */
	public boolean equals(Object object) {
		boolean result = false;

		if (object instanceof CorpusGUIMessage) {
			result = getMessage().equals(
					((CorpusGUIMessage) object).getMessage());
		}

		return result;
	}
}
