package airspaceGUI.base.corpus;

import java.util.ArrayList;
import java.util.Set;

/**
 * The types messages used to need to implement. It also defines some constants
 * realted to messages.
 * 
 * @author srbennet
 * 
 */
public interface MessageType
{
    /**
     * The substrings that make up the email validation string 
     * this is here to make debugging easier.
     */
    static final String NAME = "(\\w+ ??\\w+)\\s?";
    static final String QUOTEDNAME = "\"?(\\w+ ??\\w+)*\"? ?";
    static final String USERNAME = "(\\w+[\\.\\+]?)+";
    static final String DOMAIN = "(((\\w+)\\.)+(\\w+))";
    static final String ADDRESS = USERNAME + "@" + DOMAIN;
    /**
     * the regex string used to validate an e-mail address
     */
    public static final String EMAILVERIFICATION = "(" + NAME + ")|(" + QUOTEDNAME + ")" 
        + "(<" + ADDRESS + ">)|(" + ADDRESS + ")";

    /**
     * 
     * @return the array of labels the message can have
     */
    public abstract Set getLabels();

    /**
     * 
     * @return a String containing the time of the message
     */
    public abstract String getTime();

    /**
     * 
     * @return the messages' period
     */
    public abstract int getPeriod();

    /**
     * 
     * @return the period of the message
     */
    public abstract String getGroup();

    /**
     * 
     * @return the messages' outcome
     */
    public abstract String getOutcome();

    /**
     * 
     * @return the value of this messages' thread
     */
    public abstract int getThread();

    /**
     * 
     * @return true if this message was written by a VIP, false otherwise
     */
    public abstract boolean getVIP();

    /**
     * 
     * @return true if this message is noise, false otherwise
     */
    public abstract boolean getNoise();

    /**
     * 
     * @return true if this message has a response, false otherwise
     */
    public abstract boolean getResponse();

    /**
     * 
     * @return true if this message is backstory, false otherwise
     */
    public abstract boolean getBackstory();

    /**
     * 
     * @return this message's message id
     */
    public abstract int getId();

    /**
     * 
     * @return the id of the message that replies to this message, 0 if there
     *         are no replies
     */
    public abstract int getReplyTo();

    /**
     * 
     * @return the e-mail address in the from field of this message
     */
    public abstract String getFrom();

    /**
     * 
     * @return the e-mail address in the from field of this message
     */
    public abstract int getIntFrom();

    /**
     * 
     * @return the e-mail address(es) in the to field of this message
     */
    public abstract ArrayList getTo();

    /**
     * 
     * @return the e-mail address(es) in the cc field of this message
     */
    public abstract ArrayList getCC();

    /**
     * 
     * @return the e-mail address(es) in the bcc field of this message
     */
    public abstract ArrayList getBCC();

    /**
     * 
     * @return the subject of this message
     */
    public abstract String getSubject();

    /**
     * 
     * @return the body of this message
     */
    public abstract String getBody();

    /**
     * Sets the messageClass of the message
     * 
     * @param messageClass
     */
    public abstract void setMessageClass(int messageClass);

    /**
     * 
     * @param time -
     *            the time this messages time should be set to
     */
    public abstract void setTime(final String time);

    /**
     * 
     * @param period -
     *            the period this messages period should be set to
     */
    public abstract void setPeriod(final int period);

    /**
     * 
     * @param group -
     *            the group this messages group should be set to
     */
    public abstract void setGroup(final String group);

    /**
     * 
     * @param outcome -
     *            the outcome that this messages outcome should be set to
     */
    public abstract void setOutcome(final String outcome);

    /**
     * 
     * @param thread -
     *            the thread this messages' thread should be set to
     */
    public abstract void setThread(final int thread);

    /**
     * 
     * @param noise -
     *            the noise value this messages' noise field should be set to
     */
    public abstract void setNoise(final boolean noise);

    /**
     * 
     * @param response -
     *            the response value this messages' response field should be set
     *            to
     */
    public abstract void setResponse(final boolean response);


    /**
     * 
     * @param id -
     *            the message id this message should be set to
     */
    public abstract void setId(final int id);

    /**
     * 
     * @param replyto -
     *            the messageID of the message this message is a reply to
     */
    public abstract void setReplyTo(final int replyto);


    /**
     * 
     * @param to -
     *            the e-mail address(es) this message is to
     */
    public abstract void setTo(final ArrayList to);

    /**
     * 
     * @param cc -
     *            the e-mail address(es) this message is cc'ed to
     */
    public abstract void setCC(final ArrayList cc);

    /**
     * 
     * @param bcc -
     *            the e-mail address(es) this message is bcc'ed to
     */
    public abstract void setBCC(final ArrayList bcc);

    /**
     * This adds the given label to the set of labels that are attached to this
     * message
     * 
     * @param label
     *            the label to add
     */
    public abstract void addLabel(String label);

    /**
     * 
     * @param subject -
     *            the subject this messages subject should be set to
     */
    public abstract void setSubject(final String subject);

    /**
     * 
     * @param body -
     *            the body this message should have
     */
    public abstract void setBody(final String body);

    /**
     * returns a string of this message in xml format
     * 
     * @return this message in xml string format
     */
    public abstract String toString();

}