package airspaceGUI.base.corpus;

/* import foo goes here*/
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.StringTokenizer;

/**
 * A class to bundle all of the attributes of a variable together
 * 
 * @author srbennet
 * 
 */
public class Variable implements Comparable
{
    /**
     * The regex strings that compose the verification string below in order to
     * help in the debugging of the whole regex
     */
    private static String varParameter = "var\\s*=\\s*\"(\\w+)\"";

    private static String offsetParameter = "offset\\s*=\\s*\"([\\d-]+[dMmwhys])+\"";

    private static String formatParameter = "fmt\\s*=\\s*\"(%\\w,?\\s*)+\"";

    private static String formatOrOffset = "(" + formatParameter + "|"
            + offsetParameter + ")";

    /**
     * The regex string used to validate variables
     */
    public static final String VARIABLEVERIFICATION = "<echo\\s+"
            + varParameter + "(\\s+" + formatOrOffset + ")?(\\s+"
            + formatOrOffset + ")?\\s*/>";

    // private int id; // the id of this variable

    private String name; // the name of this variable

    private String value; // the value of this variable

    private boolean removeable; // true if this variable can be deleted

    /**
     * Default constructor, initializes the variables to default values
     * 
     */
    public Variable()
    {
        name = new String();
        value = " ";
        removeable = true;
    }

    /**
     * Constructs a variable with name name and value value
     * 
     * @param name
     * @param value
     */
    public Variable(String name, String value)
    {
        this.name = name;
        this.value = value;
    }

    /**
     * 
     * @return the name of this variable
     */
    public String getName() {
        return name;
    }

    /**
     * 
     * @return the value of this variable
     */
    public String getValue() {
        return value;
    }

    /**
     * 
     * @return true if this variable is able to be removed, false otherwise
     */
    public boolean getRemoveable() // return if the variable is able to be
    // removed
    {
        return removeable;
    }

    /**
     * Sets the name of this variable
     * 
     * @param name -
     *            the name this variables' name should be changed to
     */
    public void setName(final String name) {
        this.name = name;
    }

    /**
     * Sets the value of this variable
     * 
     * @param value -
     *            the value this variables' value should be changed to
     */
    public void setValue(final String value) {
        this.value = value;
    }

    /**
     * Sets the removeable property of this variable
     * 
     * @param remove -
     *            true if this variable can be removed, false otherwise
     */
    public void setRemoveable(final boolean remove) {
        removeable = remove;
    }

    /**
     * 
     * @return this variable's xml representation
     */
    public String toXML() // turn the variable into xml
    {
        return "<variable " + name + "=\"" + value + "\"/>\n";
    }

    /**
     * populates the people list from the database
     * 
     * @throws SQLException
     * @throws ClassNotFoundException 
     * @throws InstantiationException
     * @throws IllegalAccessException
     * 
     * 
     */
    public void saveVariable() throws SQLException, ClassNotFoundException,
            InstantiationException, IllegalAccessException {
        Class.forName("org.postgresql.Driver").newInstance();
        final Connection connect = DriverManager
                .getConnection("jdbc:postgresql://localhost/ontology/base?user=postgres&password=postgres");
        final Statement st = connect.createStatement();

        st.executeQuery(toDatabaseInsert());
        st.close();
        connect.close();
        // System.out.println(this.name); System.out.println(this.value);
    }

    /**
     * Insert this variable into a postgres database.
     * 
     * @return the insert statement to send to a postgres database
     */
    public String toDatabaseInsert() // turn this variable into an postgres
    // insert statement
    {
        return "INSERT INTO corpus.variable (name, value) VALUES(/'" + name
                + ", /'" + value + "/' );";
    }

    /**
     * 
     * @return the xml code to insert this variable into a message
     */
    public String insert() // turn the variable into xml to be inserted into an
    // e-mail
    {
        return "<echo var=\"" + name + "\" />";
    }

    /**
     * used to set the variable's "natural ordering", only returns the variable
     * name
     * 
     * @return the name of the calling object
     */
    public String toString() {
        return name;
    }

    /**
     * This function parses out a variable object from a string
     * 
     * @param s -
     *            a string to be parsed into a variable object
     * @return the variable object contained within the parameter this function
     *         was passed
     */
    public static Variable parse(final String s) // returns null if the
    // string is not
    // configured in valid xml
    {
        final StringTokenizer st = new StringTokenizer(s, "=  \"//>");
        final Variable v = new Variable();

        // System.out.println(s);
        if (st.countTokens() > 2)
        {
            st.nextToken(); // throw away the variable part
            v.name = st.nextToken();
            v.value = st.nextToken();

            while (st.hasMoreTokens())
            {
                // in case the variable value has spaces
                v.value += (" " + st.nextToken());
            }
            v.removeable = false; /*
                                     * removeable set to false since it's being
                                     * read from a file and some e-mails may
                                     * rely on it
                                     */
            return v;
        }
        return null;
    }

    /**
     * This function is used to verify that the parameter string is a actually the string representation of a variable
     * @param s
     *            the string to be validated
     * @return true if this string is a valid insertion string for a variable,
     *         false otherwise
     */
    public static boolean isValidInsertedVariable(final String s) {
        return s.matches(VARIABLEVERIFICATION);
    }

    /**
     * function to implement the comparable interface compares this variable to
     * the one passed in the parameter
     * 
     * @param o1
     *            the object to compare this to
     * @return how the other object compares to this one
     * 
     */
    public int compareTo(final Object o1) {
        return this.getName().compareToIgnoreCase(((Variable)o1).getName());
    }

}
