package airspaceGUI.base.corpus.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.JTextComponent;

import org.hibernate.Session;

import airspaceGUI.base.corpus.AddressBook;
import airspaceGUI.base.corpus.CorpusGUIMessage;
import airspaceGUI.expansion.ConstraintFactory;
import airspaceGUI.expansion.FakeComboBox;
import airspaceGUI.model.PeopleModel;
import edu.cmu.radar.base.Person;
import edu.cmu.radar.base.corpus.FakeMessageClass;
import edu.cmu.radar.base.corpus.Label;
import edu.cmu.radar.base.corpus.Recipient;

/**
 * The main part of the window, holds all of the fields and labels This isn't
 * meant to be used on its own, use at your own risk.
 * 
 * @author srbennet
 * 
 */
public class CorpusMainPanel extends JPanel implements FocusListener {
	private static final long serialVersionUID = 1L;

	/**
	 * the field to fill in/show messageClasses
	 */
	protected FakeComboBox messageClassField;

	/**
	 * the list of all possible message classes
	 */
	protected List messageClassList = new ArrayList();

	/**
	 * the date/time that this message was/will be sent
	 */
	protected BackstoryTimePanel datePanel = new BackstoryTimePanel(
			false);

	// Initialize all the text fields needed
	/**
	 * The text fields for user input
	 */
	protected JTextField periodField = new JTextField();

	protected JTextField groupField = new JTextField();

	protected JTextField outcomeField = new JTextField();

	protected JTextField threadField = new JTextField();

	protected JTextField messageIdField = new JTextField();

	protected JTextField replyToField = new JTextField();

	protected JTextField subjectField = new JTextField();

	protected FakeComboBox fromField;

	/**
	 * Supstitute actionListener for the fromListener fakeComboBox
	 */
	private ActionListener fromListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			int i = peopleFromBook.indexOf(e.getActionCommand());

			fromField.setText((String) emailFromBook.get(i));
			isModified = true;
			vipCheck.setSelected(book.getVIP(i));
		}
	};

	protected FakeComboBox toField;

	protected FakeComboBox ccField;

	protected FakeComboBox bccField;

	// Code to get the address book feature working

	protected AddressBook book = new AddressBook();

	/** holds peoples full names from the world */
	private List peopleFromBook;

	/** holds the email addresses of the people in peopleFromBook */
	private List emailFromBook;

	/**
	 * initialize the body as a textarea in order to allow for new lines and
	 * tabbing
	 * 
	 */
	protected JTextArea bodyField = new JTextArea();

	/** boolean to tell if any of the fields have been modified */
	protected boolean isModified = false;

	// implement listeners for the various components

	/**
	 * listener for keys, if it detects any typing, it knows a field has been
	 * modified
	 * 
	 */
	protected KeyListener setModified = new KeyAdapter() {
		public void keyReleased(final KeyEvent e) {
			isModified = true;
		}
	};

	/** detects if popup/combo boxes have been motified */
	private ItemListener hasBeenModified = new ItemListener() {
		public void itemStateChanged(final ItemEvent e) {
			isModified = true;
		}
	};

	/** tells if the state has changed, and therefore needs to be saved */
	private ChangeListener modified = new ChangeListener() {
		public void stateChanged(final ChangeEvent e) {
			isModified = true;
		}
	};

	// initialize the checkboxes
	protected JCheckBox vipCheck = new JCheckBox("VIP");

	protected JCheckBox responseCheck = new JCheckBox("Response");

	protected JCheckBox noiseCheck = new JCheckBox("Noise");

	protected JTextComponent hasFocus = null;

	private Session session;

	/*
	 * a list of all possible labels
	 */
	private List labelsList;

	/*
	 * a list of label checkboxes
	 */
	private List labelCheckBoxList = new ArrayList();

	private JFrame win;

	private ActionListener addMessageClassListener = new ActionListener() {

		public void actionPerformed(final ActionEvent e) {
			String newMessageClass;
			final NewMessageClassWindow nmcw = new NewMessageClassWindow(win,
					true);
			nmcw.setVisible(true);
			newMessageClass = nmcw.getNewMessageClass();
			final FakeMessageClass fmc = new FakeMessageClass(newMessageClass);
			session.save(fmc);
			session.getTransaction().commit();
			session.getTransaction().begin();
			isModified = true;
			messageClassField.addItem(newMessageClass, newMessageClass);
		}

	};

	/**
	 * Constructor, sets up the mainPanel with the correct look and feel. This
	 * also places the check boxes, fields, and labels in the correct places
	 * 
	 * @param win
	 *            the parent window
	 * @param session
	 *            an initialized hibernate session
	 * @param peopleModel
	 *            The model containing all the people in the world
	 */
	public CorpusMainPanel(final JFrame win, Session session,
			PeopleModel peopleModel) {
		super();
		this.session = session;
		this.win = win;
		initMembers(peopleModel);
		setUpPanel();
	}

	/**
	 * Lays out the panel and gets it ready to accept user input and display
	 * data
	 * 
	 * @param peopleModel
	 *            the model to retrieve Person data from
	 */
	private void setUpPanel() {
		// make all the labels for the fields
		JLabel timestampLabel = new JLabel("Timestamp:");
		JLabel periodLabel = new JLabel("Period:");
		JLabel groupLabel = new JLabel("Group:");
		JLabel outcomeLabel = new JLabel("Outcome:");
		JLabel threadLabel = new JLabel("Thread:");
		JLabel messageIdLabel = new JLabel("MessageID:");
		JLabel replyToLabel = new JLabel("ReplyTo:");
		JLabel fromLabel = new JLabel("From:");
		JLabel toLabel = new JLabel("To:");
		JLabel ccLabel = new JLabel("CC:");
		JLabel bccLabel = new JLabel("BCC:");
		JLabel subjectLabel = new JLabel("Subject:");
		JLabel bodyLabel = new JLabel("Body:");

		JButton addButton = new JButton("+");
		Dimension addSize = new Dimension(20, 20);
		// do the size dance
		addButton.setSize(addSize);
		addButton.setPreferredSize(addSize);
		addButton.setMaximumSize(addSize);
		addButton.setMinimumSize(addSize);
		// make sure the '+' actually shows on the button instead of an elipsis
		addButton.setHorizontalTextPosition(JButton.CENTER);
		addButton.setVerticalTextPosition(JButton.BOTTOM);
		addButton.setMargin(new Insets(5, 5, 8, 5));

		/**
		 * the label to label the messageClass
		 */
		final JLabel messageClassLabel = new JLabel("Message Class:");

		// set up the layout
		this.setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		int row = 0; // the variable to keep track of what row we're supposed
		// to be putting things on right now

		// add all the fields and labels to the panel
		ConstraintFactory f = new ConstraintFactory(8, 8, 8, 8, 2);
		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		this.add(messageClassLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		// since this will overlap the add button, set the insets so
		// that the 2nd button can be seen.
		gbc.insets = new Insets(8, 4, 0, 30);
		this.add(messageClassField, gbc);
		gbc = f.getConstraint(3, row, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		gbc.anchor = GridBagConstraints.EAST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(8, 8, 0, 5);
		addButton.addActionListener(addMessageClassListener);
		this.add(addButton, gbc);
		row++;

		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.fill = GridBagConstraints.NONE;
		this.add(timestampLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.weightx = 1.0;
		this.add(datePanel, gbc);
		row++;

		// this code needs to be refactored, big time!
		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		this.add(periodLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(periodField, gbc);
		row++;

		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		this.add(groupLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(groupField, gbc);
		row++;

		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		this.add(outcomeLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(outcomeField, gbc);
		row++;

		/* code for labels goes here */
		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.anchor = GridBagConstraints.NORTHEAST;
		this.add(new JLabel("Labels:"), gbc);

		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(setupLabels(f, row), gbc);
		row += labelCheckBoxList.size() / 3;

		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		this.add(new JLabel("Flags:"), gbc);
		gbc = f.getConstraint(1, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.insets = new Insets(8, 2, 8, 8);
		this.add(vipCheck, gbc);

		gbc = f.getConstraint(2, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.insets = new Insets(8, 35, 8, 24);
		this.add(noiseCheck, gbc);

		gbc = f.getConstraint(3, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(responseCheck, gbc);
		row++;
		gbc.insets = new Insets(0, 0, 4, 4);

		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		this.add(threadLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(threadField, gbc);
		row++;

		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		this.add(messageIdLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(messageIdField, gbc);
		row++;

		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		this.add(replyToLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(replyToField, gbc);
		row++;

		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		this.add(fromLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(fromField, gbc);
		row++;

		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		this.add(toLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(toField, gbc);
		row++;

		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		this.add(ccLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(ccField, gbc);
		row++;

		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		this.add(bccLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(bccField, gbc);
		row++;

		/* hopefully adding the subject field here */
		gbc = f.getConstraint(0, row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		this.add(subjectLabel, gbc);
		gbc = f.getConstraint(1, row, 3, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		this.add(subjectField, gbc);

		// add the body field
		row++;

		gbc = f.getConstraint(0, row, 1, 10.0, ConstraintFactory.LABEL,
				ConstraintFactory.BOTTOMMOST);
		this.add(bodyLabel, gbc);
		gbc = f.getConstraint(1, row, 3, 10.0, ConstraintFactory.FIELD,
				ConstraintFactory.BOTTOMMOST);
		bodyField.setLineWrap(true);
		bodyField.setWrapStyleWord(true);
		bodyField.setRows(6);
		bodyField.setColumns(36);
		bodyField.setTabSize(4);
		JScrollPane bodyPane = new JScrollPane(bodyField);
		bodyPane
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		bodyPane
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		this.add(bodyPane, gbc);
	}

	/**
	 * Initializes the data behind this panel
	 * 
	 * @param labels
	 *            the possible labels
	 * @param pm
	 *            the person data to display on the panel
	 */
	private void initMembers(final PeopleModel pm) {
		// initialize the messageClass
		if (messageClassList == null || messageClassList.isEmpty()) {
			messageClassList = session.createCriteria(FakeMessageClass.class)
					.list();
		}
		// messageClassField.setEditable(false);
		ArrayList stringMessageClass = new ArrayList();
		for (int i = 0; i < messageClassList.size(); i++) {
			stringMessageClass.add(((FakeMessageClass) messageClassList.get(i))
					.getMessageClass());
		}
		messageClassField = new FakeComboBox(stringMessageClass,
				"Add Message Class  \u25bc");

		// setup the address book
		setupAddressBook(pm);

		/* Code to change the title bar when the subject field is changed */
		subjectField.addKeyListener(new KeyAdapter() {
			public void keyReleased(final KeyEvent e) {
				setTitle();
				isModified = true;
			}
		});

		// when the subject field is modified, update the title
		subjectField.addActionListener(new ActionListener() {
			public void actionPerformed(final ActionEvent e) {
				setTitle();
			}
		});

		labelsList = session.createCriteria(Label.class).list();

		for (int i = 0; i < labelsList.size(); i++) {
			labelCheckBoxList.add(new JCheckBox(((Label) labelsList.get(i))
					.getLabel()));
		}

	}

	/**
	 * this sets the title of the window correctly based on the subject of the
	 * message
	 */
	private void setTitle() {
		if (subjectField.getText().compareTo("") != 0) {
			win.setTitle(subjectField.getText());
		} else {
			win.setTitle("Compose a Message");
		}
	}

	/**
	 * Initializes the address book
	 * 
	 * @param session
	 *            the session to get data out of, if we don't have enough
	 */
	private void setupAddressBook(final Session session) {
		/* code to get the address book set up/initialized */
		try {
			book.populateBook(session);
		} catch (Exception e) {
			e.printStackTrace();
		}

		peopleFromBook = book.getNames();
		emailFromBook = book.getEmail();

		fromField = new FakeComboBox(peopleFromBook, emailFromBook,
				"Add E-mail Address \u25bc");
		toField = new FakeComboBox(peopleFromBook, emailFromBook,
				"Add E-mail Address \u25bc");
		ccField = new FakeComboBox(peopleFromBook, emailFromBook,
				"Add E-mail Address \u25bc");
		bccField = new FakeComboBox(peopleFromBook, emailFromBook,
				"Add E-mail Address \u25bc");

		/* end address book code */
	}

	/**
	 * Get the address book set up and initialized
	 * 
	 * @param pm
	 *            the model to set the address book with
	 */
	private void setupAddressBook(final PeopleModel pm) {
		/* code to get the address book set up/initialized */
		try {
			book.populateBook(pm);
		} catch (Exception e) {
			e.printStackTrace();
		}

		peopleFromBook = book.getNames();
		emailFromBook = book.getEmail();

		fromField = new FakeComboBox(peopleFromBook, emailFromBook,
				"Add E-mail Address \u25bc");
		fromField.setSingleMode(true);
		toField = new FakeComboBox(peopleFromBook, emailFromBook,
				"Add E-mail Address \u25bc");
		ccField = new FakeComboBox(peopleFromBook, emailFromBook,
				"Add E-mail Address \u25bc");
		bccField = new FakeComboBox(peopleFromBook, emailFromBook,
				"Add E-mail Address \u25bc");

		/* end address book code */
	}

	/**
	 * @param f
	 *            the constraintFactory to use to layout the labels
	 * @param row
	 *            return a JPanel containing all the JCheckBoxes representing
	 *            all the labels.
	 */
	private JPanel setupLabels(final ConstraintFactory f, int row) {
		JPanel labelCheckPanel = new JPanel();
		/* ConstraintFactory f = new ConstraintFactory(4, 4, 4, 4, 2); */
		GridBagConstraints gbc = new GridBagConstraints();

		/*
		 * uncomment when there are a dynamic number of labels int row = 0;
		 */

		labelCheckPanel.setLayout(new GridBagLayout());
		labelCheckPanel.setLayout(new GridLayout(3,
				labelCheckBoxList.size() / 3));
		// gbc = f.getConstraint(1, row, ConstraintFactory.FIELD,
		// ConstraintFactory.MIDDLE);
		int lastRow = 0;
		for (int i = 0; i < labelCheckBoxList.size(); i++) {
			// labelCheckBoxList.add(new
			// JCheckBox(((Label)labelCheckBoxList.get(i)).getLabel()));
			((JCheckBox) labelCheckBoxList.get(i))
					.addItemListener(hasBeenModified);
			gbc.gridx = i % 3;
			gbc.anchor = GridBagConstraints.NORTHWEST;
			labelCheckPanel.add((JCheckBox) labelCheckBoxList.get(i), gbc);

			if (i % 3 > lastRow) {
				row++;
				gbc = f.getConstraint(1, row, ConstraintFactory.FIELD,
						ConstraintFactory.MIDDLE);
			}
			lastRow = i % 3;
		}

		return labelCheckPanel;
	}

	/**
	 * fills the correct fields in the GUI with data from the message parameter
	 * 
	 * @param message
	 *            - the message to populate the fields with
	 */
	public void fillInText(final CorpusGUIMessage message) {

		try {
			final GridBagConstraints gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 0;

			final Iterator messageClassItr = message.getMessageClass()
					.iterator();
			String messageClasses = "";
			while (messageClassItr.hasNext()) {
				if (!"".equals(messageClasses)) {
					messageClasses += ", ";
				}
				messageClasses += messageClassItr.next();
			}
			messageClassField.setText(messageClasses);
			datePanel.setTime(message);

		} catch (Throwable t) {
			// System.err.println(t);
			t.printStackTrace();
		}

		periodField.setText(String.valueOf(message.getPeriod()));
		groupField.setText(message.getMessageGroup());
		outcomeField.setText(message.getOutcome());
		threadField.setText(String.valueOf(message.getThread()));
		messageIdField.setText(String.valueOf(message.getMessageId()));
		replyToField.setText(String.valueOf(message.getReplyTo()));
		subjectField.setText(message.getSubject());

		fromField.setText(message.getFromEmail());

		vipCheck.setSelected(message.getVIP());

		toField.setText(getEmailAddresses(message.getTo(), session));
		ccField.setText(getEmailAddresses(message.getCC(), session));
		bccField.setText(getEmailAddresses(message.getBCC(), session));
		bodyField.setText(message.getBody());
		bodyField.setCaretPosition(0);

		noiseCheck.setSelected(message.getNoise());
		responseCheck.setSelected(message.getResponse());

		setSelectedLabels(message.getLabels());

		isModified = false;
	}

	/**
	 * This sets the selected labels on the interface based on the labels set in
	 * the parameter
	 * 
	 * @param labels
	 *            the labels to set
	 */
	private void setSelectedLabels(Set labels) {
		Iterator labelsItr = labels.iterator();
		String label;

		while (labelsItr.hasNext()) {
			label = ((Label) labelsItr.next()).getLabel();
			for (int i = 0; i < labelCheckBoxList.size(); i++) {
				if (((JCheckBox) labelCheckBoxList.get(i)).getText().equals(
						label)) {
					((JCheckBox) labelCheckBoxList.get(i)).setSelected(true);
				}
			}
		}
	}

	/**
	 * resets the checkboxes to be all unchecked
	 * 
	 */
	protected void clearSelectedLabels() {
		for (int i = 0; i < labelCheckBoxList.size(); i++) {
			((JCheckBox) labelCheckBoxList.get(i)).setSelected(false);
		}
	}

	/**
	 * Returns a list of email addresses
	 * 
	 * @param recipients
	 *            the list of recipients that are possible
	 * @param session
	 *            the session to get any needed data out of
	 * @return a string containing all email addresses of the recipients in the
	 *         list
	 */
	private String getEmailAddresses(final List recipients,
			final Session session) {
		Recipient recipient;
		String email = "";
		for (int i = 0; i < recipients.size(); i++) {
			recipient = (Recipient) recipients.get(i);

			if (i > 0) {
				email += ", ";
			}

			if (!("true".equals(recipient.getIsVariable()))) {
				email += ((Person) session.get(Person.class, recipient
						.getRecipientId())).getEmail();
			} else {
				email += ((edu.cmu.radar.base.corpus.Variable) session.get(
						edu.cmu.radar.base.corpus.Variable.class, recipient
								.getRecipientId())).toVariableString();
			}
		}
		return email;
	}

	/**
	 * record which component has focus in order to be able to insert variables
	 * into the field currently in focus
	 * 
	 * @param e
	 *            the focus event
	 * 
	 */
	public void focusGained(final FocusEvent e) {
		// System.out.println(e);
		try {
			hasFocus = (JTextComponent) e.getSource();
		} catch (ClassCastException cce) {
			cce.printStackTrace();
		}
	}

	/**
	 * Action performed when this object has lost focus
	 * 
	 * @param e
	 *            the focus event
	 */
	public void focusLost(final FocusEvent e) {
		// System.out.println(e);
	}

	/**
	 * removes all the undoListeners so that when the message is being changed,
	 * the UndoManager doesn't restore the message piecemeal
	 * 
	 * @param undoListener
	 *            The listener to undo events to remove
	 * 
	 */
	public void removeUndoListeners(final UndoableEditListener undoListener) {
		periodField.getDocument().removeUndoableEditListener(undoListener);
		groupField.getDocument().removeUndoableEditListener(undoListener);
		outcomeField.getDocument().removeUndoableEditListener(undoListener);
		threadField.getDocument().removeUndoableEditListener(undoListener);
		messageIdField.getDocument().removeUndoableEditListener(undoListener);
		replyToField.getDocument().removeUndoableEditListener(undoListener);
		bodyField.getDocument().removeUndoableEditListener(undoListener);

		fromField.removeUndoableEditListener(undoListener);
		toField.removeUndoableEditListener(undoListener);
		ccField.removeUndoableEditListener(undoListener);
		bccField.removeUndoableEditListener(undoListener);
		subjectField.getDocument().removeUndoableEditListener(undoListener);
	}

	/**
	 * adds the undoListeners so that when the message is being changed, the
	 * UndoManager will register the edits
	 * 
	 * @param undoListener
	 *            The undoListener to add so that it can get undoable edit
	 *            notifications
	 * 
	 */
	public final void addUndoListeners(final UndoableEditListener undoListener) {
		periodField.getDocument().addUndoableEditListener(undoListener);
		groupField.getDocument().addUndoableEditListener(undoListener);
		outcomeField.getDocument().addUndoableEditListener(undoListener);
		threadField.getDocument().addUndoableEditListener(undoListener);
		messageIdField.getDocument().addUndoableEditListener(undoListener);
		replyToField.getDocument().addUndoableEditListener(undoListener);
		bodyField.getDocument().addUndoableEditListener(undoListener);

		fromField.addUndoableEditListener(undoListener);
		toField.addUndoableEditListener(undoListener);
		ccField.addUndoableEditListener(undoListener);
		bccField.addUndoableEditListener(undoListener);
		subjectField.getDocument().addUndoableEditListener(undoListener);
	}

	/**
	 * adds the modificationListeners so that changes can be recorded, so know
	 * when to save
	 * 
	 * @param textFieldModificationListener
	 *            the modificationListener for textFields
	 * @param comboBoxModificationListener
	 *            the modificationListener for comboBoses
	 * @param checkBoxModificationListener
	 *            the modificationListener for checkBoxes
	 */
	public final void addModificationListeners(
			DocumentListener textFieldModificationListener,
			ItemListener comboBoxModificationListener,
			ChangeListener checkBoxModificationListener) {
		subjectField.addFocusListener(this);
		messageClassField.addDocumentListener(textFieldModificationListener);
		// Code below to register when one of the fields in the GUI has changed
		// So that the user can chose to save when exiting if they haven't done
		// so recently
		periodField.getDocument().addDocumentListener(
				textFieldModificationListener);
		groupField.getDocument().addDocumentListener(
				textFieldModificationListener);
		outcomeField.getDocument().addDocumentListener(
				textFieldModificationListener);
		threadField.getDocument().addDocumentListener(
				textFieldModificationListener);
		messageIdField.getDocument().addDocumentListener(
				textFieldModificationListener);
		replyToField.getDocument().addDocumentListener(
				textFieldModificationListener);
		bodyField.getDocument().addDocumentListener(
				textFieldModificationListener);

		fromField.addDocumentListener(textFieldModificationListener);
		toField.addDocumentListener(textFieldModificationListener);
		ccField.addDocumentListener(textFieldModificationListener);
		bccField.addDocumentListener(textFieldModificationListener);

		datePanel.addChangeListener(modified);
		// Add listeners so that a change in the state of the checkboxes can be
		// determined
		vipCheck.addItemListener(hasBeenModified);
		vipCheck.setEnabled(false);
		responseCheck.addItemListener(hasBeenModified);
		noiseCheck.addItemListener(hasBeenModified);

		for (int i = 0; i < labelCheckBoxList.size(); i++) {
			((JCheckBox) labelCheckBoxList.get(i))
					.addItemListener(hasBeenModified);
		}
	}

	public Set getSelectedLabels() {
		HashSet labels = new HashSet();

		for (int i = 0; i < labelCheckBoxList.size(); i++) {
			if (((JCheckBox) labelCheckBoxList.get(i)).isSelected()) {
				Label currentLabel = (Label) labelsList.get(i);
				labels.add(currentLabel);
			}
		}
		return labels;
	}

}
