package airspaceGUI.base.corpus.gui;

//import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Container;

//import java.io.*;
//import java.util.*;
import javax.swing.*;

/**
 * This class is used so the user can declare which fields need to be kept
 * constant from one message to the next new message. It simplifies message
 * creation when a lot of the fields remain constant.
 * 
 * @author srbennet
 * 
 */
public class Preferences extends JFrame
{

    private static final long serialVersionUID = 1L;

    // a bunch of boolean declarations to remember which fields need to be saved
    private transient boolean keepTime = false;
    
    private transient boolean keepMessageClass = false;

    private transient boolean keepPeriod = false;

    private transient boolean keepGroup = false;

    private transient boolean keepLabels = false;

    private transient boolean keepOutcome = false;

    private transient boolean keepThread = false;

    private transient boolean keepVIP = false;

    private transient boolean keepNoise = false;

    private transient boolean keepResponse = false;

    private transient boolean keepReplyTo = false;

    private transient boolean keepFrom = false;

    private transient boolean keepTo = false;

    private transient boolean keepCC = false;

    private transient boolean keepBCC = false;

    private transient boolean keepSubject = false;

    private transient boolean keepBody = false;

    // declarations of checkboxes to display representing the different fields
    // that could be kept
    private transient final JCheckBox messageClassCheck = new JCheckBox("Keep MessageClass");
    
    private transient final JCheckBox timeCheck = new JCheckBox("Keep Time");

    private transient final JCheckBox periodCheck = new JCheckBox("Keep Period");

    private transient final JCheckBox groupCheck = new JCheckBox("Keep Group");

    private transient final JCheckBox labelsCheck = new JCheckBox("Keep Labels");

    private transient final JCheckBox outcomeCheck = new JCheckBox(
            "Keep Outcome");

    private transient final JCheckBox threadCheck = new JCheckBox("Keep Thread");

    private transient final JCheckBox vipCheck = new JCheckBox("Keep VIP");

    private transient final JCheckBox noiseCheck = new JCheckBox("Keep Noise");

    private transient final JCheckBox responseCheck = new JCheckBox(
            "Keep Response");

    private transient final JCheckBox replyToCheck = new JCheckBox(
            "Keep Reply To");

    private transient final JCheckBox fromCheck = new JCheckBox("Keep From");

    private transient final JCheckBox toCheck = new JCheckBox("Keep To");

    private transient final JCheckBox ccCheck = new JCheckBox("Keep CC");

    private transient final JCheckBox bccCheck = new JCheckBox("Keep BCC");

    private transient final JCheckBox subjectCheck = new JCheckBox(
            "Keep Subject");

    private transient final JCheckBox bodyCheck = new JCheckBox("Keep Body");

    // button to signify the user is finished editing the preferences
    private transient final JButton doneButton = new JButton("Done");

    /**
     * Constructor to generate the preferences window GUI and place elements
     * within it correctly
     */
    public Preferences()
    {
        super();
        // set up swing constants to place the elements in the correct places
        Container panel = this.getContentPane();
        GridBagConstraints gbc = new GridBagConstraints();
        int y = 0;
        int x = 0;

        this.setSize(270, 490);
        this.setResizable(false);

        gbc.fill = GridBagConstraints.VERTICAL;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.insets = new Insets(4, 16, 4, 16); // set so that the text is not
        // on the edge of the window

        panel.setLayout(new GridBagLayout());

        this.setTitle("Choose which fields to keep.");

        gbc.gridx = x;
        gbc.gridy = y++;
        // set the time boolean when the timeCheck checkbox is clicked on
        messageClassCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepMessageClass = (messageClassCheck.getSelectedObjects() != null);
            }
        });
        // adding the timeCheck checkbox to the window
        panel.add(messageClassCheck, gbc);
        
        gbc.gridx = x;
        gbc.gridy = y++;
        // set the time boolean when the timeCheck checkbox is clicked on
        timeCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepTime = (timeCheck.getSelectedObjects() != null);
            }
        });
        // adding the timeCheck checkbox to the window
        panel.add(timeCheck, gbc);

        gbc.gridx = x;
        gbc.gridy = y++;
        // set the time period when the timeCheck checkbox is clicked on
        periodCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepPeriod = (periodCheck.getSelectedObjects() != null);
            }
        });
        // adding the periodCheck checkbox to the window
        panel.add(periodCheck, gbc);

        gbc.gridx = x;
        gbc.gridy = y++;
        // set the group boolean when the timeCheck checkbox is clicked on
        groupCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepGroup = (groupCheck.getSelectedObjects() != null);
            }
        });
        // adding the groupCheck checkbox to the window
        panel.add(groupCheck, gbc);

        gbc.gridx = x;
        gbc.gridy = y++;
        // set the labels boolean when the timeCheck checkbox is clicked on
        labelsCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepLabels = (labelsCheck.getSelectedObjects() != null);
            }
        });
        // adding the labelsCheck checkbox to the window
        panel.add(labelsCheck, gbc);

        gbc.gridx = x;
        gbc.gridy = y++;
        // set the outcome boolean when the timeCheck checkbox is clicked on
        outcomeCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepOutcome = (outcomeCheck.getSelectedObjects() != null);
            }
        });
        // adding the outcomeCheck checkbox to the window
        panel.add(outcomeCheck, gbc);

        gbc.gridx = x;
        gbc.gridy = y++;
        // set the thread boolean when the timeCheck checkbox is clicked on
        threadCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepThread = (threadCheck.getSelectedObjects() != null);
            }
        });
        // adding the threadCheck checkbox to the window
        panel.add(threadCheck, gbc);

        gbc.gridx = x;
        gbc.gridy = y++;
        // set the vip boolean when the timeCheck checkbox is clicked on
        vipCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepVIP = (vipCheck.getSelectedObjects() != null);
            }
        });
        // adding the vipCheck checkbox to the window
        panel.add(vipCheck, gbc);

        gbc.gridx = x;
        gbc.gridy = y++;
        // set the noise boolean when the timeCheck checkbox is clicked on
        noiseCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepNoise = (noiseCheck.getSelectedObjects() != null);
            }
        });
        // adding the noiseCheck checkbox to the window
        panel.add(noiseCheck, gbc);

        gbc.gridx = 0;
        gbc.gridy =y++;
        // set the response boolean when the timeCheck checkbox is clicked on
        responseCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepResponse = (responseCheck.getSelectedObjects() != null);
            }
        });
        // adding the responseCheck checkbox to the window
        panel.add(responseCheck, gbc);

        gbc.gridx = 0;
        gbc.gridy = y++;
        // set the replyTo boolean when the timeCheck checkbox is clicked on
        replyToCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepReplyTo = (replyToCheck.getSelectedObjects() != null);
            }
        });
        // adding the replyToCheck checkbox to the window
        panel.add(replyToCheck, gbc);

        gbc.gridx = 0;
        gbc.gridy = y++;
        // set the from boolean when the timeCheck checkbox is clicked on
        fromCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepFrom = (fromCheck.getSelectedObjects() != null);
            }
        });
        // adding the fromCheck checkbox to the window
        panel.add(fromCheck, gbc);

        gbc.gridx = 0;
        gbc.gridy = y++;
        // set the "to" boolean when the timeCheck checkbox is clicked on
        toCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepTo = (toCheck.getSelectedObjects() != null);
            }
        });
        // adding the toCheck checkbox to the window
        panel.add(toCheck, gbc);

        gbc.gridx = 0;
        gbc.gridy = y++;
        // set the cc boolean when the timeCheck checkbox is clicked on
        ccCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepCC = (ccCheck.getSelectedObjects() != null);
            }
        });
        // adding the ccCheck checkbox to the window
        panel.add(ccCheck, gbc);

        gbc.gridx = 0;
        gbc.gridy = y++;
        // set the bcc boolean when the timeCheck checkbox is clicked on
        bccCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepBCC = (bccCheck.getSelectedObjects() != null);
            }
        });
        // adding the bccCheck checkbox to the window
        panel.add(bccCheck, gbc);

        gbc.gridx = 0;
        gbc.gridy = y++;
        // set the subject boolean when the timeCheck checkbox is clicked on
        subjectCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepSubject = (subjectCheck.getSelectedObjects() != null);
            }
        });
        // adding the subjectCheck checkbox to the window
        panel.add(subjectCheck, gbc);

        gbc.gridx = 0;
        gbc.gridy = y++;
        // set the body boolean when the timeCheck checkbox is clicked on
        bodyCheck.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                keepBody = (bodyCheck.getSelectedObjects() != null);
            }
        });
        // adding the bodyCheck checkbox to the window
        panel.add(bodyCheck, gbc);

        gbc.gridx = 0;
        gbc.insets = new Insets(0, 16, 8, 16);
        gbc.gridy = y++;
        // hide the window when finished editing
        doneButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                /* return the foo to the calling object */
                setVisible(false);
            }
        });
        // adding the doneButton to the window
        panel.add(doneButton, gbc);

        this.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
    }

    /**
     * 
     * @return true if the time field should be kept, false otherwise
     */
    public boolean getKeepTime() {
        return keepTime;
    }

    /**
     * 
     * @return true if the period field should be kept, false otherwise
     */
    public boolean getKeepPeriod() {
        return keepPeriod;
    }

    /**
     * 
     * @return true if the group field should be kept, false otherwise
     */
    public boolean getKeepGroup() {
        return keepGroup;
    }

    /**
     * 
     * @return true if the outcome field should be kept, false otherwise
     */
    public boolean getKeepOutcome() {
        return keepOutcome;
    }

    /**
     * 
     * @return true if the thread field should be kept, false otherwise
     */
    public boolean getKeepThread() {
        return keepThread;
    }

    /**
     * 
     * @return true if the labels field should be kept, false otherwise
     */
    public boolean getKeepLabels() {
        return keepLabels;
    }

    /**
     * 
     * @return true if the vip field should be kept, false otherwise
     */
    public boolean getKeepVIP() {
        return keepVIP;
    }

    /**
     * 
     * @return true if the noise field should be kept, false otherwise
     */
    public boolean getKeepNoise() {
        return keepNoise;
    }

    /**
     * 
     * @return true if the response field should be kept, false otherwise
     */
    public boolean getKeepResponse() {
        return keepResponse;
    }

    /**
     * 
     * @return true if the replyTo field should be kept, false otherwise
     */
    public boolean getKeepReplyTo() {
        return keepReplyTo;
    }

    /**
     * 
     * @return true if the from field should be kept, false otherwise
     */
    public boolean getKeepFrom() {
        return keepFrom;
    }

    /**
     * 
     * @return true if the to field should be kept, false otherwise
     */
    public boolean getKeepTo() {
        return keepTo;
    }

    /**
     * 
     * @return true if the cc field should be kept, false otherwise
     */
    public boolean getKeepCC() {
        return keepCC;
    }

    /**
     * 
     * @return true if the bcc field should be kept, false otherwise
     */
    public boolean getKeepBCC() {
        return keepBCC;
    }

    /**
     * 
     * @return true if the subject field should be kept, false otherwise
     */
    public boolean getKeepSubject() {
        return keepSubject;
    }

    /**
     * 
     * @return true if the body field should be kept, false otherwise
     */
    public boolean getKeepBody() {
        return keepBody;
    }

    /**
     * @return Returns the keepMessageClass.
     */
    public boolean getKeepMessageClass() {
        return keepMessageClass;
    }
}