package airspaceGUI.base.corpus.gui;

import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.JTextComponent;

import airspaceGUI.expansion.ConstraintFactory;
import airspaceGUI.model.VariableModel;

import edu.cmu.radar.base.corpus.Variable;


/**
 * This class creates a dialog box that can be placed right along side the main
 * GUI as a reference and also will allow the automated insertion of xml
 * variable code into a message with the click of a button. This will also allow
 * the values of the variables to be changed and new variables to be created.
 * 
 * @author srbennet
 * 
 */
public final class VariableWindow extends JDialog
{
    /* TODO need to add code to handle offsets/formats in variable references*/


    private static final long serialVersionUID = 1L;

    private transient VariableModel variables; /*
                                                             * a data structure
                                                             * to hold and manage the
                                                             * variables, the
                                                             * JList reads out
                                                             * of this structure
                                                             * to generate its
                                                             * list
                                                             */

    private final transient JList variableList; // list of the variables in the
                                                // GUI window

    private final transient JScrollPane scroller; // scroll pane to contain
                                                    // the variable list

    private final transient JButton editButton = new JButton("Edit"); // button
                                                                        // to
                                                                        // edit
                                                                        // variable

    // values

    private final transient JButton insertButton = new JButton("Insert"); // variable
                                                                            // to
                                                                            // insert
                                                                            // values

    // into the GUI

    private final transient JButton newButton = new JButton("+"); // create
                                                                    // new
                                                                    // variables

    private final transient JButton removeButton = new JButton("-"); /*
                                                                         * deletes
                                                                         * newly
                                                                         * created
                                                                         * variables
                                                                         * This
                                                                         * will
                                                                         * not
                                                                         * allow
                                                                         * the
                                                                         * deletion
                                                                         * of
                                                                         * variables
                                                                         * which
                                                                         * were
                                                                         * read
                                                                         * in in
                                                                         * the
                                                                         * creation
                                                                         * of
                                                                         * the
                                                                         * variable
                                                                         * window,
                                                                         * because
                                                                         * of
                                                                         * the
                                                                         * possibility
                                                                         * that
                                                                         * other
                                                                         * messages
                                                                         * might
                                                                         * be
                                                                         * using
                                                                         * that
                                                                         * variable
                                                                         */

    private final transient JLabel name = new JLabel("Name:"); // labels for
                                                                // the text
                                                                // fields

    private final transient JLabel value = new JLabel("Value:"); // labels
                                                                    // for the
                                                                    // value
                                                                    // field

    private final transient JLabel infoLabel = new JLabel("Inspecting:"); // label
                                                                            // to
                                                                            // describe
                                                                            // what

    // mode the window is in

    private final transient JPanel editPanel = new JPanel(new GridBagLayout()); /*
                                                                                 * side panel that appears after a variable has been selected shows
                                                                                 * the value, and allows editing
                                                                                 */

    private final transient JTextField nameField = new JTextField(); // field
                                                                        // displaying
                                                                        // the
                                                                        // name

    private final transient JTextField valueField = new JTextField(); // field
                                                                        // displaying
                                                                        // the
                                                                        // value

    private transient int lastSelected = -1; // the index of the last value
                                                // in the list selected

    /**
     * Constructor for a variable window to appear to the right of the object
     * that created it.
     * 
     * @param parent
     *            the object that created this window
     * @param vm 
     *            The variable data contained in the database
     */
    public VariableWindow(final CorpusMainPanel parent, VariableModel vm)
    {
        super();
        // set up GUI to accept the components to be placed on it
        Container panel = this.getContentPane();
        GridBagConstraints gbc = new GridBagConstraints();
        ConstraintFactory f = new ConstraintFactory(8, 8, 8, 8, 2);
        panel.setLayout(new GridBagLayout());
        this.setModal(false); /*
                                 * don't make the parent wait for a return from
                                 * this frame
                                 */

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        // get the screen size in order to determine where to put the variable
        // window
        if ((parent.getWidth() + 500) < screenSize.width)
        {
            this.setLocation(parent.getWidth(), 0);
        } else
        {
            this.setLocation(screenSize.width - 500, 0);
        }
        // read in the saved variables
        //readVariables(session);
		variables = vm;

        // put the variables into the list
        variableList = new JList(variables.toArray());
        // ensures that only one variable can be selected atLabel a time
        variableList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        /*
         * add a listener to be activated whan a variable is being clicked on so
         * that the side panel will appear/change appropriately
         */
        (variableList.getSelectionModel())
                .addListSelectionListener(new ListSelectionListener()
                {
                    public void valueChanged(final ListSelectionEvent e) {
                        if (e.getValueIsAdjusting())
                        {
                            if (lastSelected >= 0)
                            {/*
                                 * if a variable has been selected before, then
                                 * save it in case its value has changed
                                 */
                                ((Variable)variables.get(lastSelected))
                                        .setValue(valueField.getText());
                            }
                        } else
                        {
                            if (variableList.getSelectedValue() != null)
                            {
                                {
                                    if (VariableWindow.this.getSize().width > VariableWindow.this
                                            .getPreferredSize().width) 
                                    {
                                        /*
                                         * resize the window
                                         */
                                        VariableWindow.this.setIgnoreRepaint(true);
                                        if (VariableWindow.this.getSize().width 
                                                < (scroller.getPreferredSize().width 
                                                        + editPanel.getPreferredSize().width))
                                        {
                                            VariableWindow.this
                                                    .setSize(
                                                            scroller
                                                            .getSize().width
                                                            + editPanel
                                                            .getPreferredSize().width,
                                                            VariableWindow.this
                                                            .getSize().height);
                                            VariableWindow.this.validate();
                                        }
                                    } else
                                    {
                                        VariableWindow.this.pack();
                                        VariableWindow.this.setSize(scroller
                                                        .getPreferredSize().width + editPanel
                                                        .getPreferredSize().width,
                                                        VariableWindow.this
                                                        .getSize().height);
                                        VariableWindow.this.validate();
                                    }
                                }
                                VariableWindow.this.setIgnoreRepaint(false);
                                // set the fields values
                                nameField.setText(((Variable)variableList
                                        .getSelectedValue()).getName());
                                valueField.setText(((Variable)variableList
                                        .getSelectedValue()).getValue());
                                lastSelected = variableList.getSelectedIndex();
                                /*
                                 * enable/disable buttons according to what task
                                 * is being preformed
                                 */
                                editButton.setEnabled(true);
                                insertButton.setEnabled(true);
                                valueField.setEditable(false);
                                infoLabel.setText("Inspecting:");
                                editPanel.setVisible(true);
                            } else
                            {
                                VariableWindow.this.setIgnoreRepaint(true);
                                editPanel.setVisible(false);
                                VariableWindow.this.setSize(VariableWindow.this
                                        .getSize().width
                                        - editPanel.getSize().width,
                                        VariableWindow.this.getSize().height);
                                VariableWindow.this.setIgnoreRepaint(false);
                            }
                        }
                    }
                });

        // add an action to be performed when the edit button is clicked
        editButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                if (editButton.getText().compareTo("Edit") == 0) // button is
                // in
                // editing
                // mode
                {
                    infoLabel.setText("Editing:"); // make the value field
                    // editable
                    valueField.setEditable(true);
                } else
                // button is in new Variable creation mode
                {
                    // save the variable and update the list
                    final Variable temp = new Variable();
                    infoLabel.setText("Inspecting:");
                    temp.setName(nameField.getText());
                    temp.setValue(valueField.getText());
                    // temp.setRemoveable(true);

                    variables.add(temp);

                    variableList.setListData(variables.toArray());
                    // set the button back to editing mode
                    editButton.setText("Edit");
                    nameField.setEditable(false);
                    valueField.setEditable(false);
                }
            }
        });

        // action to be performed when the insert button is clicked
        insertButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                final Clipboard clipboard = getToolkit().getSystemClipboard();
                // save the variable on the system clipboard
                final StringSelection s = new StringSelection(
                        ((Variable)(variables.toArray())[lastSelected]).toVariableString());
                // get where the variable is suppoesed to be inserted into
                final JTextComponent insertInto = (parent.hasFocus);

                clipboard.setContents(s, s);

                if (insertInto != null)
                { // insert the variable where focus was
                    // last
                    (parent.hasFocus).paste();
                } else
                /*
                 * there is an error and the program doesn't know where to
                 * insert the variable
                 */
                {
                    JOptionPane
                            .showMessageDialog(
                                    null,
                                    "Nothing has focus, therefore nothing can be inserted.\n"
                                            + "Please select a text field to insert into.",
                                    "Error", JOptionPane.ERROR_MESSAGE);
                }
            }

        });

        // create a new variable when the new variable button is clicked
        newButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                // set both fields editable
                VariableWindow.this.setIgnoreRepaint(true);
                valueField.setEditable(true);
                nameField.setEditable(true);

                // blank out the text
                nameField.setText("");
                valueField.setText("");

                /*
                 * set the size in case it hasn't already been and show the
                 * editing panel in case this is the first action within the
                 * variable window
                 */
                VariableWindow.this.setSize(500, 250);
                editPanel.setVisible(true);
                infoLabel.setText("Create New:");
                editButton.setText("Done");
                editButton.setEnabled(true);
                VariableWindow.this.setIgnoreRepaint(false);
            }
        });

        /*
         * remove a variable from the list, if it wasn't in the file when the
         * window started
         */
        removeButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e) {
                final Variable selected = (Variable)variableList
                        .getSelectedValue();

                // if a variable is selected
                if (selected != null)
                {
                    variables.remove(selected);
                    nameField.setText("");
                    valueField.setText("");
                    variableList.setListData(variables.toArray());
                } else
                {
                    // nothing is selected, cannot remove the null variable from
                    // the list
                    JOptionPane
                            .showMessageDialog(
                                    null,
                                    "Nothing is selected, therefore nothing will be removed.",
                                    "Error", JOptionPane.ERROR_MESSAGE);
                }
            }
        });

        /*
         * make the default doing nothing on close so we can define our own
         * function
         */
        this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        this.addWindowListener(new WindowAdapter()
        { // listen for the window closing events
                    public void windowClosing(final WindowEvent we) {
                        exit();
                    }
                });

        // add the list to the scroll pane
        scroller = new JScrollPane(variableList);

        // Code to create the editing part of the window
        gbc.gridwidth = 4;

        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.insets = new Insets(0, 6, 4, 8);
        editPanel.add(infoLabel, gbc); // add the labels to the editing panel

        gbc = f.getConstraint(0, 1, ConstraintFactory.LABEL,
                ConstraintFactory.MIDDLE);
        editPanel.add(name, gbc);

        gbc = f.getConstraint(1, 1, 4, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        nameField.setEditable(false);
        editPanel.add(nameField, gbc); // add the fields to the editing panel

        gbc = f.getConstraint(0, 2, ConstraintFactory.LABEL,
                ConstraintFactory.MIDDLE);
        editPanel.add(value, gbc);

        gbc = f.getConstraint(1, 2, 4, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        valueField.setEditable(false);
        editPanel.add(valueField, gbc);

        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.EAST;

        gbc.gridx = 3;
        gbc.gridy = 3;
        gbc.weightx = 1.0;
        editPanel.add(insertButton, gbc); // add the buttons to the editing
        // panel
        if (parent == null)
        {
            insertButton.setEnabled(false);
        }

        gbc.gridx = 4;
        gbc.gridy = 3;
        gbc.weightx = 0.0;
        editPanel.add(editButton, gbc);
        editButton.setEnabled(false);
        // end of code to create the editing part of the window

        // create the main panel of the window (always visible)
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 0.0;
        gbc.weighty = 1.0;
        gbc.gridwidth = 2;
        gbc.insets = new Insets(12, 12, 4, 12);
        // add the scrollpane containing the list
        panel.add(scroller, gbc);

        // add the edit panel, but don't show it yet
        gbc.gridx = 2;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.gridwidth = 5;
        gbc.insets = new Insets(4, 0, 4, 8);
        panel.add(editPanel, gbc);
        editPanel.setVisible(false);

        // add new and remove buttons
        Dimension smallButton = new Dimension(20,20);
        
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.NONE;
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.weightx = 0.0;
        gbc.insets = new java.awt.Insets(4, 12, 12, 4);
        gbc.anchor = GridBagConstraints.WEST;
        newButton.setPreferredSize(smallButton);
        newButton.setMinimumSize(smallButton);
        newButton.setMaximumSize(smallButton);
        newButton.setHorizontalTextPosition(JButton.CENTER);
        newButton.setVerticalTextPosition(JButton.BOTTOM);
        newButton.setMargin(new Insets(5, 5, 5, 5));
        panel.add(newButton, gbc);

        gbc.insets = new java.awt.Insets(4, 4, 12, 12);
        gbc.weightx = 1.0;
        gbc.gridx = 1;
        removeButton.setPreferredSize(smallButton);
        removeButton.setMinimumSize(smallButton);
        removeButton.setMaximumSize(smallButton);
        removeButton.setVerticalTextPosition(JButton.TOP);
        removeButton.setHorizontalTextPosition(JButton.CENTER);
        removeButton.setMargin(new Insets(5, 5, 5, 5));
        panel.add(removeButton, gbc);

        this.pack();
    }

    /**
     * Our custom exiting function, simply saves the variable file in case any
     * changes have been made, and hides the variable window from view
     */
    public void exit() {
        variables.save();
        setVisible(false);
        // don't actually exit, or else the calling program will also terminate
    }


    /**
     * This reads variables out of the database pointed atLabel by the parameter.
     * 
     * @param session
     *            A session to read out the variables from.
     *
     */
    /*Uncomment if we want to go back to using sessions instead of the models
     * private void readVariables(final Session session) {

        List variableList = session.createCriteria(
                edu.cmu.radar.base.corpus.Variable.class).addOrder(Order.asc("name")).list();

        for (int i = 0; i < variableList.size(); i++)
        {
            variables.add(variableList.get(i));
        }
    }*/

}
