package airspaceGUI.base.event;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.FocusListener;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;

import org.hibernate.Session;

import airspaceGUI.base.SubPanel;
import airspaceGUI.expansion.ConstraintFactory;
import airspaceGUI.expansion.FakeComboBox;
import airspaceGUI.model.MeetingModel;
import edu.cmu.radar.base.cmradar.AbstractMeeting;
import edu.cmu.radar.base.cmradar.Agent;

public class AbstractMeetingPanel extends SubPanel {
	/**
	 * The fields related to an abstractMeeting
	 */
	private JTextField weightField = new JTextField(); // integer

	private JTextField durationField = new JTextField(); // integer

	private JTextField legacy_idField = new JTextField(); // integer

	private JTextField typeField = new JTextField(); // string

	private FakeComboBox initiatorField; // only one!

	private FakeComboBox interestedAgentsField;

	private List agentList;

	/**
	 * Initializes this panel
	 * 
	 * @param session
	 *            the session to retrieve data from
	 * @param eventModel
	 *            the model with event data contained therein
	 */
	public AbstractMeetingPanel(Session session, MeetingModel eventModel) {
		super();
		setDataList(eventModel);
		loadSupports(session);
		setSession(session);
	}

	/**
	 * Loads all the supporting data for the panel
	 * 
	 * @param session
	 *            The sesssion to load from
	 */
	private void loadSupports(final Session session) {
		final ArrayList agentStrings = new ArrayList();

		agentList = new ArrayList(load(session, Agent.class, "agentname"));

		for (int i = 0; i < agentList.size(); i++) {
			agentStrings.add(((Agent) agentList.get(i)).getAgentname());
		}

		initiatorField = new FakeComboBox(agentStrings, "Add Initiator \u25bc");
		interestedAgentsField = new FakeComboBox(agentStrings,
				"Add Agents \u25bc");
		initiatorField.setSingleMode(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeairspaceGUI.base.BasePanel#addModificationListeners(java.awt.event.
	 * KeyListener, java.awt.event.ItemListener,
	 * javax.swing.event.ChangeListener)
	 */
	protected void addModificationListeners(
			DocumentListener textFieldModificationListener,
			ItemListener comboBoxModificationListener,
			ChangeListener checkBoxModificationListener) {
		weightField.getDocument().addDocumentListener(
				textFieldModificationListener);
		durationField.getDocument().addDocumentListener(
				textFieldModificationListener);
		legacy_idField.getDocument().addDocumentListener(
				textFieldModificationListener);
		typeField.getDocument().addDocumentListener(
				textFieldModificationListener);
		if (initiatorField != null) {
			initiatorField.addDocumentListener(textFieldModificationListener);
		}
		if (interestedAgentsField != null) {
			interestedAgentsField
					.addDocumentListener(textFieldModificationListener);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeairspaceGUI.base.BasePanel#addUndoableEditListeners(java.awt.event.
	 * UndoableEditListener)
	 */
	protected void addUndoableEditListeners(UndoableEditListener undoListener) {
		weightField.getDocument().addUndoableEditListener(undoListener);
		durationField.getDocument().addUndoableEditListener(undoListener);
		legacy_idField.getDocument().addUndoableEditListener(undoListener);
		typeField.getDocument().addUndoableEditListener(undoListener);
		if (initiatorField != null) {
			initiatorField.addUndoableEditListener(undoListener);
		}

		if (interestedAgentsField != null) {
			interestedAgentsField.addUndoableEditListener(undoListener);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * airspaceGUI.base.BasePanel#addFocusListeners(java.awt.event.FocusListener
	 * )
	 */
	protected void addFocusListeners(FocusListener fl) {
		weightField.addFocusListener(fl);
		durationField.addFocusListener(fl);
		legacy_idField.addFocusListener(fl);
		typeField.addFocusListener(fl);
		initiatorField.addFocusListener(fl);
		interestedAgentsField.addFocusListener(fl);
	}

	/**
	 * Saves the text from the panel to an object
	 * 
	 * @return the object that was saved
	 */
	public Object saveText() {
		if (getDataList() != null) {
			AbstractMeeting meeting = (AbstractMeeting) getDataList().get(
					getIndexOfDisplayed());
			meeting = saveAbstract(meeting);
			return meeting;
		} else {
			return null;
		}
	}

	/**
	 * Saves the abstractMeeting shown on this panel
	 * 
	 * @param abMeeting
	 *            the abstract meeting to save
	 * @return the saved abstractmeeting
	 */
	public AbstractMeeting saveAbstract(AbstractMeeting abMeeting) {
		// flag to tell if this should return null
		boolean nullFlag = true;
		if (!weightField.getText().equals("")) {
			try {
				abMeeting.setWeight(new Integer(weightField.getText()));
				nullFlag = false;
			} catch (NumberFormatException e) {
				JOptionPane
						.showMessageDialog(
								null,
								"The event cannot be saved, because the weight field is not an integer",
								"Error", JOptionPane.ERROR_MESSAGE);
				return null;
			}
		}

		if (!durationField.getText().equals("")) {
			abMeeting.setDuration(new Integer(durationField.getText())); // integer
			nullFlag = false;
		}

		if (!legacy_idField.getText().equals("")) {
			abMeeting.setLegacy_id(new Integer(legacy_idField.getText())); // integer
			nullFlag = false;
		}
		abMeeting.setType(typeField.getText());

		abMeeting.setInitiator(getAgent(initiatorField.getText()));

		StringTokenizer initiatorTokens = new StringTokenizer(
				interestedAgentsField.getText(), ",");
		String agentName;
		Set agents = abMeeting.getAgentsInterested();
		agents.clear();

		while (initiatorTokens.hasMoreTokens()) {
			Agent agent;
			agentName = initiatorTokens.nextToken().trim();
			if ((agent = getAgent(agentName)) != null) {
				agents.add(agent);
				nullFlag = false;
			}
		}

		if (nullFlag) {
			return null;
		} else {
			return abMeeting;
		}
	}

	/**
	 * Clears the data off of this panel
	 */
	public void createNewInstance() {
		clearPanel();
		getDataList().add(getIndexOfDisplayed(), new AbstractMeeting());
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.SubPanel#clearPanel()
	 */
	public void clearPanel() {
		weightField.setText("");
		durationField.setText(""); // integer
		legacy_idField.setText(""); // integer
		typeField.setText("");
		initiatorField.setText(""); // only one!
		interestedAgentsField.setText("");

		weightField.requestFocusInWindow();
	}

	/**
	 * Loads data if necessary. In practice this is no longer needed for this
	 * panel
	 * 
	 * @param session
	 *            the session to load from
	 * @return true if the load succeeded, false otherwise
	 */
	public boolean load(Session session) {
		// nothing should need to be loaded
		return true;
	}

	/**
	 * Saves the data back to the server
	 * 
	 * @return true if the save succeeded, false otherwise
	 */
	public boolean save() {
		Object meeting = saveText();
		super.persist(meeting);

		return true;
	}

	/**
	 * Layout this panel and return it
	 * 
	 * @return the JPanel that has been layed out
	 */
	public JPanel getPane() {
		JPanel abstractMeetingPanel = new JPanel();
		ConstraintFactory cf = new ConstraintFactory(8, 8, 8, 8, 2);
		GridBagConstraints gbc = new GridBagConstraints();
		int row = 0;
		int col = 0;
		this.setLayout(new GridBagLayout());
		abstractMeetingPanel.setLayout(new GridBagLayout());

		final JLabel weightLabel = new JLabel("Weight: ");
		final JLabel durationLabel = new JLabel("Duration: ");
		final JLabel idLabel = new JLabel("Id: ");
		final JLabel typeLabel = new JLabel("Type: ");
		final JLabel initiatorLabel = new JLabel("Initiator: ");
		final JLabel interestedAgentsLabel = new JLabel("Interested Agents: ");

		/**
		 * The fields related to an abstractMeeting
		 */
		gbc = cf.getConstraint(col++, row, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		gbc.anchor = GridBagConstraints.NORTHEAST;
		abstractMeetingPanel.add(weightLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gbc.weighty = 1;
		abstractMeetingPanel.add(weightField, gbc); // integer

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gbc.weighty = 0;
		abstractMeetingPanel.add(durationLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.NORTHEAST;
		abstractMeetingPanel.add(durationField, gbc); // integer

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gbc.weighty = 0;
		abstractMeetingPanel.add(idLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gbc.weighty = 1.0;
		abstractMeetingPanel.add(legacy_idField, gbc); // integer

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gbc.weighty = 0;
		abstractMeetingPanel.add(typeLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gbc.weighty = 1;
		abstractMeetingPanel.add(typeField, gbc); // string

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.anchor = GridBagConstraints.EAST;
		gbc.weighty = 0;
		abstractMeetingPanel.add(initiatorLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gbc.weighty = 1;
		abstractMeetingPanel.add(initiatorField, gbc); // only one!
		// private Agent initiator;

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.BOTTOMMOST);
		gbc.anchor = GridBagConstraints.EAST;
		gbc.weighty = 0;
		abstractMeetingPanel.add(interestedAgentsLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.BOTTOMMOST);
		gbc.anchor = GridBagConstraints.NORTHEAST;
		gbc.weighty = 1.0;
		abstractMeetingPanel.add(interestedAgentsField, gbc);

		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1.0;
		gbc.weighty = 1.00;
		gbc.anchor = GridBagConstraints.NORTHEAST;
		this.add(abstractMeetingPanel, gbc);

		abstractMeetingPanel.validate();
		this.validate();

		return this;
	}

	/**
	 * Loads data onto the fields on the panel
	 * 
	 * @param session
	 *            the session to load from if needed
	 */
	public void fillInText(Session session) {
		if (getDataList() == null) {
			load(session);
		}
		fillInText((AbstractMeeting) getDataList().get(getIndexOfDisplayed()));
	}

	/**
	 * Fills the panel with the data contained in the object
	 * 
	 * @param object
	 *            the data to display in the GUI
	 */
	public void fillInText(Object object) {
		fillInText((AbstractMeeting) object);
	}

	/**
	 * Fill in the next object from this panel's data list
	 * 
	 */
	public void fillInText() {
		fillInText((AbstractMeeting) getDataList().get(getIndexOfDisplayed()));
	}

	/**
	 * This fills in the fields with the data in the object
	 * 
	 * @param object
	 *            the object whose data we should fill in the panel with
	 */
	private void fillInText(final AbstractMeeting object) {
		if (object.getWeight() != null) {
			weightField.setText(object.getWeight().toString());
		}

		if (object.getDuration() != null) {
			durationField.setText(object.getDuration().toString()); // integer
		}

		if (object.getLegacy_id() != null) {
			legacy_idField.setText(object.getLegacy_id().toString()); // integer
		}
		typeField.setText(object.getType()); // string

		if (object.getInitiator() != null) {
			initiatorField.setText(object.getInitiator().getAgentname()); // only
			// one!
			// private Agent initiator;
		}

		final Iterator itr = object.getAgentsInterested().iterator();

		if (itr.hasNext()) {
			interestedAgentsField.setText(((Agent) itr.next()).getAgentname());
			while (itr.hasNext()) {
				interestedAgentsField.setText(interestedAgentsField.getText()
						+ ", " + ((Agent) itr.next()).getAgentname());
			}
		} else {
			interestedAgentsField.setText("");
		}

		setIndexOfDisplayed(getDataList().indexOf(object));
	}

	/**
	 * Displays the object at index number on the panel
	 * 
	 * @param number
	 *            the index to go to
	 */
	public void gotoNumber(int number) {
		setIndexOfDisplayed(number);
		fillInText((AbstractMeeting) getDataList().get(number));
	}

	/**
	 * Gets the name of this panel if it's being dieplayed in tabs
	 * 
	 * @return the title of this panel
	 */
	public String getTitle() {
		return "Abstract Meeting";
	}

	/**
	 * Returns the Agent who has name name
	 * 
	 * @param name
	 *            the name to search for
	 * @return the Agent who possesses that name
	 */
	private Agent getAgent(final String name) {
		for (int i = 0; i < agentList.size(); i++) {
			if (((Agent) agentList.get(i)).getAgentname().equals(name)) {
				return (Agent) agentList.get(i);
			}
		}
		return null;
	}

	/**
	 * Returns the statusMessage of this panel to inform the user where they are
	 * 
	 * @return the status message of this panel
	 */
	public String getStatusText() {
		return "On abstract meeting " + (getIndexOfDisplayed() + 1) + " of "
				+ getDataList().size() + " abstract meetings";
	}
}
