package airspaceGUI.base.event;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;

import org.hibernate.HibernateException;
import org.hibernate.Session;

import airspaceGUI.base.SubPanel;
import airspaceGUI.expansion.ConstraintFactory;
import airspaceGUI.expansion.FakeComboBox;
import airspaceGUI.model.AgentModel;
import airspaceGUI.model.MeetingModel;
import edu.cmu.radar.base.cmradar.AbstractMeeting;
import edu.cmu.radar.base.cmradar.Agent;
import edu.cmu.radar.base.cmradar.Owner;

public class AgentPanel extends SubPanel {
	/*
	 * String username, String userid, String agentname, String agenttype, Set
	 * abstractMeetingsInitiated, Set abstractMeetingsInterested, Owner owner
	 */
	private JTextField usernameField = new JTextField();
	private JTextField useridField = new JTextField();
	private JTextField agentNameField = new JTextField();
	private JTextField agentTypeField = new JTextField();
	private transient FakeComboBox meetingsInitiatedField; // (Both are
	// abstractMeetings)
	private transient FakeComboBox meetingsInterestedField;
	private transient FakeComboBox ownerField;
	private boolean loaded = false;
	private List ownersList;
	private MeetingModel meetingList;

	/**
	 * Initalizes this panel
	 * 
	 * @param session
	 *            the session to read any data out of
	 * @param am
	 *            the model of agents to display on this panel
	 * @param mm
	 *            the meetings that could be associated to the agents
	 */
	public AgentPanel(Session session, AgentModel am, MeetingModel mm) {
		super();
		setDataList(am);
		setSession(session);
		meetingList = mm;
		load(session, mm);
		addFocusListeners();
	}

	/**
	 * Adds the focusListeners to the fields on the panel.
	 */
	protected void addFocusListeners() {
		usernameField.addFocusListener(this);
		useridField.addFocusListener(this);
		agentNameField.addFocusListener(this);
		agentTypeField.addFocusListener(this);
		meetingsInitiatedField.addFocusListener(this);
		meetingsInterestedField.addFocusListener(this);
		ownerField.addFocusListener(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * airspaceGUI.base.SubPanel#addModificationListeners(java.awt.event.KeyListener
	 * , java.awt.event.ItemListener, javax.swing.event.ChangeListener)
	 */
	protected void addModificationListeners(
			DocumentListener textFieldModificationListener,
			ItemListener comboBoxModificationListener,
			ChangeListener checkBoxModificationListener) {
		usernameField.getDocument().addDocumentListener(
				textFieldModificationListener);
		useridField.getDocument().addDocumentListener(
				textFieldModificationListener);
		agentNameField.getDocument().addDocumentListener(
				textFieldModificationListener);
		agentTypeField.getDocument().addDocumentListener(
				textFieldModificationListener);
		if (meetingsInitiatedField != null) {
			meetingsInitiatedField
					.addDocumentListener(textFieldModificationListener);
		}

		if (meetingsInterestedField != null) {
			meetingsInterestedField
					.addDocumentListener(textFieldModificationListener);
		}

		if (ownerField != null) {
			ownerField.addDocumentListener(textFieldModificationListener);
		}
	}

	protected void addUndoableEditListeners(
			final UndoableEditListener undoListener) {
		meetingsInitiatedField.addUndoableEditListener(undoListener);
		meetingsInterestedField.addUndoableEditListener(undoListener);
		ownerField.addUndoableEditListener(undoListener);
	}

	/*
	 * protected void setStatus(int numberOn) {
	 * getStatusLabel().setText("On Agent " + numberOn + " of " +
	 * getDataList().size() + " Agents"); }
	 */

	public Object saveText() {
		boolean empty = true;
		final Agent currentAgent = (Agent) getDataList().get(
				this.getIndexOfDisplayed());

		empty = empty && "".equals(usernameField.getText());
		empty = empty && "".equals(useridField.getText());
		empty = empty && "".equals(agentNameField.getText());
		empty = empty && "".equals(agentTypeField.getText());
		empty = empty && "".equals(ownerField.getText());

		currentAgent.setUsername(usernameField.getText());
		currentAgent.setUserid(useridField.getText());
		currentAgent.setAgentname(agentNameField.getText());
		currentAgent.setAgenttype(agentTypeField.getText());
		currentAgent.setOwner(getOwnerByName(ownerField.getText().replaceAll(
				"Owner whose agent is ", "")));

		if (meetingsInitiatedField.getText().length() > 0) {
			/*
			 * String[] meetingsInitTokens =
			 * meetingsInitiatedField.getText().split(", "); //HashSet
			 * initiatedMeetingsSet = new HashSet(); for(int i = 0; i <
			 * meetingsInitTokens.length; i++) { String token =
			 * meetingsInitTokens[i];
			 * currentAgent.addAbstractMeetingInitiated(meetingList
			 * .findMeeting(token)); }
			 */
			final Object[] selected = meetingsInitiatedField
					.getSelectedObjects();

			currentAgent.setAbstractMeetingsInitiated(arrayToSet(selected));
			empty = false;
		}

		if (meetingsInterestedField.getText().length() > 0) {
			/*
			 * String[] meetingsInterTokens =
			 * meetingsInterestedField.getText().split("(, )"); HashSet
			 * interestedMeetingsSet = new HashSet(); for( int i = 0; i <
			 * meetingsInterTokens.length; i++) { String token =
			 * meetingsInterTokens[i];
			 * currentAgent.addAbstractMeetingInterested(
			 * meetingList.findMeeting(token)); }
			 */
			final Object[] selected = meetingsInterestedField
					.getSelectedObjects();

			currentAgent.setAbstractMeetingsInterested(arrayToSet(selected));
			empty = false;
		}

		if (empty) {
			return null;
		}

		return currentAgent;
	}

	/**
	 * @param selected
	 *            the objects to put into the Set
	 * @return The objects in the array now in a set
	 */
	private Set arrayToSet(final Object[] selected) {
		final HashSet selectedSet = new HashSet();

		for (int i = 0; i < selected.length; i++) {
			selectedSet.add(selected[i]);
		}
		return selectedSet;
	}

	private Owner getOwnerByName(final String ownerAgent) {
		Owner owner = null;
		if (ownersList != null) {
			for (int i = 0; i < ownersList.size(); i++) {
				if (((Owner) ownersList.get(i)).getAgent().getAgentname()
						.equals(ownerAgent)) {
					owner = (Owner) ownersList.get(i);
				}
			}
		}
		return owner;
	}

	public void createNewInstance() {
		clearPanel();
		getDataList().add(getIndexOfDisplayed(), new Agent());
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.SubPanel#clearPanel()
	 */
	public void clearPanel() {
		usernameField.setText("");
		useridField.setText("");
		agentNameField.setText("");
		agentTypeField.setText("");
		meetingsInitiatedField.setText(""); // (Both are abstractMeetings)
		meetingsInterestedField.setText("");
		ownerField.setText("");
		
		usernameField.requestFocusInWindow();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#load(org.hibernate.Session)
	 */
	public boolean load(final Session session) {
		return load(session, (MeetingModel) null);
	}

	/**
	 * Loads all the data to display on this panel
	 * 
	 * @param session
	 *            the session to load the data from
	 * @param mm
	 *            the meetings that the agents haev scheduled
	 * @return true if the load succeeded, false otherwise
	 */
	public final boolean load(final Session session, final MeetingModel mm) {

		if (!loaded) {
			ArrayList tempList;
			if (mm == null) {
				// System.out.println("meeting");
				tempList = new ArrayList(load(session, AbstractMeeting.class,
						"type"));
			} else {
				tempList = mm;
			}
			/*
			 * final AbstractMeeting temp; final ArrayList stringMeetings = new
			 * ArrayList(); StringBuffer meetingString;
			 */
			/*
			 * This is used only if the abstract/concrete meetings don't have a
			 * toString function to generate a more user-readable identifying
			 * string for(int i = 0; i < tempList.size(); i++) { //can I figure
			 * out a better way to do this? temp =
			 * ((AbstractMeeting)tempList.get(i)); meetingString = new
			 * StringBuffer("Proposed meeting between "); if
			 * (temp.getInitiator() != null) {
			 * meetingString.append(temp.getInitiator().getAgentname()); }
			 * 
			 * 
			 * final Iterator interestedIter =
			 * temp.getAgentsInterested().iterator(); // final Object[]
			 * interestedIter = temp.getAgentsInterested().toArray();
			 * //System.out.println(interestedIter); //for(int j = 0; j <
			 * interestedIter.length; j++) while(interestedIter.hasNext()) {
			 * //meetingString += ", and " +
			 * ((Agent)interestedIter[j]).getAgentname();
			 * meetingString.append(", and " +
			 * ((Agent)interestedIter.next()).getAgentname()); }
			 * stringMeetings.add(meetingString.toString()); }
			 */

			meetingsInitiatedField = new FakeComboBox(tempList,
					"Add meeting \u25bc");
			meetingsInterestedField = new FakeComboBox(tempList,
					"Add meeting \u25bc");

			if (ownersList == null) {
				tempList = new ArrayList(load(session, Owner.class));
				ownersList = new ArrayList(tempList);
				final ArrayList stringOwners = new ArrayList();
				final Iterator ownersItr = tempList.iterator();
				Owner tempOwner;
				while (ownersItr.hasNext()) {
					tempOwner = ((Owner) ownersItr.next());
					// System.out.println(tempOwner);
					stringOwners.add("Owner whose agent is "
							+ tempOwner.getAgent().getAgentname());
				}
				// if the ownersList is not null, then this field should already
				// be initialized
				ownerField = new FakeComboBox(stringOwners, "Add owner \u25bc");
			}

			if (getDataList() == null) {
				setDataList(load(session, Agent.class, "agentname"));
			}

			loaded = (getDataList() != null);
			return (getDataList() != null);
		} else {
			return loaded;
		}
	}

	public boolean save() {
		try {
			getSession().flush();
			return true;
		} catch (Exception e) {
			return false;
		}
	}

	public JPanel getPane() {
		final JLabel usernameLabel = new JLabel("Username:");
		final JLabel useridLabel = new JLabel("User Id:");
		final JLabel agentNameLabel = new JLabel("Name:");
		final JLabel agentTypeLabel = new JLabel("Type:");
		final JLabel meetingsInitiatedLabel = new JLabel("Meetings Initiated:");
		final JLabel meetingsInterestedLabel = new JLabel(
				"Meetings Interested:");
		final JLabel ownerLabel = new JLabel("Owner:");

		final ConstraintFactory cf = new ConstraintFactory(8, 8, 8, 8, 2);
		GridBagConstraints gbc = new GridBagConstraints();
		this.setLayout(new GridBagLayout());
		int row = 0, col = 0;

		gbc = cf.getConstraint(col++, row, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		add(usernameLabel, gbc);

		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		add(usernameField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		add(useridLabel, gbc);

		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		add(useridField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		add(agentNameLabel, gbc);

		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		add(agentNameField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		add(agentTypeLabel, gbc);

		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		add(agentTypeField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.anchor = GridBagConstraints.NORTHWEST;
		add(meetingsInitiatedLabel, gbc);

		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weighty = 1.0;
		add(meetingsInitiatedField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.fill = GridBagConstraints.NONE;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.weighty = 0.0;
		add(meetingsInterestedLabel, gbc);

		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weighty = 1.0;
		add(meetingsInterestedField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.BOTTOMMOST);
		gbc.fill = GridBagConstraints.NONE;
		gbc.weighty = 0.0;
		add(ownerLabel, gbc);

		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.BOTTOMMOST);
		add(ownerField, gbc);

		this.validate();

		return this;
	}

	public void fillInText(final Session session) {
		if (getDataList() == null) {
			load(session);
		}
		fillInText((Agent) getDataList().get(getIndexOfDisplayed()));
	}

	public void fillInText(final Object object) {
		fillInText((Agent) object);
	}

	public void fillInText() {
		fillInText((Agent) getDataList().get(getIndexOfDisplayed()));
	}

	public void gotoNumber(final int number) {
		setIndexOfDisplayed(number);
		fillInText((Agent) getDataList().get(number));
	}

	private void fillInText(final Agent object) {

		usernameField.setText(object.getUsername());
		useridField.setText(object.getUserid());
		agentNameField.setText(object.getAgentname());
		agentTypeField.setText(object.getAgenttype());

		final Iterator meetingsInitiated = object
				.getAbstractMeetingsInitiated().iterator();
		StringBuffer text = new StringBuffer("");
		while (meetingsInitiated.hasNext()) {
			if (text.length() > 0) {
				text.append(", ");
			}
			text.append(meetingsInitiated.next());
		}
		meetingsInitiatedField.setText(text.toString());

		try {
			final Iterator meetingsInterested = object
					.getAbstractMeetingsInterested().iterator();
			text = new StringBuffer("");
			while (meetingsInitiated.hasNext()) {
				text.append(meetingsInterested.next());
			}
			meetingsInterestedField.setText(text.toString());
		} catch (HibernateException e) {
			e.printStackTrace();
		}

		if (object.getOwner() != null) {
			ownerField.setText("Owner whose agent is "
					+ object.getOwner().getAgent().getAgentname());
		}
		setIndexOfDisplayed(getDataList().indexOf(object));
	}

	public String getTitle() {
		return "Edit Agent";
	}

	public String getStatusText() {
		return "On agent " + (getIndexOfDisplayed() + 1) + " of "
				+ getDataList().size() + " agents";
	}
}
