/**
 * 
 */
package airspaceGUI.base.person;

import java.awt.Color;
import java.awt.Component;
import java.awt.HeadlessException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTabbedPane;

import airspaceGUI.expansion.FindWindow;

import edu.cmu.radar.base.Person;

/**
 * This class is used to be able to go to a person with a specific name instantly.
 * @author srbennet
 *
 */
public class FindPerson extends FindWindow
{
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    
    private transient ArrayList personList = new ArrayList();

    private static final String NAME = "Name";
    
    private static final String EMAIL = "Email";
    
    private static Object[] keywordList = {NAME, EMAIL};


    private transient PeoplePanel panel;

    /**
     * @param people the list of people to search through
     * @param panel the panel this is attached to.
     * @throws HeadlessException
     */
    public FindPerson(final PeoplePanel panel, List people) throws HeadlessException
    {
        super(keywordList);
        initPanel(panel, people);
    }

    public FindPerson(PeoplePanel panel, List people, JTabbedPane tab)
    {
        super(keywordList, tab);
        initPanel(panel, people);
    }
    
    private void initPanel(final PeoplePanel panel, final List people)
    {
        this.panel = panel;
        parent = panel;
        for(int i = 0; i < people.size(); i++)
        {
            personList.add(people.get(i));
        }
        //setSize(new Dimension(220, 85));
    }

    /**
     * Searches the list of possibilities for the list of people the user could
     * wish to see
     * 
     * @param name
     *            the name the user typed into the field
     * @return All the objects that fit the criteria the user entered
     */
    public ArrayList search(final String name) {
        if (name != null)
        {
            final ArrayList results = new ArrayList();
            String testingFirstName, testingLastName, testingFirstAlias = "", testingLastAlias = "";
        
            for(int i = 0; i < personList.size(); i++)
            {
                //System.out.println("Searching");
                testingFirstName = ((Person)personList.get(i)).getFirst() 
                    + " " + ((Person)personList.get(i)).getLast();
                testingLastName = ((Person)personList.get(i)).getLast() 
                    + " " + ((Person)personList.get(i)).getFirst();
                if (((Person)personList.get(i)).getAkafirst() != null 
                        && ((Person)personList.get(i)).getAkafirst().length() > 0)
                {
                    testingFirstAlias = ((Person)personList.get(i)).getAkafirst() 
                            + " " + ((Person)personList.get(i)).getLast();
                    testingLastAlias = ((Person)personList.get(i)).getLast() 
                        + " " + ((Person)personList.get(i)).getAkafirst();
                }
                if (((testingFirstName.length() >= name.length() && name.compareToIgnoreCase(testingFirstName.substring(0, name.length())) == 0 )
                        || (testingFirstName.length() >= name.length() 
                                && name.compareToIgnoreCase(testingLastName.substring(0, name.length())) == 0)
                        || (testingFirstAlias != null && !testingFirstAlias.equals("")
                                && name.compareToIgnoreCase(testingFirstAlias.substring(0, name.length() - 1)) == 0)
                        || (testingLastAlias != null && !testingLastAlias.equals("")
                                && name.compareToIgnoreCase(testingLastAlias.substring(0, name.length() - 1)) == 0)))
                {
                    results.add(personList.get(i));
                }
            }
        
            return results;
        } else
        {
            return null;
        }
    }

    /**
     * Updates the main panel with the person the user wishes to view/edit
     * 
     * @param wanted
     *            the object the user wishes to view/edit
     */
    public void updatePanel(final Object wanted) {
        try{
            final Person selected = (Person)wanted;
            panel.fillInText(selected);
            tabs.setSelectedComponent(panel);
            return;
        } catch (ClassCastException e)
        {
            return;
        }
    }

    protected ArrayList search(final Map searchTerms) {
        ArrayList results = search((String)searchTerms.get(NAME));
        
        if (results == null)
        {
            results = new ArrayList();
        }
        
        if (searchTerms.get(EMAIL) != null)
        {
            String email = ((String)searchTerms.get(EMAIL)).toLowerCase();
            boolean result = false;
            Person workingOn;
            
            for(int i = 0; i < personList.size(); i++)
            {
                workingOn = (Person)personList.get(i);
                if (workingOn.getEmail() != null && workingOn.getEmail().toLowerCase().contains(email))
                {
                    result = true;
                }
                
                if(result)
                {
                    results.add(personList.get(i));
                    result = false;
                }
            }
            
        }
        
        return results;
    }

    public Component getListCellRendererComponent(final JList list, final Object person, 
            final int arg2, final boolean isSelected, final boolean arg4) {
        final JLabel label = new JLabel();
        
        label.setText(person.toString());
        
        if (isSelected) {
            label.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY));
            label.setOpaque(true);
            label.setBackground(list.getSelectionBackground());
            label.setForeground(list.getSelectionForeground());
            FindPerson.this.validate();
       }
        else {
            label.setBorder(BorderFactory.createLineBorder(list.getBackground()));
            label.setOpaque(false);
            label.setBackground(list.getBackground());
            label.setForeground(list.getForeground());
       }
        label.setEnabled(list.isEnabled());
        label.setFont(list.getFont());
        
        return label;
    }

    public void updateSearchList(List search) {
       personList = (ArrayList) search;
    }

}
