/**
 * 
 */
package airspaceGUI.base.person;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.FocusListener;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;

import org.hibernate.Session;

import airspaceGUI.base.BasePanel;
import airspaceGUI.expansion.ConstraintFactory;
import airspaceGUI.expansion.FindWindow;
import airspaceGUI.model.PeopleModel;
import edu.cmu.radar.base.Person;

/**
 * The panel for creating, editing, and deleting people from the fake world.
 * 
 * @author srbennet
 * 
 */
public class PeoplePanel extends BasePanel implements FocusListener {

	/**
     * 
     */
	private static final long serialVersionUID = 1L;

	/**
	 * the regex string used to validate an e-mail address This validates the
	 * email address only, and not other associated parts that may be in an
	 * email that is sent (it doesn't account for preceding names)
	 */

	public static final String EMAILVERIFICATION = " *(\\w+[.//+]?)*@(((\\w+)\\.)+(\\w+))";

	/**
	 * the regex string used to validate a phone number
	 */
	public static final String PHONEVERIFICATION = " *\\(?\\d{3}\\)?-? *\\d{3} *-?\\d{4} *";

	private JTextField firstNameField = new JTextField();

	private JTextField middleNameField = new JTextField();

	private JTextField lastNameField = new JTextField();

	// not sure what to do with eids
	// eid TEXT, -- FirstLast (identifier for STP and other things)
	private JTextField officePhoneField = new JTextField();// TEXT
																			// ,
	// --

	// (xxx)
	// xxx-xxxx
	// US format

	private JTextField homePhoneField = new JTextField();

	private JTextField cellPhoneField = new JTextField();

	private JTextField labPhoneField = new JTextField();

	private JTextField faxPhoneField = new JTextField();

	private JTextField emailField = new JTextField(); // --
	// xx@xx.xx

	// format

	private JTextField organizationField = new JTextField();// --

	// organization
	// affiliation

	private JTextField personalhrefField = new JTextField(); // --

	// personal
	// web page

	private JTextField bioField = new JTextField(); // -- one
	// line

	// description of
	// interests

	private JTextField pictureField = new JTextField(); // --
	// url
	// of

	// image

	private JTextField akaFirstField = new JTextField(); // --
	// 'a'lso

	// 'k'nown 'a's
	// first name

	private JCheckBox vipCheck = new JCheckBox("VIP");

	// MAKE CHECK BOX vip TEXT, -- vip status, used for the corpus generator and
	// -- (eventually) scoring?.
	private JCheckBox typeCheck = new JCheckBox("Type");

	// JComboBox? JCheckBox? JTextField type -- 'www' if the person appears on
	// the website,
	// -- null otherwise
	private JTextField weightField = new JTextField();// --

	// this
	// person's

	// weight for STP

	/**
	 * Initializes the People Panel
	 * 
	 * @param tab
	 *            the tabbed panel upon which this is, used so that the find
	 *            window can change tabs when needed
	 * @param peopleModel
	 *            The model containing all people that should be displayed and
	 *            edited on this panel.
	 * @param s
	 *            a session used to save the data back to the database after
	 *            it's been edited
	 */
	public PeoplePanel(JTabbedPane tab, PeopleModel peopleModel, Session s) {
		super();
		setSession(s);
		setDataList(peopleModel);
		nextButton.setText("Next Person");
		prevButton.setText("Previous Person");
		newButton.setText("New Person");
		deleteButton.setText("Delete Person");
		statusLabel = new JLabel("I am the status label");
		prevButton.setEnabled(false);
		findWindow = new FindPerson(PeoplePanel.this, peopleModel, tab);
		addModificationListeners(textFieldModificationListener,
				comboBoxModificationListener, checkBoxModificationListener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#load(org.hibernate.Session)
	 */
	public boolean load(Session session) {
		// setDataList(load(session, Person.class, "last");
		// return getDataList() != null && !(getDataList().isEmpty());
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.base.BasePanel#save()
	 */
	public boolean save() {
		Person person = (Person) getDataList().get(getIndexOfDisplayed()); 
		saveText(person);
		return super.persist(person);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#getPane()
	 */
	public JPanel getPane() {
		final JLabel firstNameLabel = new JLabel("First:");
		final JLabel middleNameLabel = new JLabel("Middle:");
		final JLabel lastNameLabel = new JLabel("Last:");
		final JLabel officePhoneLabel = new JLabel("Office:");
		final JLabel homePhoneLabel = new JLabel("Home:");
		final JLabel cellPhoneLabel = new JLabel("Cell:");
		final JLabel labPhoneLabel = new JLabel("Lab:");
		final JLabel faxPhoneLabel = new JLabel("Fax:");
		final JLabel emailLabel = new JLabel("Email:");
		final JLabel organizationLabel = new JLabel("Organization:");
		final JLabel personalhrefLabel = new JLabel("Personal Website:");
		final JLabel bioLabel = new JLabel("Interests:");
		final JLabel pictureLabel = new JLabel("Photo URL:");
		final JLabel aliasLabel = new JLabel("Alias:");
		final JLabel weightLabel = new JLabel("STP Weight:");

		this.setName("Person");
		final JPanel namePanel = new JPanel();
		namePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createEtchedBorder(), "Name:"));
		final JPanel phonePanel = new JPanel();
		phonePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createEtchedBorder(), "Phone Numbers:"));
		final JPanel miscPanel = new JPanel();
		miscPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createEtchedBorder(), "Misc. Information:"));
		final ConstraintFactory cf = new ConstraintFactory(8, 8, 8, 8, 2);
		GridBagConstraints gbc = new GridBagConstraints();
		this.setLayout(new GridBagLayout());
		namePanel.setLayout(new GridBagLayout());
		phonePanel.setLayout(new GridBagLayout());
		miscPanel.setLayout(new GridBagLayout());
		int row = 0;
		int col = 0;

		// construct namePanel
		gbc = cf.getConstraint(col++, row, ConstraintFactory.LABEL,
				ConstraintFactory.TOPMOST);
		namePanel.add(firstNameLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		namePanel.add(firstNameField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		namePanel.add(aliasLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		namePanel.add(akaFirstField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		namePanel.add(middleNameLabel, gbc);

		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		namePanel.add(middleNameField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		namePanel.add(lastNameLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		namePanel.add(lastNameField, gbc);

		// add namePanel to the mainPanel
		gbc = cf.getConstraint(0, 0, 2, ConstraintFactory.FIELD,
				ConstraintFactory.TOPMOST);
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		this.add(namePanel, gbc);

		// construct phonePanel
		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		phonePanel.add(officePhoneLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		phonePanel.add(officePhoneField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		phonePanel.add(homePhoneLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		phonePanel.add(homePhoneField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		phonePanel.add(cellPhoneLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		phonePanel.add(cellPhoneField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		phonePanel.add(labPhoneLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		phonePanel.add(labPhoneField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		phonePanel.add(faxPhoneLabel, gbc);
		gbc = cf.getConstraint(col--, row, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		phonePanel.add(faxPhoneField, gbc);

		// add the phone panel to the mainPanel
		gbc = cf.getConstraint(0, 1, 2, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		this.add(phonePanel, gbc);

		// construct miscPanel
		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		miscPanel.add(emailLabel, gbc);
		gbc = cf.getConstraint(col--, row, 2, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		miscPanel.add(emailField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		miscPanel.add(organizationLabel, gbc);
		gbc = cf.getConstraint(col--, row, 2, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		miscPanel.add(organizationField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		miscPanel.add(personalhrefLabel, gbc);
		gbc = cf.getConstraint(col--, row, 2, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		miscPanel.add(personalhrefField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		miscPanel.add(pictureLabel, gbc);
		gbc = cf.getConstraint(col--, row, 2, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		miscPanel.add(pictureField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		miscPanel.add(bioLabel, gbc);
		gbc = cf.getConstraint(col--, row, 2, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		miscPanel.add(bioField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		miscPanel.add(weightLabel, gbc);
		gbc = cf.getConstraint(col--, row, 2, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		miscPanel.add(weightField, gbc);

		gbc = cf.getConstraint(col++, ++row, ConstraintFactory.LABEL,
				ConstraintFactory.BOTTOMMOST);
		miscPanel.add(new JLabel("Options:"), gbc);
		gbc = cf.getConstraint(col++, row, ConstraintFactory.FIELD,
				ConstraintFactory.BOTTOMMOST);
		miscPanel.add(vipCheck, gbc);
		gbc = cf.getConstraint(col++, row, ConstraintFactory.FIELD,
				ConstraintFactory.BOTTOMMOST);
		miscPanel.add(typeCheck, gbc);
		col = 0;

		// add the miscPanel to the mainPanel
		gbc = cf.getConstraint(0, 2, 2, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		this.add(miscPanel, gbc);

		gbc = cf.getConstraint(0, 3, 2, ConstraintFactory.FIELD,
				ConstraintFactory.BOTTOMMOST);
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.SOUTHEAST;
		gbc.insets = new Insets(0, 0, 12, 0);
		this.add(getButtonPanel(), gbc);

		// add focus listeners to all the fields to enable cut/copy/paste
		firstNameField.addFocusListener(this);
		middleNameField.addFocusListener(this);
		lastNameField.addFocusListener(this);
		akaFirstField.addFocusListener(this);
		officePhoneField.addFocusListener(this);
		homePhoneField.addFocusListener(this);
		cellPhoneField.addFocusListener(this);
		labPhoneField.addFocusListener(this);
		faxPhoneField.addFocusListener(this);
		emailField.addFocusListener(this);
		organizationField.addFocusListener(this);
		personalhrefField.addFocusListener(this);
		bioField.addFocusListener(this);
		pictureField.addFocusListener(this);

		return this;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see airspaceGUI.expansion.PanelUtils#fillInText()
	 */
	public void fillInText(Session session) {
		if (getDataList() == null || getDataList().isEmpty()) {
			setSession(session);
			load(session);
		}
		fillInText((Person) getDataList().get(getIndexOfDisplayed()));
	}

	public void fillInText(Object object) {
		if (object instanceof Person) {
			fillInText((Person) object);
		}
	}

	/**
	 * Fills in the text of the GUI display with the data from the person
	 * parameter
	 * 
	 * @param person
	 *            The data to display on the PeoplePanel.
	 */
	protected void fillInText(Person person) {
		firstNameField.setText(person.getFirst());
		middleNameField.setText(person.getMiddle());
		lastNameField.setText(person.getLast());
		akaFirstField.setText(person.getAkafirst());

		// make this more complecated and put in code to make it look like a
		// normal phone number?
		officePhoneField.setText(person.getOfficePhone());
		homePhoneField.setText(person.getHomePhone());
		cellPhoneField.setText(person.getCellPhone());
		labPhoneField.setText(person.getLabPhone());
		faxPhoneField.setText(person.getFaxPhone());
		emailField.setText(person.getEmail());
		organizationField.setText(person.getOrganization());
		personalhrefField.setText(person.getPersonalHref());
		bioField.setText(person.getBio());
		pictureField.setText(person.getPictureHref());
		if (person.getWeight() != null) {
			weightField.setText(person.getWeight().toString());
		}

		if (person.getVip() != null) {
			vipCheck.setSelected(person.getVip().equals("true"));
		}
		typeCheck.setSelected(person.getType() != null);

		// just in case something doesn't increment getIndexOfDisplayed()
		// properly
		setIndexOfDisplayed(getDataList().indexOf(person));
		setStatus(getIndexOfDisplayed() + 1);
	}

	/**
	 * Sets the status field on this panel correctly so that the user will know
	 * which piece they're working on and how many there are total.
	 * 
	 * @param numOn
	 *            The index of the currently displayed data + 1
	 * 
	 */
	protected void setStatus(int numOn) {
		statusLabel.setText("On person " + numOn + " of "
				+ getDataList().size() + " people");
	}

	/**
	 * Clears the text out of the GUI, sometimes in preparation for the next set
	 * of data to be applied.
	 */
	public void createNewInstance() {
		clearPanel();
		
		getDataList().add(getIndexOfDisplayed(), new Person());
	}
	
	/* (non-Javadoc)
	 * @see airspaceGUI.base.BasePanel#clearPanel()
	 */
	public void clearPanel() {
		firstNameField.setText("");
		middleNameField.setText("");
		lastNameField.setText("");
		akaFirstField.setText("");
		officePhoneField.setText("");
		homePhoneField.setText("");
		cellPhoneField.setText("");
		labPhoneField.setText("");
		faxPhoneField.setText("");
		emailField.setText("");
		organizationField.setText("");
		personalhrefField.setText("");
		bioField.setText("");
		pictureField.setText("");
		weightField.setText("");

		vipCheck.setSelected(false);
		typeCheck.setSelected(false);		
		
		firstNameField.requestFocusInWindow();
	}

	public void gotoNumber(int number) {
		// we let the user decide if he wants to save the changes or not
		// saveText((Person)getDataList().get(getIndexOfDisplayed()));
		fillInText((Person) getDataList().get(number));
		setIndexOfDisplayed(number);
		// setStatus(getIndexOfDisplayed() + 1);
	}

	/**
	 * Saves the text that is being displayed back to an object
	 * 
	 * @return the object that the text was saved to.
	 */
	public Object saveText() {
		Person person = (Person) getDataList().get(getIndexOfDisplayed());
		
		if (saveText(person)) {
			return person;
		} else {
			return null;
		}
	}

	private boolean saveText(Person person) {
		if (this != null) {
			throw new RuntimeException();
		}
		
		person.setFirst(firstNameField.getText());
		person.setMiddle(middleNameField.getText());
		person.setLast(lastNameField.getText());
		person.setAkafirst(akaFirstField.getText());
		person.setEid(person.getFirst() + person.getLast());

		// Need to add Phone verification
		if (officePhoneField.getText().matches(PHONEVERIFICATION)
				|| officePhoneField.getText().equals("")) {
			person.setOfficePhone(officePhoneField.getText().replaceAll(
					"\\((\\d{3})\\)", "$1 ").replaceAll("-", " "));
		} else {
			// TODO Notify user of phone error
			JOptionPane.showMessageDialog(this,
					"There is an error in the office " + "phone number of "
							+ person.getFirst() + " " + person.getLast(),
					"Error", JOptionPane.ERROR_MESSAGE);
			return false;
		}

		if (homePhoneField.getText().matches(PHONEVERIFICATION)
				|| homePhoneField.getText().equals("")) {
			person.setHomePhone(homePhoneField.getText().replaceAll(
					"\\((\\d{3})\\)", "$1 ").replaceAll("-", " "));
		} else {
			JOptionPane.showMessageDialog(this,
					"There is an error in the home " + "phone number of "
							+ person.getFirst() + " " + person.getLast(),
					"Error", JOptionPane.ERROR_MESSAGE);
			return false;
		}

		if (cellPhoneField.getText().matches(PHONEVERIFICATION)
				|| cellPhoneField.getText().equals("")) {
			person.setCellPhone(cellPhoneField.getText().replaceAll(
					"\\((\\d{3})\\)", "$1 ").replaceAll("-", " "));
		} else {
			// TODO Notify user of phone error
			JOptionPane.showMessageDialog(this,
					"There is an error in the cell " + "phone number of "
							+ person.getFirst() + " " + person.getLast(),
					"Error", JOptionPane.ERROR_MESSAGE);
			return false;
		}

		if (labPhoneField.getText().matches(PHONEVERIFICATION)
				|| labPhoneField.getText().equals("")) {
			person.setLabPhone(labPhoneField.getText().replaceAll(
					"\\((\\d{3})\\)", "$1 ").replaceAll("-", " "));
		} else {
			// TODO Notify user of phone error
			JOptionPane.showMessageDialog(this, "There is an error in the lab "
					+ "phone number of " + person.getFirst() + " "
					+ person.getLast(), "Error", JOptionPane.ERROR_MESSAGE);
			return false;
		}

		if (faxPhoneField.getText().matches(PHONEVERIFICATION)
				|| faxPhoneField.getText().equals("")) {
			person.setFaxPhone(faxPhoneField.getText());
		} else {
			// TODO Notify user of phone error
			JOptionPane.showMessageDialog(this, "There is an error in the fax "
					+ "phone number of " + person.getFirst() + " "
					+ person.getLast(), "Error", JOptionPane.ERROR_MESSAGE);
			return false;
		}

		// verify email is valid
		if (emailField.getText().matches(EMAILVERIFICATION)) {
			person.setEmail(emailField.getText());
		} else {
			// TODO Notify user of phone error
			JOptionPane.showMessageDialog(this,
					"There is an error in the email address" + " of "
							+ person.getFirst() + " " + person.getLast(),
					"Error", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		person.setOrganization(organizationField.getText());
		person.setPersonalHref(personalhrefField.getText());
		person.setBio(bioField.getText());
		person.setPictureHref(pictureField.getText());

		if (!weightField.getText().equals("")) {
			try {
				person.setWeight(new Integer(weightField.getText()));
			} catch (NumberFormatException e) {
				e.printStackTrace();
			}
		} else {
			person.setWeight(null);
		}

		person.setVip(Boolean.toString(vipCheck.isSelected()));
		if (typeCheck.isSelected()) {
			person.setType("www");
		} else {
			person.setType(null);
		}
		
		return true;
	}

	public String getTitle() {
		return "Edit Person";
	}

	protected void addModificationListeners(
			DocumentListener textFieldModificationListener,
			ItemListener checkBoxModificationListener,
			ChangeListener comboBoxModificationListener) {
		firstNameField.getDocument().addDocumentListener(textFieldModificationListener);
		middleNameField.getDocument().addDocumentListener(textFieldModificationListener);
		lastNameField.getDocument().addDocumentListener(textFieldModificationListener);
		akaFirstField.getDocument().addDocumentListener(textFieldModificationListener);
		officePhoneField.getDocument().addDocumentListener(textFieldModificationListener);
		homePhoneField.getDocument().addDocumentListener(textFieldModificationListener);
		cellPhoneField.getDocument().addDocumentListener(textFieldModificationListener);
		labPhoneField.getDocument().addDocumentListener(textFieldModificationListener);
		faxPhoneField.getDocument().addDocumentListener(textFieldModificationListener);
		emailField.getDocument().addDocumentListener(textFieldModificationListener);
		organizationField.getDocument().addDocumentListener(textFieldModificationListener);
		personalhrefField.getDocument().addDocumentListener(textFieldModificationListener);
		bioField.getDocument().addDocumentListener(textFieldModificationListener);
		pictureField.getDocument().addDocumentListener(textFieldModificationListener);
		weightField.getDocument().addDocumentListener(textFieldModificationListener);

		vipCheck.addItemListener(checkBoxModificationListener);
		typeCheck.addItemListener(checkBoxModificationListener);
	}

	protected void addUndoableEditListeners(
			final UndoableEditListener undoListener) {
		firstNameField.getDocument().addUndoableEditListener(undoListener);
		middleNameField.getDocument().addUndoableEditListener(undoListener);
		lastNameField.getDocument().addUndoableEditListener(undoListener);
		akaFirstField.getDocument().addUndoableEditListener(undoListener);
		officePhoneField.getDocument().addUndoableEditListener(undoListener);
		homePhoneField.getDocument().addUndoableEditListener(undoListener);
		cellPhoneField.getDocument().addUndoableEditListener(undoListener);
		labPhoneField.getDocument().addUndoableEditListener(undoListener);
		faxPhoneField.getDocument().addUndoableEditListener(undoListener);
		emailField.getDocument().addUndoableEditListener(undoListener);
		organizationField.getDocument().addUndoableEditListener(undoListener);
		personalhrefField.getDocument().addUndoableEditListener(undoListener);
		bioField.getDocument().addUndoableEditListener(undoListener);
		pictureField.getDocument().addUndoableEditListener(undoListener);
		weightField.getDocument().addUndoableEditListener(undoListener);
	}

	public FindWindow getFindWindow() {
		// findWindow.setSearchList(getDataList());
		return findWindow;
	}

}
