/**
 * 
 */
package airspaceGUI.base.test;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.text.JTextComponent;

import airspaceGUI.expansion.FindWindow;
import airspaceGUI.expansion.TabbedPanelMenu;


import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * @author srbennet
 *
 */
public class BasePanelTest extends SubPanelTest
{
    
    public static Test suite() {
        TestSuite suite= new TestSuite("CorpusGUI Tests");

        //add individual test classes
        suite.addTestSuite(BasePanelTest.class);
        
        return suite;
    }

    /* (non-Javadoc)
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
    }

    /* (non-Javadoc)
     * @see junit.framework.TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }
    
    protected void pressNextButton()
    {
        JButton next = getButton(thisPanel, "next");
        
        int nextX = next.getLocationOnScreen().x + next.getWidth()/2;
        int nextY = next.getLocationOnScreen().y + next.getHeight()/2;
        
        robot.mouseMove(nextX, nextY);
        robot.mousePress(InputEvent.BUTTON1_MASK);
        robot.mouseRelease(InputEvent.BUTTON1_MASK);
    }
    
    protected void pressPrevButton()
    {
        JButton prev = getButton(thisPanel, "prev");
        //ensure that the fields have something in them
       
        int prevX = prev.getLocationOnScreen().x + prev.getWidth()/2;
        int prevY = prev.getLocationOnScreen().y + prev.getHeight()/2;
       
        robot.mouseMove(prevX, prevY);

        robot.mousePress(InputEvent.BUTTON1_MASK);
        robot.mouseRelease(InputEvent.BUTTON1_MASK);
    }
    
    protected void pressNewButton()
    {
        JButton newButton = getButton(thisPanel, "new");
        //ensure that the fields have something in them
       
        int newX = newButton.getLocationOnScreen().x + newButton.getWidth()/2;
        int newY = newButton.getLocationOnScreen().y + newButton.getHeight()/2;
       
        robot.mouseMove(newX, newY);

        robot.mousePress(InputEvent.BUTTON1_MASK);
        robot.mouseRelease(InputEvent.BUTTON1_MASK);
    }
    
    protected void pressDeleteButton()
    {
        JButton delete = getButton(thisPanel, "delete");
        //ensure that the fields have something in them
       
        int delX = delete.getLocationOnScreen().x + delete.getWidth()/2;
        int delY = delete.getLocationOnScreen().y + delete.getHeight()/2;
       
        robot.mouseMove(delX, delY);

        robot.mousePress(InputEvent.BUTTON1_MASK);
        robot.mouseRelease(InputEvent.BUTTON1_MASK);
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#addModificationListeners(java.awt.event.KeyListener, java.awt.event.ItemListener, javax.swing.event.ChangeListener)}.
     */
    public void testAddModificationListeners() {
        super.testAddModificationListeners();
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#addUndoableEditListeners(javax.swing.event.UndoableEditListener)}.
     */
    public void testAddUndoableEditListeners() {
        super.testAddUndoableEditListeners();
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#deleteDisplayedObject()}.
     */
    public void testDeleteDisplayedObject() {
        super.testDeleteDisplayedObject();
        
        thisPanel.fillInText(session);
        Object saved = thisPanel.saveText();
        pressNextButton();
        
        pressPrevButton();
        
        Object newSaved = thisPanel.saveText();
        boolean result = saved.equals(newSaved);
        assertTrue(result);
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#getStatusText()}.
     */
    public void testGetStatusText() {
        super.testGetStatusText();
        // not sure what to do here
        // the panel should be blank now
        String startingText = thisPanel.getStatusText();
        thisPanel.fillInText(session);
        
//        System.out.println(startingText);
//        System.out.println(thisPanel.getStatusText());
        
        //assertFalse(thisPanel.getStatusText().equals(startingText));
        startingText = thisPanel.getStatusText();
        pressNextButton();

//        System.out.println("right before check" + startingText);
//        System.out.println(thisPanel.getStatusText());
        assertFalse(thisPanel.getStatusText().equals(startingText));
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#exit()}.
     */
    public void testExit() {
        //TODO check if this is really necessary
        super.testExit();
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#saveText()}.
     */
    public void testSaveText() {
        super.testSaveText();
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#createNewInstance()}.
     */
    public void testClearText() {
        super.testClearText();
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#getButtonPanel()}.
     */
    public void testGetButtonPanel() {
        try{
            Class thisClass = Class.forName("airspaceGUI.base.BasePanel");
            Method getButtons = thisClass.getDeclaredMethod("getButtonPanel", null);
            getButtons.setAccessible(true);
            JPanel buttons = (JPanel)getButtons.invoke(thisPanel, null);
        
            assertNotNull( buttons);
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        } 
        //TODO figure out if there are any other things we should be doing here
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#getMinimumSize()}.
     */
    public void testGetMinimumSize() {
        Dimension d = new Dimension(0,0);
        assertTrue(thisPanel.getMinimumSize().height > d.height 
                && thisPanel.getMinimumSize().width > d.width);
        
        try{
            Class thisClass = Class.forName("airspaceGUI.base.BasePanel");
            Method getButtons = thisClass.getDeclaredMethod("getButtonPanel", null);
            getButtons.setAccessible(true);
            JPanel buttons = (JPanel)getButtons.invoke(thisPanel, null);
            
            assertTrue(thisPanel.getMinimumSize().width >= buttons.getMinimumSize().width 
                    && thisPanel.getMinimumSize().height >= buttons.getMinimumSize().height);
        
        }catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        } 
        
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#setStatus(int)}.
     */
    public void testSetStatus() {
        int startIndex = thisPanel.getIndexOfDisplayed();
        try{
            Class thisClass = Class.forName("airspaceGUI.base.BasePanel");
            Class[] parameterClasses = {Integer.TYPE};
            Method setStatus = thisClass.getDeclaredMethod("setStatus", parameterClasses);
            
            thisPanel.fillInText(session);
            setStatus.setAccessible(true);
            
            String startString = thisPanel.getStatusText();
            //add one because the index is one behind the status
            Integer startStatus = new Integer(thisPanel.getIndexOfDisplayed() + 1);
            
            Integer testInt = new Integer(1337);
            Object[] parameters = {testInt};
            setStatus.invoke(thisPanel, parameters);
            String newStatus = thisPanel.getStatusText();
            
            //System.out.println(newStatus + " should contain " + testInt);
           // System.out.println("The index of Displayed is currently " + thisPanel.getIndexOfDisplayed());
            
            assertTrue(newStatus.contains(String.valueOf((testInt))));
            assertFalse(newStatus.equals(startString));
            
            Object[] returnParams = {startStatus};
            setStatus.invoke(thisPanel, returnParams);
            newStatus = thisPanel.getStatusText();
            
            assertEquals(startString, newStatus);
        
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        } 
        finally{
            //restore the panel to its previous state
            thisPanel.gotoNumber(startIndex);
        }
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#setMenuBar(airspaceGUI.expansion.TabbedPanelMenu)}.
     */
    public void testSetMenuBar() {
        try{
            Class thisClass = Class.forName("airspaceGUI.base.BasePanel");
            Class[] setMenuBarClass = {TabbedPanelMenu.class};
            Method setMenuBar = thisClass.getDeclaredMethod("setMenuBar", setMenuBarClass);
            
            //a hack so we can access the private or protected method
            setMenuBar.setAccessible(true);
            
            TabbedPanelMenu tpm = new TabbedPanelMenu();
            Object[] parameters = {tpm};
            setMenuBar.invoke(thisPanel, parameters);
            
            //ensure all menu items have one action associated with them
            //assertTrue(tpm.getFind().getActionListeners().length == 1);
            assertTrue(tpm.getSave().getActionListeners().length == 1);
            assertTrue(tpm.getCut().getActionListeners().length == 1);
            assertTrue(tpm.getCopy().getActionListeners().length == 1);
            assertTrue(tpm.getPaste().getActionListeners().length == 1);
            assertTrue(tpm.getDelete().getActionListeners().length == 1);
            assertTrue(tpm.getNewMenu().getActionListeners().length == 1);
            assertTrue(tpm.getNext().getActionListeners().length == 1);
            assertTrue(tpm.getPrev().getActionListeners().length == 1);
//            assertTrue(tpm.getQuit().getActionListeners().length == 1);
            assertTrue(tpm.getRedo().getActionListeners().length == 1);
            assertTrue(tpm.getUndo().getActionListeners().length == 1);          
        
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        } 
        
        //TODO Make more consise tests to ensure that the 
        //actions actually do the correct things
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#setButtonPermissions()}.
     */
    public void testSetButtonPermissions() {
        // go to the first message and ensure that the previous button is disabled
        // go to the last message and ensure that the next button is disabled
        // go somewhere in teh middle and go to each
        try{
            Class thisClass = Class.forName("airspaceGUI.base.BasePanel");
            Class subPanel = Class.forName("airspaceGUI.base.SubPanel");
            Class[] setIndexParameterClasses = {Integer.TYPE};
            Method setButtonPermissions = thisClass.getDeclaredMethod("setButtonPermissions", null);
            Method setIndexOfDisplayed = subPanel
                .getDeclaredMethod("setIndexOfDisplayed", setIndexParameterClasses);
            Method maxIndex = subPanel.getDeclaredMethod("maxIndex", null);
            Method getDataList = subPanel.getDeclaredMethod("getDataList", null);
            JButton next = getButton(thisPanel, "next");
            JButton prev = getButton(thisPanel, "prev");
            Object[] zeroParameter = {new Integer(0)};
      
            
            setButtonPermissions.setAccessible(true);
            setIndexOfDisplayed.setAccessible(true);
            getDataList.setAccessible(true);
            
            List dataList = (List)getDataList.invoke(thisPanel, null);
            Object[] lastParameter = {maxIndex.invoke(thisPanel, null)};
            Object[] middleParameter = {new Integer(1)};
            int last = ((Integer)lastParameter[0]).intValue();
            
            // the panel should now be on the next piece of data so we can't 
            //go to a previous one
            setIndexOfDisplayed.invoke(thisPanel, zeroParameter);
            setButtonPermissions.invoke(thisPanel, null);
            assertFalse(prev.isEnabled());
            assertTrue(next.isEnabled());
            
            // the panel should now be on the last piece of data so we can't go to the next
            setIndexOfDisplayed.invoke(thisPanel, lastParameter);
            setButtonPermissions.invoke(thisPanel, null);
            assertFalse(next.isEnabled());
            assertTrue(prev.isEnabled());
            
            for(int i = 1; i < last; i++)
            {
                setIndexOfDisplayed.invoke(thisPanel, middleParameter);
                setButtonPermissions.invoke(thisPanel, null);
                assertTrue(next.isEnabled());
                assertTrue(prev.isEnabled());
                //middleParameter = i;
                middleParameter[0] = new Integer(i);
            }
            
            
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        } 
    }

    /**
     * Test method for {@link airspaceGUI.base.BasePanel#getFindWindow()}.
     */
    public void testGetFindWindow() {
        try{
            Class thisClass = Class.forName("airspaceGUI.base.BasePanel");
            Method getFindWindow = thisClass.getDeclaredMethod("getFindWindow", null);
            
            getFindWindow.setAccessible(true);
            
            FindWindow f = (FindWindow)getFindWindow.invoke(thisPanel, null);
            assertNotNull(f);
            
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        } 
        
    }

    /**
     * Test method for {@link airspaceGUI.base.SubPanel#getDataList()}.
     */
    public void testGetDataList() {
        super.testGetDataList();
    }

    /**
     * Test method for {@link airspaceGUI.base.SubPanel#getIndexOfDisplayed()}.
     */
    public void testGetIndexOfDisplayed() {
        super.testGetIndexOfDisplayed();
        thisPanel.fillInText(session);
        // be really sure that we're on number 0
        thisPanel.gotoNumber(0);
        Thread.yield();
        int firstIndex = thisPanel.getIndexOfDisplayed();
        //System.out.println("BasePanel 1 The first index is " + firstIndex + " should be 0");
        assertTrue(firstIndex == 0);
        
        pressNextButton();
        
        pressPrevButton();
        
        int secondIndex = thisPanel.getIndexOfDisplayed();
       // System.out.println("BasePanel 2 The first index is " + firstIndex 
        //       + " second is " + secondIndex);
        Thread.yield();
        assertEquals(firstIndex, secondIndex);
        //System.out.println("They are equal");
    }

    /**
     * Test method for {@link airspaceGUI.base.SubPanel#getSession()}.
     */
    public void testGetSession() {
        super.testGetSession();
    }

    /**
     * Test method for {@link airspaceGUI.base.SubPanel#getHasFocus()}.
     */
    public void testGetHasFocus() {
        super.testGetHasFocus();
    }

    /**
     * Test method for {@link airspaceGUI.base.SubPanel#setHasFocus(javax.swing.text.JTextComponent)}.
     */
    public void testSetHasFocus() {
        super.testSetHasFocus();
    }

    /**
     * Test method for {@link airspaceGUI.base.SubPanel#setDataList(java.util.List)}.
     */
    public void testSetDataList() {
        super.testSetDataList();
    }

    /**
     * Test method for {@link airspaceGUI.base.SubPanel#setIndexOfDisplayed(int)}.
     */
    public void testSetIndexOfDisplayed() {
        super.testSetIndexOfDisplayed();
    }

    /**
     * Test method for {@link airspaceGUI.base.SubPanel#setSession(org.hibernate.Session)}.
     */
    public void testSetSession() {
        super.testSetSession();
    }

    /**
     * Test method for {@link airspaceGUI.base.SubPanel#isModified()}.
     */
    public void testIsModified() {
        super.testIsModified();
        
        try{
            Method isModified = Class.forName("airspaceGUI.base.SubPanel")
                .getDeclaredMethod("isModified", null);
            thisPanel.fillInText(session);
            isModified.setAccessible(true);
        
            // tab to a field so we can type something
            Component[] components = thisPanel.getComponents();
            int i = 0;
            while (components[i] instanceof JTextComponent)
            {
                typeKey(KeyEvent.VK_TAB);
                i++;
            }
            typeKey(KeyEvent.VK_1);
           
            assertTrue("The panel doesn't detect modifications",
                    ((Boolean)isModified.invoke(thisPanel, null))
                            .booleanValue());
            typeKey(KeyEvent.VK_BACK_SPACE);
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        }
    }

    /**
     * Test method for {@link airspaceGUI.base.SubPanel#setModified(boolean)}.
     */
    public void testSetIsModified() {
        super.testSetIsModified();
    }

    /**
     * Test method for {@link airspaceGUI.base.SubPanel#save(java.util.List)}.
     */
    public void testSave() {
        super.testSave();
    }

    /**
     * Test method for {@link airspaceGUI.base.SubPanel#fillInText(java.lang.Object)}.
     */
    public void testFillInTextObject() {
        super.testFillInTextObject();
    }

    public void testGotoNumber() {
        //need reflection to check and ensure that 
        //the object that is showing is equivalent to 
        //the one in the datalist at the number that was passed
        // to this function
        super.testGotoNumber();
    }


    public void testGetPane() {
        super.testGetPane();
        //TODO not sure how to automatically test if the getPane works or not...
    }

    public void testFillInTextSession() {
        super.testFillInTextSession();
    }
    
}
