package airspaceGUI.base.test;

import java.awt.AWTException;
import java.awt.Component;
import java.awt.Robot;
import java.awt.TextComponent;
import java.awt.event.InputEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.swing.AbstractButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.hibernate.Session;
import org.hibernate.cfg.Configuration;

import airspaceGUI.base.corpus.gui.CorpusMainPanel;
import airspaceGUI.model.PeopleModel;

/**
 * I'm not sure this is even necessary, since all the calls to these functions
 * would be coming through the main window, and therefore its tests should cover these functions
 * @author srbennet
 *
 */
public class CorpusMainPanelTest extends TestCase
{

    protected CorpusMainPanel thisPanel;
    private Robot robot;
    protected Session session;

    
    KeyListener keyListener = new KeyListener(){

        public void keyPressed(KeyEvent arg0) {
            // empty
           
        }

        public void keyReleased(KeyEvent arg0) {
            // empty
        }

        public void keyTyped(KeyEvent arg0) {
            // erase the character that was just typed to 
            // show that control got here
            typeKey(KeyEvent.VK_BACK_SPACE);
        }
        
    };
    
    ItemListener itemListener = new ItemListener(){
        public void itemStateChanged(ItemEvent arg0) {
            ((AbstractButton)arg0.getItem())
                .setSelected(((AbstractButton)arg0.getItem())
                        .getSelectedObjects() != null);
        }
        
    };
    
    ChangeListener changeListener = new ChangeListener(){
        public void stateChanged(ChangeEvent arg0) {
            // figure out what goes here
        }
        
    };
    private UndoableEditListener undoableEditListener = new UndoableEditListener(){

        public void undoableEditHappened(UndoableEditEvent arg0) {
            // undo the edit, like it was never done.
            arg0.getEdit().undo();
        }
        
    };
    
    public static Test suite() {
        TestSuite suite= new TestSuite("CorpusGUI Tests");

        //add individual test classes
        suite.addTestSuite(CorpusMainPanelTest.class);
        
        return suite;
    }

    protected void setUp() throws Exception {
        super.setUp();
        JFrame frame = new JFrame();
        
        if (robot == null)
        {
            try
            {
                robot = new Robot();
            } catch (AWTException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        try
        {
            if (session == null)
            {
                // Create the SessionFactory from hibernate.cfg.xml

                final File configFile = new File("corpus.hibernate.xml");
                final Configuration config = new Configuration()
                        .configure(configFile);
                //System.out.println("ummm... opening Session");
                session = config.buildSessionFactory().openSession();
            }
            
            if (!this.getClass().equals(DataKeeper.lastClass))
            {
                DataKeeper.peopleModel = new PeopleModel(session);
                DataKeeper.lastClass = this.getClass();
            }
            
            if (thisPanel == null){
                thisPanel = new CorpusMainPanel(frame, session, DataKeeper.peopleModel);
                frame.getContentPane().add(thisPanel);
                frame.pack();
            }

        } catch (Exception ex)
        {
            // Make sure you log the exception, as it might be swallowed
            System.err.println("Initial SessionFactory creation failed." + ex);
            throw new ExceptionInInitializerError(ex);
        }
        frame.setVisible(true);
        robot.setAutoWaitForIdle(true);
        robot.setAutoDelay(300);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }
    
    protected void typeKey(int keyCode)
    {
        robot.keyPress(keyCode);
        robot.keyRelease(keyCode);
    }

    public void testFillInText() {
        fail("Not yet implemented"); // TODO
    }

    public void testClearSelectedLabels() {
        fail("Not yet implemented"); // TODO
    }

    public void testRemoveUndoListeners() {
        fail("Not yet implemented"); // TODO
    }

    public void testAddUndoListeners() {
        fail("Not yet implemented"); // TODO
    }

    public void testAddModificationListeners() {
        Component[] components = thisPanel.getComponents();
        Class thisClass = thisPanel.getClass();
        
        Class[] parametersClasses = {KeyListener.class, 
                ItemListener.class, ChangeListener.class};
        Object[] parameters = {keyListener, itemListener, changeListener};
        try
        {
            Method m = thisClass.getDeclaredMethod("addModificationListeners", parametersClasses);
            m.setAccessible(true);
            m.invoke(thisPanel, parameters);
            
            for(int i = 0; i < components.length; i++)
            {
                // check and make sure each component has the correct listener 
                // on it
                if (components[i] instanceof TextComponent){
                    // type something and ensure that the keylistener fires
                    components[i].requestFocusInWindow();
                    typeKey(KeyEvent.VK_S);
                    
                    assertTrue(((TextComponent)components[i]).getText().equals(""));

                    typeKey(KeyEvent.VK_BACK_SPACE);
                    
                } else if (components[i] instanceof AbstractButton)
                {
                    // this is for buttons and checkboxes
                    Component component = components[i];
                    boolean selectedStartValue = ((AbstractButton)component).isSelected();
                    
                    int xPos = component.getX() + component.getWidth()/3;
                    int yPos = component.getY() + component.getHeight()/2;
                    robot.mouseMove(xPos, yPos);
                    robot.mousePress(InputEvent.BUTTON1_DOWN_MASK);
                    robot.mouseRelease(InputEvent.BUTTON1_DOWN_MASK);
                    
                    assertFalse(((AbstractButton)component).isSelected() == selectedStartValue);
                    
                } else if (components[i] instanceof JComboBox)
                {
                    // this is i believe, for everything else
//                  this is i believe, for everything else
                    Component component = components[i];
                    Object selected = ((JComboBox)component).getSelectedItem();
                    
                    int xPos = component.getX() + (3 * component.getWidth())/4;
                    int yPos = component.getY() + component.getHeight()/2;
                    robot.mouseMove(xPos, yPos);
                    robot.mousePress(InputEvent.BUTTON1_DOWN_MASK);
                    robot.mouseRelease(InputEvent.BUTTON1_DOWN_MASK);
                    
                    assertTrue(((JComboBox)component).getSelectedItem().equals(selected));

                }
            }
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } 
    }

    public void testGetSelectedLabels() {
        fail("Not yet implemented"); // TODO
    }

}
