package airspaceGUI.base.test;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;

import javax.swing.JFrame;

import airspaceGUI.base.person.PeoplePanel;
import airspaceGUI.model.PeopleModel;

import junit.framework.Test;
import junit.framework.TestSuite;

import edu.cmu.radar.base.Person;

public class PeoplePanelTest extends BasePanelTest
{

    JFrame frame = new JFrame();
    
    protected void setUp() throws Exception {
        super.setUp();
        if (!this.getClass().equals(DataKeeper.lastClass))
        {
            DataKeeper.peopleModel = new PeopleModel(session);
            DataKeeper.lastClass = this.getClass();
        }
        
        if (thisPanel == null)
        {
            thisPanel = new PeoplePanel(null, DataKeeper.peopleModel, session);
            frame.getContentPane().add(thisPanel.getPane());
            frame.pack();
            frame.setVisible(true);
        }
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        frame.setVisible(false);

    }
    
    public static Test suite() {
        TestSuite suite= new TestSuite("CorpusGUI Tests");

        //add individual test classes
        suite.addTestSuite(PeoplePanelTest.class);
        
        return suite;
    }

    public void testAddModificationListeners() {
        super.testAddModificationListeners();
    }

    public void testAddUndoableEditListeners() {
        super.testAddUndoableEditListeners();
    }

    public void testSaveText() {
        super.testSaveText();
    }

    public void testClearText() {
        super.testClearText();
    }

    public void testFillInTextObject() {
        super.testFillInTextObject();
    }

    public void testSetStatus() {
        super.testSetStatus();
    }

    public void testGetFindWindow() {
        super.testGetFindWindow();
    }

    public void testSave() {
        super.testSave();
    }

    public void testGetPane() {
        super.testGetPane();
        //TODO not sure how to automatically test if the getPane works or not...
    }

    public void testFillInTextSession() {
        super.testFillInTextSession();
    }

    public void testFillInTextPerson() {
        try
        {
            Method method = Class.forName("airspaceGUI.base.SubPanel").getDeclaredMethod("getDataList", null);
            method.setAccessible(true);
            List l = (List)method.invoke(thisPanel, null);
            
            Person toBeDisplayed = (Person)l.get(thisPanel.getIndexOfDisplayed());
            
            thisPanel.fillInText(toBeDisplayed);
            assertTrue(thisPanel.saveText().equals(toBeDisplayed));
            
            
        } catch (SecurityException e)
        {
            e.printStackTrace();
            fail("Security exception");
        } catch (NoSuchMethodException e)
        {
            e.printStackTrace();
            fail("No such method exception");
        } catch (IllegalArgumentException e)
        {
            e.printStackTrace();
            fail("Illegal argument exception");
        } catch (IllegalAccessException e)
        {
            e.printStackTrace();
            fail("Ilegal access exception");
        } catch (InvocationTargetException e)
        {
            e.printStackTrace();
            fail("Invocation target exception");
        } catch (ClassNotFoundException e)
        {
            e.printStackTrace();
            fail("Class not found exception");
        } 
    }

    public void testGotoNumber() {
        //need reflection to check and ensure that 
        //the object that is showing is equivalent to 
        //the one in the datalist at the number that was passed
        // to this function
        super.testGotoNumber();
    }

    public void testDeleteDisplayedObject() {
        super.testDeleteDisplayedObject();
    }

    public void testGetStatusText() {
        super.testGetStatusText();
    }

    public void testExit() {
        super.testExit();
    }

    public void testSetButtonPermissions() {
        super.testSetButtonPermissions();
    }

    public void testMaxIndex() {
        super.testMaxIndex();
    }

    public void testGetDataList() {
        super.testGetDataList();
    }

    public void testGetIndexOfDisplayed() {
        //TODO figure out if this is necessary
        super.testGetIndexOfDisplayed();
    }

    public void testGetSession() {
        super.testGetSession();
    }

    public void testGetHasFocus() {
        super.testGetHasFocus();
    }

    public void testSetHasFocus() {
        super.testSetHasFocus();
    }

    public void testSetDataList() {
        //need to generate a datalist and then
        // test and make sure the data list in the panel is 
        // the same as this generated datalist
    }

    public void testSetIndexOfDisplayed() {
        //similar to testGotoNumber
    }

    public void testSetSession() {
        super.testSetSession();
    }

    public void testLoadSessionClassString() {
        List list = thisPanel.load(session, Person.class, "id");
        assertNotNull(list);
        Person lastPerson = (Person)list.get(0);
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Person);
            assertTrue(lastPerson.getId().intValue() < ((Person)list.get(i)).getId().intValue());
        }
        
        list = thisPanel.load(session, Person.class, "first");
        assertNotNull(list);
        
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Person);
            // check that the first value is increasing
        }
        
        list = thisPanel.load(session, Person.class, "middle");
        assertNotNull(list);
        
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Person);
            // check that the middle value is increasing
        }
        
        list = thisPanel.load(session, Person.class, "last");
        assertNotNull(list);
        
        for(int i = 1; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Person);
            // check that the last value is increasing
        }

    }

    public void testLoadSessionClass() {
        List list = thisPanel.load(session, Person.class);
        assertNotNull(list);
        
        for(int i = 0; i < list.size(); i++)
        {
            assertTrue(list.get(i) instanceof Person);
        }
    }

    public void testIsModified() {
        super.testIsModified();
    }

    public void testSetIsModified() {
        super.testIsModified();
    }

    public void testSaveList() {
        super.testSaveList();
    }

}
