package airspaceGUI.expansion;

import java.awt.Container;
import java.awt.Font;
import java.awt.HeadlessException;
import java.awt.Image;

import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import relativelayout.Binding;
import relativelayout.Direction;
import relativelayout.Edge;
import relativelayout.RelativeConstraints;
import relativelayout.RelativeLayout;
import relativelayout.RelativePosition;

/**
 * This defines what is shown in the about box dialog in the user menu on a mac
 * 
 * @author srbennet
 * 
 */
public class AboutBox extends JDialog
{
    private final static String VERSION = "Version 0.1";

    private final static String AUTHOR = "Rachael Bennett";

    private final static String COPYRIGHT = "\u00A9 2007";

    private final static String DISCLAIMER = "Please review the Airspace " +
            "website regarding the use of the supplied content: "
            + "http:////www.cs.cmu.edu//~airspace//";

    private final static String CORPUSCONTRIBUTORS = "Email authors include: " +
            "Sarah Barbour, Rachael Bennett, Michelle Bova, Diane Keene, " +
            "Stephanie Kimssy, Pablo-Alejandro Quinones, Jennifer Suh, and" +
            " Django Wexler.";

    private final static String WORLDCONTRIBUTORS = "Simulated world designers include: " +
            "Aaron Spaulding and Aaron Steinfeld.";
    
    private final static String DATABASECONTRIBUTORS = "Database authors include: " +
            "Rachael Bennett, Kyle Cunningham, Matt Lahut, Robert McGuire, "
            + "Isaac Simmons, Anthony Tomasic, and Django Wexler.";

    private final static String DESCRIPTION = "This work was done under the " +
            "Reflective Agents with Distributed Adaptive Reasoning (RADAR) project " +
            "within the DARPA Personalized Assistant that Learns (PAL) program.\n\n  " +
            "This material is based upon work supported by the Defense Advanced " +
            "Research Projects Agency (DARPA) under Contract No. NBCHD030010.  " +
            "Any opinions, findings and conclusions or recommendations expressed " +
            "in this material are those of the authors and do not necessarily " +
            "reflect the views of the DARPA or the Department of Interior-National " +
            "Business Center (DOI-NBC).";

    private final JTextArea aboutField = new JTextArea();

    private final ImageIcon aboutIcon = new ImageIcon("RadarLogo-white.png",
            "RADAR Icon");

    private final JLabel titleLabel = new JLabel("AirspaceGUI", JLabel.CENTER);

    private final JScrollPane aboutFieldScroll = new JScrollPane(aboutField);

    public AboutBox(JFrame parent) throws HeadlessException
    {
        super();
        this.setDefaultCloseOperation(AboutBox.DISPOSE_ON_CLOSE);
        setTitle(titleLabel.getText());
        Image icon = aboutIcon.getImage().getScaledInstance(250, 125,
                Image.SCALE_SMOOTH);
        Container contentPanel = this.getContentPane();
        final JLabel versionLabel = new JLabel(VERSION, JLabel.CENTER);
        contentPanel.setLayout(new RelativeLayout());

        // Bindings for each edge of the frame
        Binding leftEdge = new Binding(Edge.LEFT, 8, Direction.RIGHT,
                Edge.LEFT, this);
        Binding rightEdge = new Binding(Edge.RIGHT, 8, Direction.LEFT,
                Edge.RIGHT, this);
        Binding topEdge = new Binding(Edge.TOP, 4, Direction.BELOW, Edge.TOP,
                this);
        Binding bottomEdge = new Binding(Edge.BOTTOM, 4, Direction.ABOVE,
                Edge.BOTTOM, this);

        RelativePosition justBelow = new RelativePosition(Edge.TOP, 0,
                Direction.BELOW, Edge.BOTTOM);
        RelativePosition justAbove = new RelativePosition(Edge.BOTTOM, 0,
                Direction.ABOVE, Edge.TOP);
        RelativePosition twelveBelow = new RelativePosition(Edge.TOP, 12,
                Direction.BELOW, Edge.BOTTOM);
        RelativePosition verticalCenter = new RelativePosition(
                Edge.VERTICAL_CENTER, 0, Direction.ABOVE, Edge.VERTICAL_CENTER);
        RelativePosition horizontalCenter = new RelativePosition(
                Edge.HORIZONTAL_CENTER, 0, Direction.RIGHT, Edge.HORIZONTAL_CENTER);
        RelativePosition eightRightOf = new RelativePosition(Edge.LEFT, 8,
                Direction.RIGHT, Edge.RIGHT);
        RelativePosition eightLeftOf = new RelativePosition(Edge.RIGHT, 8,
                Direction.LEFT, Edge.LEFT);

        // back to grid bag layout
        RelativeConstraints titleConstraints = new RelativeConstraints(
                new Binding[] {leftEdge, topEdge, rightEdge});
        titleLabel.setFont(titleLabel.getFont().deriveFont(Font.TRUETYPE_FONT, 16));
        contentPanel.add(titleLabel, titleConstraints);

        JLabel iconLabel = new JLabel();
        iconLabel.setIcon(new ImageIcon(icon));
        RelativeConstraints iconConstraints = new RelativeConstraints(
                new Binding[] {new Binding(horizontalCenter, this),
                        new Binding(twelveBelow, titleLabel)});
        contentPanel.add(iconLabel, iconConstraints);

        RelativeConstraints versionConstraints = new RelativeConstraints(
                new Binding[] {leftEdge, rightEdge,
                        new Binding(twelveBelow, iconLabel)});
        contentPanel.add(versionLabel, versionConstraints);

        RelativeConstraints aboutFieldConstraints = new RelativeConstraints(
                new Binding[] {leftEdge,
                        new Binding(twelveBelow, versionLabel), bottomEdge,
                        rightEdge});
        aboutField.setEditable(false);
        fillAboutField();
        contentPanel.add(aboutFieldScroll, aboutFieldConstraints);

        this.setSize(350, 350);
        this.setResizable(false);
        // this.pack();
        this.setLocationRelativeTo(parent);
    }

    private void fillAboutField() {
        aboutFieldScroll.setHorizontalScrollBarPolicy(
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        aboutField.setFont(aboutField.getFont().deriveFont(Font.TRUETYPE_FONT, 11));
        aboutField.setLineWrap(true);
        aboutField.setWrapStyleWord(true);
        aboutField.setText(DISCLAIMER + "\n\n" + CORPUSCONTRIBUTORS + "\n\n" 
                + WORLDCONTRIBUTORS + "\n\n" + DATABASECONTRIBUTORS + "\n\n" 
                + DESCRIPTION);
        
    }

}
