/**
 * 
 */
package airspaceGUI.expansion;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.PopupMenu;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;

/**
 * Implements a fake combo box with auto-complete capabilities, as well as the
 * ability to select more than one item from the menu and make the items
 * appearing in the menu different from those that may be inserted into the text
 * field
 * 
 * @author srbennet
 * 
 */
public class FakeComboBox extends JPanel {

	private transient JButton popupTriggerButton = new JButton();

	private transient AutoCompleteField textField = new AutoCompleteField();

	private transient JScrollPane fieldScroller;

	private transient PopupMenu menu = new PopupMenu();

	private transient List dataForMenu;

	private transient List dataToInsert;

	private transient Map stringToObjectMap = new HashMap();

	private transient boolean modified = false;

	private transient ArrayList selected = new ArrayList();

	/**
	 * if the combo box can only select one value
	 * */
	private boolean singleMode;

	/**
	 * listener to popup the menu atLabel the correct place with the correct
	 * information.
	 * 
	 */
	private transient ActionListener popupListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			menu.show(popupTriggerButton, 0, 30);
		}
	};

	/**
	 * Adds the string representation of the selected object to the text field
	 * 
	 */
	private transient ActionListener selectionListener = new ActionListener() {
		public void actionPerformed(final ActionEvent e) {
			int i = dataForMenu.indexOf(e.getActionCommand());
			selected.add(dataForMenu.get(i));
			String value = (String) dataToInsert.get(i);
			
			String text = textField.getText();
			
			if (singleMode || text.length() == 0) {
				textField.setText(value);
			} else if (!text.contains((String) dataToInsert.get(i))) {
				textField.setText(textField.getText() + ", " + value);
			}

			modified = true;
		}
	};

	/**
	 * Default constructor, does nothing
	 */
	private FakeComboBox() {
		super();
		// TODO Auto-generated constructor stub
	}

	/**
	 * Constructor, initializes everything. Used when the insert text is
	 * different from the menu text
	 * 
	 * @param data
	 *            the data that should be shown in the menu, must be Strings
	 * @param toInsert
	 *            the data to insert into the text field when an item is
	 *            clicked, if different from above
	 * @param buttonText
	 *            The text that should be displayed on the button
	 */
	public FakeComboBox(List data, List toInsert, String buttonText) {
		super();
		init(data, toInsert, buttonText);
	}

	/**
	 * Constructor, initializes everything
	 * 
	 * @param data
	 *            the data that should be shown in the menu
	 * @param buttonText
	 *            The text that should be displayed on the button
	 */
	public FakeComboBox(List data, String buttonText) {
		super();
		init(data, data, buttonText);
	}

	/**
	 * Initalizes all the private fields, this is here to limit the duplication
	 * of code
	 * 
	 * @param data
	 *            the data that should be shown in the menu
	 * @param toInsert
	 *            the data to insert into the text field when an item is
	 *            clicked, if different from above
	 * @param buttonText
	 *            The text that should be displayed on the button
	 */
	private void init(List data, List toInsert, String buttonText) {
		final List dataToSearch = initMenu(data, toInsert, buttonText);
		final Dimension scrollPaneSize = new Dimension(10, 36);

		textField = new AutoCompleteField(dataToSearch);
		fieldScroller = new JScrollPane(textField);
		/*
		 * textField.setSize(scrollPaneSize);
		 * textField.setMinimumSize(scrollPaneSize);
		 * textField.setMaximumSize(scrollPaneSize);
		 */
		textField.setWrapStyleWord(true);
		textField.setLineWrap(true);
		fieldScroller.setPreferredSize(scrollPaneSize);
		fieldScroller.setMinimumSize(scrollPaneSize);
		// fieldScroller.setMaximumSize(scrollPaneSize);

		menu.addActionListener(selectionListener);
		initPanel();
	}

	/**
	 * Initializes the menu that pops up when the button is clicked
	 * 
	 * @param data
	 *            The list of data to search over
	 * @param toInsert
	 *            The list of the items to insert when selected
	 * @param buttonText
	 *            The text the button should display
	 * @return the list of data to search over
	 */
	private List initMenu(List data, List toInsert, String buttonText) {
		ArrayList dataToSearch = new ArrayList();
		// ensure that the menu is empty
		menu.removeAll();
		dataForMenu = data;
		dataToInsert = toInsert;
		popupTriggerButton.setText(buttonText);
		popupTriggerButton.addActionListener(popupListener);
		popupTriggerButton.add(menu);

		if (data != null) {
			// as long as there's data to add
			for (int i = 0; i < data.size(); i++) {
				Object currentObject = data.get(i);
				String stringRep = currentObject.toString();
				menu.add(stringRep);
				stringToObjectMap.put(stringRep, currentObject);
				// System.out.println(i + "data is " + (String)data.get(i));
			}
			dataToSearch = new ArrayList(data);
			if (!data.equals(toInsert)) {
				// System.out.println(dataToSearch);
				dataToSearch.addAll(toInsert);
				// System.out.println(dataToSearch);
			}
		}
		return dataToSearch;
	}

	/**
	 * Controls how this panel is viewed/laid out
	 * 
	 */
	private void initPanel() {
		GridBagConstraints gbc = new GridBagConstraints();
		// int col = 0;
		final ConstraintFactory f = new ConstraintFactory(0, 0, 0, 0, 2);
		this.setLayout(new GridBagLayout());

		gbc = f.getConstraint(0, 0, ConstraintFactory.FIELD,
				ConstraintFactory.MIDDLE);
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		this.add(fieldScroller, gbc);

		gbc = f.getConstraint(1, 0, ConstraintFactory.LABEL,
				ConstraintFactory.MIDDLE);
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		this.add(popupTriggerButton, gbc);
	}

	/**
	 * This method should be used to update what is shown in the menu that pops
	 * up when the user clicks on the button. This is used when the data in the
	 * list changes, or items get added. Rebuilds the popup menu
	 * 
	 * 
	 */
	public void refreshMenu() {
		initMenu(dataForMenu, dataToInsert, popupTriggerButton.getText());
	}

	/**
	 * Sets the text on the button in this panel
	 * 
	 * @param text
	 *            the text that will be shown on the button
	 */
	public void setButtonText(String text) {
		popupTriggerButton.setText(text);
	}

	/**
	 * Gets the text that is being shown in the textfield and returns it
	 * 
	 * @return the text that is shown in the textfield
	 */
	public String getText() {
		return textField.getText();
	}

	/**
	 * Sets the text in the text field
	 * 
	 * @param text
	 *            the test to display in the text field
	 */
	public void setText(String text) {
		// you might need to process this string to extract out the
		// objects to put into the selected list
		textField.setText(text);
		String[] splitString = text.split("(, )");
		Object[] selectedObjects = new Object[splitString.length];

		// System.out.println(dataToInsert);

		for (int i = 0; i < splitString.length; i++) {
			/*
			 * System.out.println(splitString[i]);
			 * System.out.println(dataToInsert.indexOf(splitString[i]));
			 * System.out.println(dataToInsert.get(0).getClass());
			 */
			Object temp = getObjectFromString(splitString[i]);
			selectedObjects[i] = temp;
		}
		try {
			setSelectedObjects(selectedObjects);
		} catch (ObjectNotFoundException e) {
			// can safely ignore since we just found the objects ourselves.
		}
	}

	private Object getObjectFromString(String s) {
		return stringToObjectMap.get(s);
	}

	/**
	 * Sets the list of selected objects, and loads them into the text field.
	 * The objects contained in the list have to be within the set of data that
	 * is displayed in this object's menu
	 * 
	 * @param objects
	 *            the objects to show in the text field
	 * @throws ObjectNotFoundException
	 *             this is thrown when one of the objects in the array don't
	 *             exist in the menu
	 */
	private void setSelectedObjects(Object[] objects)
			throws ObjectNotFoundException {
		selected.clear();
		// StringBuffer text = new StringBuffer();

		for (int i = 0; i < objects.length; i++) {
			selected.add(objects[i]);
			/*
			 * try {
			 * text.append(dataToInsert.get(dataForMenu.indexOf(objects[i]))
			 * .toString() + ", "); } catch (ArrayIndexOutOfBoundsException e) {
			 * throw new ObjectNotFoundException(); }
			 */
		}
	}

	/*
	 * Sets the list of selected objects, and loads them into the text field.
	 * 
	 * @param objects the objects to show in the text field
	 * 
	 * @throws ObjectNotFoundException this is thrown when one of the objects in
	 * the array don't exist in the menu
	 * 
	 * public void setSelectedObjects(List objects) throws
	 * ObjectNotFoundException { setSelectedObjects(objects.toArray()); }
	 */

	/**
	 * Returns whether or not this field has been modified
	 * 
	 * @return true if this has been modified, false otherwise
	 */
	public boolean isModified() {
		return modified;
	}

	public Object[] getSelectedObjects() {
		return selected.toArray();
	}

	/**
	 * Adds a custom selected listener to the menu. This controls what happens
	 * when an item is selected from the list
	 * 
	 * @param selectedListener
	 *            the listener that controls what will happen when the user
	 *            clicks on an option in the menu
	 */
	public void addSelectionListener(ActionListener selectedListener) {
		menu.removeActionListener(selectionListener);
		menu.addActionListener(selectedListener);
	}

	/**
	 * Adds an undoManager listener to the text field so that edits can be
	 * undone
	 * 
	 * @param undoListener
	 *            the listener for undoable edits to be added
	 */
	public void addUndoableEditListener(UndoableEditListener undoListener) {
		textField.getDocument().addUndoableEditListener(undoListener);
	}

	/**
	 * Removes the specified undoListener from the text field
	 * 
	 * @param undoListener
	 *            the listener to remove
	 */
	public void removeUndoableEditListener(
			final UndoableEditListener undoListener) {
		textField.getDocument().removeUndoableEditListener(undoListener);
	}

	/**
	 * Adds a focus listener to the text field
	 * 
	 * @param fl
	 *            the focus listener to add
	 * 
	 */
	public void addFocusListener(FocusListener fl) {
		if (textField != null) {
			textField.addFocusListener(fl);
		}
	}

	/**
	 * Adds an item to the menu, as well as the data behind this object
	 * 
	 * @param newItem
	 *            The new object to add
	 * @param stringToInsert
	 *            The string representation of the object to insert
	 */
	public void addItem(Object newItem, String stringToInsert) {
		dataForMenu.add(newItem);

		if (!stringToInsert.equals(newItem)) {
			dataToInsert.add(stringToInsert);
		} else {
			dataToInsert.add(newItem);
		}

		initMenu(dataForMenu, dataToInsert, popupTriggerButton.getText());
	}

	/**
	 * adds the document listener
	 * */
	public void addDocumentListener(DocumentListener docListener) {
		textField.getDocument().addDocumentListener(docListener);
	}

	/**
	 * @return the singleMode
	 */
	public boolean isSingleMode() {
		return singleMode;
	}

	/**
	 * @param singleMode
	 *            the singleMode to set
	 */
	public void setSingleMode(boolean singleMode) {
		this.singleMode = singleMode;
	}

}
