package airspaceGUI.expansion;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.Box.Filler;
import javax.swing.event.ChangeEvent;

/**
 * Defines the behavior and look of all find windows in the airspaceGUI expansion.
 * This also abstracts out common factors to all find windows, all find windows
 * should subclass this class
 * 
 * @author srbennet
 * 
 */
public abstract class FindWindow extends JPanel implements ListCellRenderer
{
    /**
     * The button to execute the find search
     */
    protected JButton findButton = new JButton("Find");

    /**
     * Cancels the find, banishing the find window
     */
    protected JButton cancelButton = new JButton("Cancel");

    /**
     * Panel to specify the parameters of the search
     */
    protected ParameterPanel parameterPanel;

    /**
     * The panel this is searching on
     */
    protected JComponent parent;

    /**
     * Spinny busy indicator to tell the user the program is working
     */
    private SmallBusyIndicator busyIndicator = new SmallBusyIndicator();

    /**
     * the visual representation of the List of possible objects the user could
     * be looking for
     * 
     */
    JList possibilitiesList = new JList();

    /**
     * Yeild the interface to the main run thread
     */
    private final Runnable closerRunner = new Runnable()
    {
        public void run() {
            Thread.yield();
        }
    };

    /**
     * The action to be performed when the find button is pressed
     */
    private ActionListener findListener = new ActionListener()
    {

        public void actionPerformed(ActionEvent arg0) {
            busyIndicator.startSpinning();
            resultsLabel.setVisible(false);
            FindWindow.this.validate();
            final Thread thisThread = new Thread("BusyThread")
            {
                private ActionListener yieldListener = new ActionListener()
                {

                    public void actionPerformed(ActionEvent arg0) {
                        Thread.yield();
                    }

                };

                public void run() {
                    // try
                    {
                        // need to synchronize the panel to make sure the gui
                        // doesn't get
                        // into an inconsistent state
                        synchronized (FindWindow.this)
                        {
                            Timer timer = new Timer(100, yieldListener);
                            timer.start();
                            try
                            {
                                sleep(100);
                            } catch (InterruptedException e)
                            {
                                e.printStackTrace();
                            }
                            ArrayList possibilities;

                            if (parent != null)
                            {
                                parent
                                        .setCursor(Cursor
                                                .getPredefinedCursor(Cursor.WAIT_CURSOR));
                            }
                            possibilities = search(parameterPanel
                                    .getSearchTerms());

                            if (possibilities == null
                                    || possibilities.isEmpty())
                            {
                                possibilitiesScroll.setVisible(false);
                                SwingUtilities.invokeLater(new Thread()
                                {
                                    public void run() {
                                        JOptionPane
                                                .showMessageDialog(
                                                        null,
                                                        "No Matches",
                                                        "Note",
                                                        JOptionPane.INFORMATION_MESSAGE);
                                    }
                                });
                                parameterPanel.highlightFirst();
                            } else
                            {
                                if (possibilities.size() == 1)
                                {
                                    openList(possibilities);
                                    updatePanel(possibilities.get(0));
                                    // dispose();
                                } else
                                {
                                    // need to get a JListBox in a frame on here
                                    openList(possibilities);
                                }
                                resultsLabel.setText(possibilities.size() + " results");
                                resultsLabel.setVisible(true);
                            }
                            // Thread.yield();
                            SwingUtilities.invokeLater(new Thread()
                            {
                                public void run() {
                                    FindWindow.this.validate();
                                    ((JFrame)FindWindow.this.getTopLevelAncestor()).pack();
                                }
                            });
                            parent
                                    .setCursor(Cursor
                                            .getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                            // busyIndicator.startSpinning();
                            busyIndicator.stopSpinning();
                            timer.stop();
                        }
                        SwingUtilities.invokeLater(closerRunner);
                    }
                }
            };
            thisThread.start();
        }

    };

    /**
     * Cancels the find operation, hiding and freeing the window
     */
    ActionListener cancelListener = new ActionListener()
    {

        public void actionPerformed(ActionEvent arg0) {
            // TODO Auto-generated method stub
            getTopLevelAncestor().setVisible(false);
        }

    };

    private JScrollPane possibilitiesScroll;

    protected JTabbedPane tabs = null;
    
    private JLabel resultsLabel = new JLabel();

    public FindWindow()
    {
        initWindow(null);
    }

    public FindWindow(Object[] terms)
    {
        ArrayList searchTerms = new ArrayList();

        for (int i = 0; i < terms.length; i++)
        {
            searchTerms.add(terms[i]);
        }
        initWindow(searchTerms);
    }

    public FindWindow(ArrayList terms)
    {
        initWindow(terms);
    }

    public FindWindow(Object[] terms, JTabbedPane tab)
    {
        ArrayList searchTerms = new ArrayList();

        for (int i = 0; i < terms.length; i++)
        {
            searchTerms.add(terms[i]);
        }
        initWindow(searchTerms);
        this.tabs = tab;
    }

    /**
     * Initializes the window with all the controls
     * 
     * @param terms
     *            The data to search over
     */
    private void initWindow(List terms) {
        GridBagConstraints gbc = new GridBagConstraints();
        int x = 0, y = 0;
        Dimension min = new Dimension(0,0);
        Dimension max = new Dimension(this.getSize());
        setLayout(new GridBagLayout());
        //this.setDefaultButton(findButton);

        gbc.insets = new Insets(8, 8, 8, 8);
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridx = x++;
        gbc.gridwidth = 3;
        gbc.weightx = 1.0;
        add((parameterPanel = new ParameterPanel(false, terms)), gbc);

        // reset the x counter since we're moving to a new y-position
        x = 0;
        gbc.insets = new Insets(4, 4, 4, 4);
        gbc.fill = GridBagConstraints.NONE;
        gbc.gridx = x++;
        gbc.gridwidth = 1;
        gbc.weightx = 1.0;
        add(busyIndicator, gbc);
        resultsLabel.setVisible(false);
        add(resultsLabel, gbc);
        
        gbc.weighty = 1.0;
        gbc.weightx = 0.75;
        gbc.fill = GridBagConstraints.BOTH;
        add(new Filler(min, max, max), gbc);


        gbc.anchor = GridBagConstraints.NORTHEAST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.insets = new Insets(0, 0, 8, 0);
        gbc.gridwidth = 1;
        gbc.weighty = 0.0;
        gbc.gridx = x++;
        gbc.gridy = ++y;
        gbc.weightx = 1.0;
        cancelButton.addActionListener(cancelListener);
        add(cancelButton, gbc);

        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.gridx = x++;
        findButton.addActionListener(findListener);
        add(findButton, gbc);

        possibilitiesScroll = new JScrollPane(possibilitiesList);
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.insets = new Insets(0, 0, 14, 0);
        gbc.gridwidth = x;
        gbc.gridy = ++y;
        gbc.gridx = 0;
        add(possibilitiesScroll, gbc);
        possibilitiesScroll.setVisible(false);
        
        gbc.weighty = 1;
        gbc.weightx = 1;
        add(new Filler(min, max, max), gbc);

        //pack();
    }

    /**
     * Opens a listbox with all the possibilities so that the user can choose
     * which of the objects they wish to view/edit
     * 
     * @param possibilities
     *            The list of possible things the user could have meant.
     */
    private void openList(ArrayList possibilities) {
        possibilitiesList.setCellRenderer(this);
        if (possibilities != null)
        {
            possibilitiesList.setListData(possibilities.toArray());
        }
        possibilitiesList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        if (possibilitiesList.getModel().getSize() > 0)
        {
            possibilitiesList.setSelectedIndex(0);
            Thread.yield();
        }
        possibilitiesList.setVisible(true);
        possibilitiesScroll.setVisible(true);
        // ensure the scroll pane is atLabel least this large
        possibilitiesScroll.setSize(350, 150);

        /*
         * add a listener to be activated whan an item is being clicked on so
         * that the main panel will appear/change appropriately
         */
        possibilitiesList.addMouseListener(new MouseListener()
        {

            public void mouseClicked(MouseEvent arg0) {
                // TODO Auto-generated method stub
                if (arg0.getClickCount() == 2)
                {
                    updatePanel(possibilitiesList.getSelectedValue());
                    parent.requestFocusInWindow();
                    new ChangeEvent(parent);
                    // FindWindow.this.dispose();
                }
            }

            public void mousePressed(MouseEvent arg0) {
                return;
            }

            public void mouseReleased(MouseEvent arg0) {
                return;
            }

            public void mouseEntered(MouseEvent arg0) {
                return;
            }

            public void mouseExited(MouseEvent arg0) {
                return;
            }
        });

        //this.pack();
        this.validate();
        Thread.yield();
    }

    /**
     * Resets the search terms to search with to the list in the parameter
     * 
     * @param searchTerms
     *            The new list of search terms to search with
     */
    public void setSearchTerms(List searchTerms) {
        initWindow(searchTerms);
    }
    
    /**
     * Updates this find windows's search list with teh list provided
     * @param search
     */
    public abstract void updateSearchList(List search); 
    
    /*public void setLocationRelativeTo(JComponent component){
        ((Frame)getParent()).setLocationRelativeTo(component);
    }*/

    /**
     * Options have been found, update the panel's interface with the parameter
     * object
     * 
     * @param object
     *            The opject to show on the GUI
     */
    protected abstract void updatePanel(Object object);

    /**
     * Searches the data list to find all possible pieces of data the user could
     * be looking for
     * 
     * @param searchTerms
     *            The terms over which to search
     * @return A list of the solutions
     */
    protected abstract ArrayList search(Map searchTerms);
    
    public JButton getDefaultButton()
    {
        return findButton;
    }
}
