/**
 * 
 */
package airspaceGUI.expansion;

import javax.swing.JPanel;

import org.hibernate.Session;

/**
 * This is an interface of utilities that all panels on the expanded version of
 * airspaceGUI needs to implement, especially if they are not subclassing BasePanel.
 * 
 * @author srbennet
 * 
 */
public interface PanelUtils
{
    /**
     * Loads the necessary objects into the gui.
     * @param session -
     *            an initialized Hibernate Session that is used to recover all
     *            instances of the object(s) this panel represents out of the
     *            database
     * @return true if the objects were loaded, false otherwise.
     */
    public boolean load(Session session);

    /**
     * This function saves the objects behind this panel back to the database
     * 
     * @return true if it was a successful save, false otherwise
     */
    public boolean save();

    /**
     * This function creates/initializes the implementing JPanel, and returns it
     * to be added to the JTabbedPane
     * 
     * @return the JPanel to be added to the JTabPane.
     */
    public JPanel getPane();
    
    /**
     * This method is what is called when there is ab object to be loaded onto this panel.
     * @param session A session from which to get the data to fill in if necessary
     *
     */
    public void fillInText(Session session);
    
    /**
     * This method is used to navigate to a specific message
     * @param number
     */
    public void gotoNumber(int number);

    /**
     * This returns the title of the panel that is showing
     * @return The text that should be shown in the titlebar
     */
    public String getTitle();

}
