package airspaceGUI.expansion;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;

import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.JWindow;
import javax.swing.SwingUtilities;

/**
 * An implementation of a Splash Window to show while an application is loading.
 * 
 * @author srbennet
 * 
 */
public class Splash extends JWindow
{
    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    protected volatile boolean done = false;

    private volatile Thread thisThread;

    /*
     * public static void main(String[] args) { Splash splash = new Splash();
     * splash.show(); invokeMain(CorpusGUITest.class, null); splash.done = true;
     * splash.thisThread.notify(); }
     */
    protected volatile JProgressBar progress = new JProgressBar();

    /**
     * Note: the progress bar is indexed atLabel a maximum of 100% done.
     * 
     */
    public Splash()
    {
        super();
        getContentPane().setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(4, 4, 8, 4);
        gbc.anchor = GridBagConstraints.WEST;
        JLabel l = new JLabel("Loading data... Please Wait.");
        getContentPane().add(l, gbc);

        progress.setString("Loading...");
        progress.setStringPainted(true);
        progress.setMaximum(100);
        progress.setIndeterminate(true);
        gbc.gridy = 1;
        gbc.insets = new Insets(4, 4, 4, 4);
        gbc.fill = GridBagConstraints.BOTH;
        getContentPane().add(progress, gbc);
        pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension labelSize = l.getPreferredSize();
        setLocation(screenSize.width / 2 - (labelSize.width / 2),
                screenSize.height / 2 - (labelSize.height / 2));
        final Runnable closerRunner = new Runnable()
        {
            public void run() {
                setVisible(false);
                dispose();
                // thisThread.destroy();
            }
        };
        thisThread = new Thread("SplashThread")
        {
            public void run() {
                try
                {
                    synchronized (Splash.this)
                    {
                        while (!done)
                        {
                            Splash.this.wait();
                        }
                        SwingUtilities.invokeLater(closerRunner);
                    }

                } catch (InterruptedException e)
                {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } /*
                     * catch (InvocationTargetException e) { // TODO
                     * Auto-generated catch block e.printStackTrace(); }
                     */
            }
        };

        setVisible(true);
        thisThread.setDaemon(false);//true);
        // thisThread = new Thread(waitRunner, "SplashThread");
        thisThread.start();
    }

    /**
     * Removes the splash screen from view and disposes of it.
     */
    public void removeSplash() {
        if (this.isVisible())
        {
            setVisible(false);
            dispose();
        }
    }

    /**
     * This updates the progress of the application in loading.
     * 
     * @param percentMoreDone
     *            the additional percent that has been acocmplished since the
     *            last update, based on a 100 = done scale
     */
    public void updateProgress(int percentMoreDone) {
        synchronized (this)
        {
            progress.setValue(progress.getValue() + percentMoreDone);
            progress.setIndeterminate(false);
        }
    }

    /**
     * This sets whether the application is done loading or not.
     * 
     * @param b
     *            the status of the application, true for done laoding, false
     *            for still loading
     */
    public void setDone(boolean b) {
        synchronized (this)
        {
            done = b;
        }
    }

}
