package airspaceGUI.expansion;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.text.DateFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.StringTokenizer;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerListModel;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeListener;

/**
 * This defines the GUI representation of a general time panel
 * 
 * @author srbennet
 * 
 */
public class TimePanel extends JPanel
{
    /**
     * This is the constant used to represent a normal message timestamp
     * formatted time.
     */
    public static final String MESSAGETIME = "Message timestamp";

    /**
     * Constant used to indicate the time will be delievered in an all numbers
     * format
     */
    public static final String ABREVIATEDTIME = "All numbers";

    // set up models for the time spinners
    private transient final Integer[] yearsList = {new Integer(2003),
            new Integer(2004), new Integer(2005), new Integer(2006),
            new Integer(2007), new Integer(2008), new Integer(2009),
            new Integer(2010), new Integer(2011), new Integer(2012),
            new Integer(2013), new Integer(2014), new Integer(2015)};

    /**
     * below are models behind the teim fields
     */
    private final transient SpinnerModel years = new SpinnerListModel(Arrays
            .asList(yearsList));

    private final transient SpinnerNumberModel hours = new SpinnerNumberModel(
            1, 1, 12, 1);

    private final transient SpinnerNumberModel minutes = new SpinnerNumberModel(
            0, 0, 59, 1);

    private final transient SpinnerNumberModel backDays = new SpinnerNumberModel(
            1, 1, 31, 1);

    private final transient String[] months = new DateFormatSymbols()
            .getShortMonths();

    private final transient SpinnerModel backMonths = new SpinnerListModel(
            Arrays.asList(months));

    private final transient String[] ampm = new DateFormatSymbols()
            .getAmPmStrings();

    private final transient SpinnerModel amPm = new SpinnerListModel(Arrays
            .asList(ampm));

    // initialize all the time spinners
    private final transient JSpinner month = new JSpinner(backMonths);

    private final transient JSpinner day = new JSpinner(backDays);

    private final transient JSpinner hour = new JSpinner(hours);

    private final transient JSpinner minute = new JSpinner(minutes);

    private final transient JSpinner timeOfDay = new JSpinner(amPm);

    private final transient JSpinner year = new JSpinner(years);

    public TimePanel()
    {
        super();
        layoutPanel();

    }

    /**
     * Sets up the TimePanel, layout out all the components in the correct places
     */
    private void layoutPanel() {
        this.setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        int col = 0;
        ConstraintFactory f = new ConstraintFactory(0, 0, 0, 0, 2);
        JLabel colonLabel = new JLabel(" :");
        JLabel commaLabel = new JLabel(",");

        gbc = f.getConstraint(col++, 0, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        this.add(month, gbc);
        gbc = f.getConstraint(col++, 0, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        this.add(day, gbc);
        gbc = f.getConstraint(col++, 0, ConstraintFactory.LABEL,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        // gbc.insets = new Insets(0, 0, 0, -8);
        this.add(commaLabel, gbc);
        gbc = f.getConstraint(col++, 0, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        this.add(year, gbc);
        gbc = f.getConstraint(col++, 0, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        gbc.insets = new Insets(0, 16, 0, 0);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        this.add(hour, gbc);
        gbc = f.getConstraint(col++, 0, ConstraintFactory.LABEL,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        // gbc.insets = new Insets(8, 0, 8, 0);
        this.add(colonLabel, gbc);
        gbc = f.getConstraint(col++, 0, ConstraintFactory.FIELD,
                ConstraintFactory.MIDDLE);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        this.add(minute, gbc);
    }

    /**
     * Sets the text of this panel to be the time passed into it by the sting
     * 
     * @param time
     *            The time to display
     * @param timeType
     *            The type of the time sting, e.g. what format the time sting is
     *            in (either MESSAGETIME or ABREVIATEDTIME)
     */
    public void setText(String time, String timeType) {
        StringTokenizer timeTokens = new StringTokenizer(time, ": ");
        String min;

        //System.out.println(time);

        if (timeType.equals(MESSAGETIME))
        {
            month.setValue(timeTokens.nextToken());
            day.setValue(new Integer(timeTokens.nextToken()));
            hour.setValue(new Integer(timeTokens.nextToken()));
            min = timeTokens.nextToken();
            minute.setValue(new Integer(min.substring(0, 2)));
            timeOfDay.setValue(min.substring(2, 4).toUpperCase());
            year.setValue(new Integer(timeTokens.nextToken()));
        } else if (timeType.equals(ABREVIATEDTIME))
        {
            // TOOD figure out if this is needed
        }
    }

    /**
     * Adds a change listener to all the fields on this panel so that the
     * listner can be notified of all updates
     * 
     * @param listener
     *            The listener that wishes to listen for all updates to this
     *            panel
     */
    public void addChangeListener(ChangeListener listener) {
        month.addChangeListener(listener);
        day.addChangeListener(listener);
        year.addChangeListener(listener);
        minute.addChangeListener(listener);
        hour.addChangeListener(listener);
        timeOfDay.addChangeListener(listener);
    }

    /**
     * Sets the interface to the default time
     * 
     */
    public void clearTime() {
        backDays.setValue(new Integer(7));
        hour.setValue(new Integer(1));
        minute.setValue(new Integer(00));
        timeOfDay.setValue("AM");
        year.setValue(new Integer(2005));
        month.setValue("Jun");
    }

    /**
     * Returns the time in string format
     * 
     * @param timeType
     *            the format of the time string, either MESSAGETIME or
     *            ABREVIATEDTIME
     * @return a correctly formated text string
     */
    public String getTime(String timeType) {
        if (timeType.equals(MESSAGETIME))
        {
            String min;
            if (new Integer(10).compareTo((Integer)minute.getValue()) > 0)
            {
                min = "0" + minute.getValue();
            } else
            {
                min = (((Integer)minute.getValue())).toString();
            }

            return (month.getValue() + " " + day.getValue() + " "
                    + hour.getValue() + ":" + min + timeOfDay.getValue() + " " + year
                    .getValue());
        } else if (timeType.equals(ABREVIATEDTIME))
        {
            // TODO figure out if this is needed
            return null;
        }
        return null;
    }

    /**
     * Sets the date fields to the specified date
     * 
     * @param date
     *            the date to show in the fields
     */
    public void setDate(Date date) {
        if (date != null)
        {
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MMM-dd");
            String dateString = df.format(date);
            StringTokenizer dateTokens = new StringTokenizer(dateString, "- ");
            //System.out.println("Setting the date " + dateString);

            String yearString = (String)dateTokens.nextElement();
            //System.out
            //        .println(dateString + " the year string is " + yearString);
            year.setValue(new Integer(yearString));

            SpinnerListModel model = (SpinnerListModel)month.getModel();
            String monthString = dateTokens.nextToken();
            // need to translate the numerical represenation of the month into a
            // string representation for the spinner
            // minus 1 because the list is indexed from 0 and the months are
            // indexed from 1
            model.setValue(monthString);

            Integer dayoMonth = new Integer((String)dateTokens.nextElement());
            //System.out.println("The day of the month is " + dayoMonth);
            day.setValue(dayoMonth);
        }
    }

    /**
     * Sets the time fields to the specified time
     * 
     * @param time
     *            the time to show in the fields
     */
    public void setTime(Date time) {
        if (time != null)
        {
            SimpleDateFormat df = new SimpleDateFormat("HH:mm:ss");

            String timeString = df.format(time);
            StringTokenizer timeTokens = new StringTokenizer(timeString, ":");
            //System.out.println("Setting the Time " + timeString);
            hour.setValue(new Integer(timeTokens.nextToken()));
            minutes.setValue(new Integer(timeTokens.nextToken()));
        }
    }

    /**
     * Returns the date portion of the time displayed on this panel
     * 
     * @return The date that is displayed on the interface
     */
    public Date getDate() {
        if (year.getValue().equals(new Integer(2003))
                && month.getValue().equals("Jan")
                && day.getValue().equals(new Integer(1)))
        {
            return null;
        } else
        {
            SpinnerListModel slm = (SpinnerListModel)month.getModel();
            Calendar c = Calendar.getInstance();
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");

            c.set(((Integer)year.getValue()).intValue(), slm.getList().indexOf(
                    month.getValue()), ((Integer)day.getValue()).intValue());

            /*System.out.println(df.getTimeZone()
                    + " Time zone.\n The formatted string is "
                    + df.format(c.getTime()));
                    */
            return c.getTime();
        }
    }

    /**
     * Used to get the time portion of the data displayed on this panel.
     * 
     * @return the time that the panel is currently displaying
     */
    public Date getTime() {
        if (hour.getValue().equals(new Integer(1))
                && minute.getValue().equals(new Integer(0)))
        {
            return null;
        } else
        {
            SpinnerListModel slm = (SpinnerListModel)month.getModel();
            Calendar c = Calendar.getInstance();
            c.set(((Integer)year.getValue()).intValue(), slm.getList().indexOf(
                    month.getValue()), ((Integer)day.getValue()).intValue(),
                    ((Integer)hour.getValue()).intValue(), ((Integer)minute
                            .getValue()).intValue(), 0);
            return c.getTime();
        }
    }

}
