package openinjector;

/*import java.sql.Connection;
 import java.sql.DriverManager;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;*/
import java.util.*;
import java.io.*;
import javax.xml.parsers.*; //import java.sql.*;
import org.w3c.dom.*;
import org.xml.sax.SAXParseException; //import org.xml.sax.*;
import javax.swing.JOptionPane;

/**
 * This class is used to load rMessages into a vector
 * 
 * @author srbennet
 * 
 */
public class MessageLoader {

	protected static final String ERROR_BOX_TITLE = "Error";
	private static final String MESSAGE_BUG_WARNING = "There appears to be a bug in message number ";
	protected static final String ENTER_VALID_VARIABLE = "\n Please enter a valid e-mail or variable";
	protected static final String STRINGTRUE = "true";

	/**
	 * A function to load messages from the file given in path This used to be
	 * the main function, but needed to be called something else so that the GUI
	 * could call it without a problem
	 * 
	 * @param path
	 *            the path to the file to load
	 * @return an ArrayList containing all the messages in the file
	 */
	public static ArrayList loadMessage(final String path) {
		final RMessage mess = new RMessage();
		try {
			// create new document
			final DocumentBuilderFactory docBF = DocumentBuilderFactory
					.newInstance();

			final DocumentBuilder docB = docBF.newDocumentBuilder();
			Document doc = null;
			// create new DOM document of xml file
			try {
				// parse the file
				doc = docB.parse(new File(path));
			} catch (SAXParseException ex) {
				// the file could not be parsed
				ex.printStackTrace();
				JOptionPane.showMessageDialog(null, "XML syntax error. \n"
						+ ex.getMessage(), "Cannot load file: ",
						JOptionPane.ERROR_MESSAGE);
				return null;
			} catch (FileNotFoundException e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, e.getLocalizedMessage(),
						"Cannot load file", JOptionPane.ERROR_MESSAGE);
				return null;
			} catch (Exception e) {
				// either the file cannot be parsed for some other reason
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, "XML syntax error",
						"Cannot load file", JOptionPane.ERROR_MESSAGE);
				return null;
			}

			// getting what is needed, parsing out each author/publication node
			final NodeList emails = doc.getElementsByTagName("email");
			final ArrayList messList = new ArrayList();

			for (int i = 0; i < emails.getLength(); i++) {
				// For each email in the xml file, create a "real" email
				messList.add(RMessage.getEmail(emails.item(i)));
			}
			if (valid(messList)) { // ensure that all messages have required
									// components
				// returns the Vector of messages to the calling object
				return messList;
			} else {
				return null;
			}

		} // end of try
		catch (Exception e) {
			e.printStackTrace();
			if (mess != null) {
				JOptionPane
						.showMessageDialog(
								null,
								"Cannot open file. \n There appears to be a bug in the file ",
								ERROR_BOX_TITLE, JOptionPane.ERROR_MESSAGE);
			} else {
				JOptionPane.showMessageDialog(null, "Cannot open file.",
						ERROR_BOX_TITLE, JOptionPane.ERROR_MESSAGE);
			}

		}

		// if something fails, then return null
		return null;
	}

	/**
	 * A function to load messages from the file given in the parameter.
	 * 
	 * @param file
	 *            the file to load
	 * @return an ArrayList containing all the messages in the file
	 */
	public static ArrayList loadMessage(File file) {
		return loadMessage(file.getPath());
	}

	/**
	 * Function used to convert from xml format to normal string format. This
	 * replaces all instances of &lt; to <, &gt; to >, and amp; to &
	 * 
	 * @param s
	 *            The string to transform
	 * @return The string without those character codes
	 */
	protected static String revalidate(final String s) {
		String string;
		string = s.replaceAll("amp;", "&");
		string = string.replaceAll("&lt;", "<");
		string = string.replaceAll("&gt;", ">");
		return string;
	}

	/**
	 * This is a function used to validate that each message in the list has the
	 * correct parts to it
	 * 
	 * @param messList
	 *            The list of messages to validate
	 * @return true if the list is valid, false otherwise
	 */
	private static boolean valid(final ArrayList messList) {
		for (int i = 0; i < messList.size(); i++) {
			final RMessage current = (RMessage) messList.get(i);
			if (current.getBody() == null || current.getBody().equals("")) {
				// not valid, has no body
				JOptionPane.showMessageDialog(null, MESSAGE_BUG_WARNING
						+ current.getId()
						+ "Invalid e-mail message: No \"body\" field",
						ERROR_BOX_TITLE, JOptionPane.ERROR_MESSAGE);
				return false;
			}

			if (current.getSubject().equals("")) {
				// not valid, has not subject
				JOptionPane.showMessageDialog(null, MESSAGE_BUG_WARNING
						+ current.getId()
						+ "Invalid message: no \"subject\" field",
						ERROR_BOX_TITLE, JOptionPane.ERROR_MESSAGE);
				return false;
			}

			if (current.getFrom().equals("")) {
				// not valid, has no sender
				JOptionPane.showMessageDialog(null, MESSAGE_BUG_WARNING
						+ current.getId()
						+ "\n Invalid message: no \"from\" field",
						ERROR_BOX_TITLE, JOptionPane.ERROR_MESSAGE);
				return false;
			}

			if (current.getGroup().equals("")) {
				// not valid, has no group
				JOptionPane.showMessageDialog(null, MESSAGE_BUG_WARNING
						+ current.getId()
						+ "Invalid message: no \"group\" field",
						ERROR_BOX_TITLE, JOptionPane.ERROR_MESSAGE);
				return false;
			}

			if (current.getTime().equals("")) {
				// not valid, has no time
				JOptionPane.showMessageDialog(null, MESSAGE_BUG_WARNING
						+ current.getId()
						+ "Invalid message: no \"time\" field",
						ERROR_BOX_TITLE, JOptionPane.ERROR_MESSAGE);
				return false;
			}

			if (current.getTo().isEmpty()) {
				// not valid, has no recipient
				JOptionPane
						.showMessageDialog(
								null,
								"Invalid message: no \"to\" field \n There appears to be a bug in message number "
										+ current.getId(), ERROR_BOX_TITLE,
								JOptionPane.ERROR_MESSAGE);
				return false;
			}
		}
		return true;
	}

} // end of class
