package openinjector;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import relativelayout.Binding;
import relativelayout.Direction;
import relativelayout.Edge;
import relativelayout.RelativeConstraints;
import relativelayout.RelativeLayout;
import relativelayout.RelativePosition;

/**
 * This class is used to present a more user-friendly, attractive front to the
 * new Injector, to make it more useable
 * 
 * @author srbennet
 * 
 */
public class OpenInjectorGUI extends JFrame
{

    /**
     * Specification for what the program does, and what OutputFiles takes as
     * parameters 
     * -i recipientCheck [sends injected messages] 
     * -s server:port [server to send the injected messages through (optional)] 
     * -m [turns on Unix 'mbox' mode] 
     * -w [turns on Windows 'eml' mode]\ 
     * -c [turns on 'cheat' mode] 
     * -l [turns on 'label' mode] 
     * -d dayCheck [set which day - default is 1] 
     * -p len [set which session - default is 1] 
     * -r len [turn on 'wRap' mode default is off] 
     * -v [turns on 'verbose' mode]
     */
    /**
     * checl boxes, labels and fields that make up the GUI interface
     */
    private JCheckBox mboxCheck = new JCheckBox("Export unix mbox messages");

    private JCheckBox emlCheck = new JCheckBox("Export windows eml messages");

    private JCheckBox cheatCheck = new JCheckBox(
            "Export all XRADAR learning tags");

    private JCheckBox labelCheck = new JCheckBox("Export message labels");

    private JCheckBox dayCheck = new JCheckBox(
            "Change the day on which the messages are sent");
    
    private JTextField dayField = new JTextField();
    
    private JLabel dayLabel = new JLabel("day(s)");

    private JCheckBox sessionCheck = new JCheckBox("Change the session");
    
    private JTextField sessionField = new JTextField();
    
    private JLabel sessionLabel = new JLabel("session");

    private JCheckBox wrappingCheck = new JCheckBox("Wrap lines in the message");

    private JCheckBox recipientCheck = new JCheckBox(
            "Specify an external recipient");

    private JLabel addressLabel = new JLabel("Address:");

    private JTextField usernameField = new JTextField();

    private JLabel atLabel = new JLabel("@");

    private JTextField urlField = new JTextField();

    private JCheckBox serverCheck = new JCheckBox(
            "Specify a server to send the message");

    private JLabel serverLabel = new JLabel("Server:");

    private JTextField serverNameField = new JTextField();

    private JLabel portLabel = new JLabel("Port:");

    private JTextField serverPortField = new JTextField();

    /**
     * ItemListener to control whether the recipient fields are visible or not
     * based on recipientCheck
     */
    private ItemListener recipientListener = new ItemListener()
    {

        public void itemStateChanged(ItemEvent arg0) {
            atLabel.setVisible(!atLabel.isVisible());
            usernameField.setVisible(!usernameField.isVisible());
            urlField.setVisible(!urlField.isVisible());
        }

    };

    /**
     * 
     * ItemListener to control whether the server fields are visible or not
     * based on serverCheck
     */
    private ItemListener serverListener = new ItemListener()
    {

        public void itemStateChanged(ItemEvent e) {
            serverLabel.setVisible(!serverLabel.isVisible());
            portLabel.setVisible(!portLabel.isVisible());
            serverNameField.setVisible(!serverNameField.isVisible());
            serverPortField.setVisible(!serverPortField.isVisible());
        }

    };

    /**
     * ItemListener to control whether the line wrapping fields are visible or
     * not based on wrapCheck
     */
    private ItemListener wrapListener = new ItemListener()
    {

        public void itemStateChanged(ItemEvent e) {
            wrapLengthField.setVisible(!wrapLengthField.isVisible());
            lengthLabel.setVisible(!lengthLabel.isVisible());
        }

    };
    
    private ItemListener dayListener = new ItemListener(){

        public void itemStateChanged(ItemEvent arg0) {
            dayField.setVisible(!dayField.isVisible());
            dayLabel.setVisible(!dayLabel.isVisible());
        }
        
    };
    
    private ItemListener sessionListener = new ItemListener(){

        public void itemStateChanged(ItemEvent e) {
            sessionField.setVisible(!sessionField.isVisible());
            sessionLabel.setVisible(!sessionLabel.isVisible());
        }
        
    };

    JPanel buttonPanel = new JPanel();

    JButton sendButton = new JButton("Send");

    JButton cancelButton = new JButton("Cancel");

    private JTextField wrapLengthField = new JTextField();

    private JLabel lengthLabel = new JLabel("characters");
    
    private ActionListener sendButtonListener = new ActionListener(){

        public void actionPerformed(ActionEvent arg0) {
            // TODO Auto-generated method stub
            new Thread(){
            String[] args = OpenInjectorGUI.this.getParameters();
            OutputFiles of = new OutputFiles(args);
            };
        }

    };
    
    private ActionListener cancelButtonListener = new ActionListener(){

        public void actionPerformed(ActionEvent arg0) {
            OpenInjectorGUI.this.setVisible(false);
            OpenInjectorGUI.this.dispose();
        }

    };

    /**
     * @param args
     */
    public static void main(String[] args) {
        OpenInjectorGUI oig = new OpenInjectorGUI();
        oig.setVisible(true);
    }

    /**
     * 
     * Constructs an GUI interface that allows for the selection of options
     * 
     */
    public OpenInjectorGUI()
    {
        super();
        layoutPanel();
        dayCheck.addItemListener(dayListener);
        sessionCheck.addItemListener(sessionListener);
        serverCheck.addItemListener(serverListener);
        recipientCheck.addItemListener(recipientListener);
        wrappingCheck.addItemListener(wrapListener);
        sendButton.addActionListener(sendButtonListener);
        cancelButton.addActionListener(cancelButtonListener);
    }

    private void layoutPanel() {
        this.setLayout(new RelativeLayout());
        //RelativeLayout.setDebug(true);
        Dimension minFieldSize = new Dimension(60, 20);
        this.setResizable(false);
        // Bindings for each edge of the frame
        Binding leftEdge = new Binding(Edge.LEFT, 8, Direction.RIGHT,
                Edge.LEFT, this);
        Binding rightEdge = new Binding(Edge.RIGHT, 8, Direction.LEFT,
                Edge.RIGHT, this);
        Binding topEdge = new Binding(Edge.TOP, 0, Direction.BELOW, Edge.TOP,
                this);
        Binding bottomEdge = new Binding(Edge.BOTTOM, 4, Direction.ABOVE,
                Edge.BOTTOM, this);

        RelativePosition justBelow = new RelativePosition(Edge.TOP, 0,
                Direction.BELOW, Edge.BOTTOM);
        RelativePosition justAbove = new RelativePosition(Edge.BOTTOM, 0,
                Direction.ABOVE, Edge.TOP);
        RelativePosition eightBelow = new RelativePosition(Edge.TOP, 8,
                Direction.BELOW, Edge.BOTTOM);
        RelativePosition twelveBelow = new RelativePosition(Edge.TOP, 12,
                Direction.BELOW, Edge.BOTTOM);
        RelativePosition verticalCenter = new RelativePosition(
                Edge.VERTICAL_CENTER, 0, Direction.ABOVE, Edge.VERTICAL_CENTER);
        RelativePosition eightRightOf = new RelativePosition(Edge.LEFT, 8,
                Direction.RIGHT, Edge.RIGHT);
        RelativePosition eightLeftOf = new RelativePosition(Edge.RIGHT, 8,
                Direction.LEFT, Edge.LEFT);

        RelativeConstraints mBoxConstraints = new RelativeConstraints(
                new Binding[] {leftEdge, topEdge});
        this.add(mboxCheck, mBoxConstraints);

        RelativeConstraints emlConstraints = new RelativeConstraints(
                new Binding[] {leftEdge, new Binding(eightBelow, mboxCheck)});
        this.add(emlCheck, emlConstraints);

        RelativeConstraints cheatConstraints = new RelativeConstraints(
                new Binding[] {leftEdge, new Binding(eightBelow, emlCheck)});
        this.add(cheatCheck, cheatConstraints);

        RelativeConstraints labelConstraints = new RelativeConstraints(
                new Binding[] {leftEdge, new Binding(eightBelow, cheatCheck)});
        this.add(labelCheck, labelConstraints);

        RelativeConstraints dayConstraints = new RelativeConstraints(
                new Binding[] {leftEdge, new Binding(eightBelow, labelCheck)});
        this.add(dayCheck, dayConstraints);
        
        RelativeConstraints dayFieldConstraints = new RelativeConstraints(new Binding[]{
                new Binding(eightRightOf, dayCheck), new Binding(eightBelow, labelCheck), 
                new Binding(eightLeftOf, dayLabel)});
        this.add(dayField, dayFieldConstraints);
        
        RelativeConstraints dayLabelConstraints = new RelativeConstraints(new Binding[]{
                new Binding(eightBelow, labelCheck), rightEdge});
        this.add(dayLabel, dayLabelConstraints);

        RelativeConstraints sessionConstraints = new RelativeConstraints(
                new Binding[] {leftEdge, new Binding(eightBelow, dayCheck)});
        this.add(sessionCheck, sessionConstraints);
        
        RelativeConstraints sessionFieldConstraints = new RelativeConstraints(new Binding[]{
                new Binding(eightRightOf, sessionCheck), new Binding(eightBelow, dayCheck), 
                new Binding(eightLeftOf, sessionLabel)});
        this.add(sessionField, sessionFieldConstraints);
        
        RelativeConstraints sessionLabelConstraints = new RelativeConstraints(new Binding[]{
                new Binding(eightBelow, dayCheck), rightEdge});
        this.add(sessionLabel, sessionLabelConstraints);

        RelativeConstraints wrappingConstraints = new RelativeConstraints(
                new Binding[] {leftEdge, new Binding(eightBelow, sessionCheck)});
        this.add(wrappingCheck, wrappingConstraints);

        RelativeConstraints wrapLengthConstraints = new RelativeConstraints(
                new Binding[] {new Binding(eightRightOf, wrappingCheck),
                        new Binding(verticalCenter, wrappingCheck),
                        new Binding(eightBelow, sessionCheck),
                        new Binding(eightLeftOf, lengthLabel)});
        this.add(wrapLengthField, wrapLengthConstraints);
        wrapLengthField.setSize(minFieldSize);

        RelativeConstraints lengthLabelConstraints = new RelativeConstraints(
                new Binding[] {new Binding(eightBelow, sessionCheck),
                        new Binding(verticalCenter, wrappingCheck), rightEdge});
        this.add(lengthLabel, lengthLabelConstraints);

        Binding belowWrappingBinding = new Binding(eightBelow, wrappingCheck);
        RelativeConstraints recipientConstraints = new RelativeConstraints(
                new Binding[] {leftEdge, belowWrappingBinding});
        this.add(recipientCheck, recipientConstraints);

        RelativeConstraints usernameFieldConstraints = new RelativeConstraints(
                new Binding[] {new Binding(eightRightOf, recipientCheck),
                        new Binding(verticalCenter, recipientCheck),
                        belowWrappingBinding});
        this.add(usernameField, usernameFieldConstraints);
        usernameField.setSize(minFieldSize);

        RelativeConstraints atLabelConstraints = new RelativeConstraints(
                new Binding[] {new Binding(eightRightOf, usernameField),
                        new Binding(verticalCenter, recipientCheck),
                        belowWrappingBinding});
        this.add(atLabel, atLabelConstraints);

        RelativeConstraints urlFieldConstraints = new RelativeConstraints(
                new Binding[] {new Binding(eightRightOf, atLabel),
                        new Binding(verticalCenter, recipientCheck),
                        belowWrappingBinding, rightEdge});
        this.add(urlField, urlFieldConstraints);
        urlField.setSize(minFieldSize);

        Binding belowRecipientBinding = new Binding(twelveBelow, recipientCheck);
        RelativeConstraints serverCheckConstraints = new RelativeConstraints(
                new Binding[] {leftEdge, belowRecipientBinding});
        this.add(serverCheck, serverCheckConstraints);

        RelativeConstraints serverLabelConstraints = new RelativeConstraints(
                new Binding[] {new Binding(eightRightOf, serverCheck),
                        new Binding(verticalCenter, serverCheck)});
        this.add(serverLabel, serverLabelConstraints);

        RelativeConstraints serverNameFieldConstraints = new RelativeConstraints(
                new Binding[] {new Binding(eightRightOf, serverLabel),
                        new Binding(verticalCenter, serverCheck),
                        belowRecipientBinding});
        this.add(serverNameField, serverNameFieldConstraints);
        serverNameField.setSize(minFieldSize);

        RelativeConstraints portLabelConstraints = new RelativeConstraints(
                new Binding[] {new Binding(eightRightOf, serverNameField),
                        new Binding(verticalCenter, serverCheck),
                        belowRecipientBinding});
        this.add(portLabel, portLabelConstraints);

        RelativeConstraints serverPortConstraints = new RelativeConstraints(
                new Binding[] {new Binding(eightRightOf, portLabel),
                        new Binding(verticalCenter, serverCheck), rightEdge,
                        belowRecipientBinding});
        this.add(serverPortField, serverPortConstraints);
        serverPortField.setSize(minFieldSize);

        RelativeConstraints cancelConstraints = new RelativeConstraints(
                new Binding[] {new Binding(eightLeftOf, sendButton),
                        bottomEdge, new Binding(eightBelow, serverPortField)});
        this.add(cancelButton, cancelConstraints);

        RelativeConstraints sendConstraints = new RelativeConstraints(
                new Binding[] {rightEdge, bottomEdge, new Binding(eightBelow, serverPortField)});
        this.add(sendButton, sendConstraints);

        this.setSize(350, 317);


        dayField.setVisible(false);
        dayLabel.setVisible(false);
        sessionField.setVisible(false);
        sessionLabel.setVisible(false);
        serverPortField.setVisible(false); 
        portLabel.setVisible(false);
        serverNameField.setVisible(false); 
        serverLabel.setVisible(false);
        usernameField.setVisible(false); 
        atLabel.setVisible(false);
        urlField.setVisible(false); 
        lengthLabel.setVisible(false);
        wrapLengthField.setVisible(false);

    }


    protected String[] getParameters() {
        StringBuffer lineSequence = new StringBuffer();
        /**
         * Specification for what the program does, and what OutputFiles takes as
         * parameters 
         * -i recipientCheck [sends injected messages] 
         * -s server:port [server to send the injected messages through (optional)] 
         * -m [turns on Unix 'mbox' mode] 
         * -w [turns on Windows 'eml' mode]\ 
         * -c [turns on 'cheat' mode] 
         * -l [turns on 'label' mode] 
         * -d dayCheck [set which day - default is 1] 
         * -p len [set which session - default is 1] 
         * -r len [turn on 'wRap' mode default is off] 
         * -v [turns on 'verbose' mode]
         */
        
        //refactor me to not hard code these in here
        if (mboxCheck.isSelected())
        {
            lineSequence.append("-m ");
        }
        if (emlCheck.isSelected())
        {
            lineSequence.append("-w ");
        }

        if (cheatCheck.isSelected())
        {
            lineSequence.append("-c ");
        }

        if (labelCheck.isSelected())
        {
            lineSequence.append("-l ");
        }

        if (dayCheck.isSelected())
        {
            lineSequence.append("-d " + dayField.getText());
        }

        if (sessionCheck.isSelected())
        {
            lineSequence.append("-p " + sessionField.getText());
        }

        if (wrappingCheck.isSelected())
        {
            lineSequence.append("-r " + wrapLengthField.getText());
        }

        if (recipientCheck.isSelected())
        {
            lineSequence.append("-i " + usernameField.getText() + "@" + urlField.getText() + " ");
        }

        if (serverCheck.isSelected())
        {
            lineSequence.append("-s " + serverNameField.getText() + ":" + serverPortField.getText() + " ");
        }
        
        System.out.println(lineSequence);
        System.out.println(lineSequence.toString().split(" "));
        
        return lineSequence.toString().split(" ");
    }
}
