package openinjector;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileFilter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.mail.Address;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;

import airspaceGUI.base.corpus.CorpusGUIMessage;
import airspaceGUI.base.corpus.MessageType;

import edu.cmu.radar.base.Person;
import edu.cmu.radar.base.corpus.NormalFakeMessage;
import edu.cmu.radar.base.corpus.Recipient;
import edu.cmu.radar.base.corpus.Variable;
import edu.cmu.radar.base.corpus.VariableSenderFakeMessage;

public class OutputFiles
{
    // the messages that might not need variable substitutions
    // they may still need it for variables embedded in subject or
    // body text
    private List normalMessages;

    /* the messages that explicitly need variable substitutions
    private List variableMessages;*/

    // the variables to substitute into the messages above
    private Map variables = new HashMap();

    private int startTime;

    private static final String XRADAR = "X-RADAR-";

    private Session session;

    private boolean labels = false;

    private boolean all_radar = false;

    private boolean text = false;

    private boolean eml = false;
    
    private String recipient;
    
    private String mailServer;

    private int dayOffset = 0;
    
    private int sessionOffset = 1;

    private int wrapLine = 0;

    private SessionFactory sessionFactory;

    private boolean database = false;

    /**
     * @param args
     */
    public static void main(String[] args) {
        OutputFiles of = new OutputFiles(args);
    }

    /**
     * Sets up the test environment for this class by initiating a Hibernate
     * SessionFactory based on the default configuration (the file
     * "corpus.hibernate.xml" on the classpath.
     */
    protected final void setUp() {
        try
        {
            // Create the SessionFactory from hibernate.cfg.xml

            final File configFile = new File("corpus.hibernate.xml");
            final Configuration config = new Configuration()
                    .configure(configFile);

            sessionFactory = config.buildSessionFactory();
        } catch (Exception ex)
        {
            // Make sure you log the exception, as it might be swallowed
            System.err.println("Initial SessionFactory creation failed." + ex);
            throw new ExceptionInInitializerError(ex);
        }
    }

    public OutputFiles(String[] args, List messages)
    {
        init();
        normalMessages = messages;
    }
    
    /**
     * inits all the session stuff
     *
     */
    private void init()
    {
        setUp();
        session = sessionFactory.openSession();
        session.beginTransaction();
        this.session = session;
        startTime = (int)System.currentTimeMillis() * -1;
        getVariables();
    }
    
    public OutputFiles(String[] args)
    {
        init();
        
        //System.out.println(args);
        
        getHostname();

        int i = resolveArguments(args);
        
        //System.out.println("server is " + mailServer);
        if (i >= 0)
        {
            // System.out.println(database);
            if (!database && i <= args.length)
            {
                normalMessages = new ArrayList();
                //System.out.println("not a database");
                while (i < args.length)
                {
                    String filepath = args[i];
                    //System.out.println(i + "The path is " + filepath);
                    String pathMinusSome = filepath.substring(0, filepath
                            .lastIndexOf("/"));
                    File temp = new File(pathMinusSome);
                    File test = new File(filepath);
                    //System.out.println(temp.isDirectory());

                    if (!test.isFile())
                    {
                        //System.out.println("not a file");
                        FileFilter filter = new ExpandWildCardFileNames(
                                filepath);
                        File[] files = temp.listFiles(filter);

                        int j = 0;
                        while (j < files.length)
                        {
                            normalMessages.addAll(MessageLoader
                                    .loadMessage(files[j]));
                            j++;
                        }
                    } else
                    {
                        normalMessages.addAll(MessageLoader
                                .loadMessage(args[i]));
                    }
                    i++;
                }
            } else if (database)
            {
                getMessages();
            }

            outputMessages();
        }
    }

    private void outputMessages()
    {
        if (recipient != null)
        {
            //System.out.println("ummm... you should have messages by now");
            sendMessages(recipient);
        }
        
        if (text)
        {
            createTextMessages();
        }

        if (eml)
        {
            createEML();
        }
    }
    
    /**
     * Parses out the arguments from the string.
     * @param args the arguments that were passed to this program
     * @return the index of the last argument that was used, -1 if there was an error
     */
    private int resolveArguments(String[] args)
    {
        String argument;
        int i = 0;
        

        if (args == null || args.length == 0)
        {
            printHelpText();
        }


        /**
         * -i recipient [sends injected messages] 
         * -s server:port [server to send the injected messages through (optional)]
         * -m [turns on Unix 'mbox' mode]
         * -w [turns on Windows 'eml' mode]\ 
         * -c [turns on 'cheat' mode] 
         * -l [turns on 'label' mode] 
         * -d day [set which day - default is 1] 
         * -p len [set which session - default is 1] 
         * -r len [turn on 'wRap' mode - default is off] 
         * -v [turns on 'verbose' mode]
         */
        int lastArg = 0;
        while (args.length > i)
        {
            
            argument = args[i];
            //System.out.println(argument);
            if ("-m".equals(argument))
            {
                // output in 'mbox' mode
                text = true;
                lastArg = i;
            } else if ("-w".equals(argument))
            {
                // output an 'eml'
                eml = true;
                lastArg = i;
            } else if ("-c".equals(argument))
            {
                // write out all the cheating x-radar labels (time, period,
                // outcome... etc)
                all_radar = true;
                lastArg = i;
            } else if ("-l".equals(argument))
            {
                // print out labels
                labels = true;
                lastArg = i;
            } else if ("-d".equals(argument))
            {
                // the day should be dayOffset number of days from now
                if (validateArgument(args[i+1]))
                {
                    dayOffset = new Integer(args[++i]).intValue();
                    lastArg = i;
                } else
                {
                    printHelpText();
                    return -1;
                }
            } else if ("-p".equals(argument))
            {
                if (validateArgument(args[i+1]))
                {
                    sessionOffset = new Integer(args[++i]).intValue();
                    lastArg = i;
                } else
                {
                    printHelpText();
                    return -1;
                }
            } else if ("-r".equals(argument))
            {
                if (validateArgument(args[i+1]))
                {
                    wrapLine = new Integer(args[++i]).intValue();
                    lastArg = i;
                } else
                {
                    printHelpText();
                    return -1;
                }
            } else if ("-v".equals(argument))
            {
                // print verbose debugging information
                // TODO figure out what verbose should mean
                lastArg = i;
            } else if ("-i".equals(argument))
            {
                if (validateArgument(args[i+1]))
                {
                    recipient = args[++i];
                
                    if (mailServer == null && recipient.matches(CorpusGUIMessage.EMAILVERIFICATION))
                    {
                        Matcher full = Pattern.compile(CorpusGUIMessage.EMAILVERIFICATION)
                                    .matcher(recipient);
                        full.find();
                        //System.out.println(recipient);
                        mailServer = full.group(3) ;//+ ":25";
                    } else if (!recipient.matches(CorpusGUIMessage.EMAILVERIFICATION))
                    {
                        printHelpText();
                        return -1;
                    }
                    database = true;
                    lastArg = i;
                    // send to recipient
                } else
                {
                    printHelpText();
                    return -1;
                }
            } else if("-s".equals(argument))
            {
                mailServer = args[++i];
                lastArg = i;
            }else if (argument.contains("-"))
            {
                printInvalidHelpText();
                return -1;
            } else{
                database = false;
            }

            i++;
            //System.out.println(lastArg);
        }
        //System.out.println(lastArg);
        return ++lastArg;
    }
    
    private boolean validateArgument(String s){
        /**
         * -i recipient [sends injected messages] 
         * -s server:port [server to send the injected messages through (optional)]
         * -m [turns on Unix 'mbox' mode]
         * -w [turns on Windows 'eml' mode]\ 
         * -c [turns on 'cheat' mode] 
         * -l [turns on 'label' mode] 
         * -d day [set which day - default is 1] 
         * -p len [set which session - default is 1] 
         * -r len [turn on 'wRap' mode - default is off] 
         * -v [turns on 'verbose' mode]
         */
        return !s.matches("(-)[i,s,m,w,c,l,d,p,r,v]");
    }
    
    private void printInvalidHelpText() {
        //System.out.println("Invalid argument");
        printHelpText();
    }

    private void printHelpText() {
        System.out
                .println("Useage: ./OutpusFiles [-m -w -l -c -d -p -r -v [-i recipient] [-s server:port]");
        System.out.println(// "-i recipient [sends injected messages]\n" +
                "-m [turns on Unix 'mbox' mode]\n"
                        + "-w [turns on Windows 'eml' mode]\n"
                        + "-c [turns on 'cheat' mode]\n"
                        + "-l [turns on 'label' mode]\n"
                        + "-d day [set which day - default is 1]\n"
                        + "-p len [set which session - default is 1]\n"
                        + "-r len [wrap the lines atLabel length len]\n"
                        + "-i recipient [send the messages that are parsed to this email address]"
                        + "-s server:port [send the messages through this server (optional, \n\t if not specified will default to the hostname of the recipient)]"
                        /* "-v [turns on 'verbose' mode]" */);
    }

    private static String getHostname() {
        try
        {
            //in the future, perhaps save this value to
            // speed the function up some?
            InetAddress addr = InetAddress.getLocalHost();

            // Get IP Address
            byte[] ipAddr = addr.getAddress();

            // Get hostname
            String hostname = addr.getHostName();
            if (hostname.contains(".local"))
            {
                hostname = hostname.replace(".local", "");
            }
            return hostname;
        } catch (UnknownHostException e)
        {
            e.printStackTrace();
        }
        return null;
    }

    private void createTextMessages() {
        getMessages();
        int messageNumber = 1;
        String directory = System.getProperty("user.dir") + "/msgs/";
        (new File(directory)).mkdir();

        //System.out.println(directory);
        /*
         * execute this command set when this is attached to a GUI JFileChooser
         * selectDirectory = new JFileChooser();
         * selectDirectory.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
         * selectDirectory.showDialog(, arg1)
         */

        for (int i = 0; i < normalMessages.size(); i++)
        {
            try
            {
                FileWriter messagesOut = new FileWriter(directory + "msg" + i
                        + ".txt");
                messagesOut
                        .write(printTextMessage((MessageType)normalMessages
                                .get(i)));
                messagesOut.close();
                //System.out.println("message " + i + " eml");
            } catch (IOException e)
            {
                e.printStackTrace();
            }
        }
    }

    private String printTextMessage(MessageType fm) {
        String message;
        // example of a message in text format
        // From afurr@cmu.edu Fri Jan 19 03:28:54 2007
        // Received: (injector v1.31); 19 Jan 2007 03:28:54 -0000
        // Message-Id: <1169177274.1169177334.injector.11239@CURUNIR>
        // X-RADAR-replyto: 0
        // Date: Fri, 19 Jan 2007 03:28:54 -0000 (GMT)
        // From: afurr@cmu.edu
        // To: bor@cs.cmu.edu
        // Subject: More Rangos updates
        // X-RADAR-time: Jun 7 1
        // X-RADAR-period: 0
        // X-RADAR-group: Query
        // X-RADAR-outcome: blank
        // X-RADAR-thread: 0
        // X-RADAR-VIP: false
        // X-RADAR-noise: false
        // X-RADAR-response: false
        // X-RADAR-label: CHANGE-ROOM
        // Sender: afurr@cmu.edu
        //
        // Hey, we just switched around Rangos' setup, and in a banquet
        // configuration of one section of Rangos, it can only hold about 110
        // people. Sorry about the overestimate.
        message = "From " + fm.getFrom()/*.getEmail()*/ + " "
                + dateString(fm, "EEE MMM dd yyyy hh:mm:ss ") + "\n";
        message += "Recieved: (crossinjector v.0.1); "
                + dateString(fm, "dd MMM yyyy hh:mm:ss Z") + "\n";
        message += "Message-Id: <" + startTime + "."
                + (int)(System.currentTimeMillis() * -1) + ".injector."
                + fm.getId() + "@" + getHostname() + ">\n";
        message += XRADAR + "replyto: " + fm.getReplyTo() + "\n";
        message += "Date: "
                + dateString(fm, "EEE, dd MM yyyy hh:mm:ss Z (zzz)") + "\n";
        message += "From: " + fm.getFrom()/*.getEmail()*/ + "\n";
        message += getRecipientsString("To: ", fm.getTo());
        message += getRecipientsString("CC: ", fm.getCC());
        message += getRecipientsString("BCC: ", fm.getBCC());
        message += "Subject: " + fillInVariable(fm.getSubject()) + "\n";
        message += XRADAR + "time: " + fm.getTime() + "\n";

        if (all_radar)
        {
            message += getXRADAR(fm);
        } else if (labels)
        {
            message = getLabels(fm, message);
        }

        message += "Sender: " + fm.getFrom()/*.getEmail()*/ + "\n\n";

        message = writeBody(fm, message);

        return message;
    }
    
    /**
     * @param fm
     *            The message to get the body from
     * @param message
     *            The message string that the body should be apended to
     * @return The message string with the body attached, possibly limited to
     *         the line length in wrapLine
     */
    private String writeBody(MessageType fm, String message) {
        
        if (wrapLine > 0 && fm.getBody().length() > wrapLine)
        {
            String body = fm.getBody();
            int l;
            int i = 0;
            String substring;

            body = fillInVariable(body);
            
            l = body.length();
            
            while (i < l)
            {
                if ((i + wrapLine) < l)
                {
                    // there is more than wrapline left on the string 
                    //to be written to another one
                    substring = body.substring(i, i + wrapLine);
                    
                    if (substring.contains(" ") )
                    {
                        substring = substring.substring(0, substring
                                .lastIndexOf(" ") + 1);
                    } else if (substring.contains("/") && substring.contains("http://"))
                    {
                        substring = substring.substring(0, substring
                                .lastIndexOf("/") + 1);
                    }
                    // need to add the 1 to save the space atLabel the end of this line
                    //instead of the beginning of the next line
                    if (substring.contains("\n"))
                    {
                        substring = substring.substring(0, substring.indexOf("\n") + 1);
                    }

                } else
                {
                    substring = body.substring(i, l);
                }

                i += substring.length();
                if (message != null)
                {
                    message += substring + "\n";
                } else
                {
                    message = substring+"\n";
                }
            }
        } else
        {
            // there are less than wrapline characters left
            if (message != null)
            {
                message += fm.getBody();
            } else
            {
                message = fm.getBody();
            }
        }
        return message;
    }

    /**
     * Returns all the XRADAR fields
     * @param fm The message to read the XRADAR fields from
     * @return The string containing all the XRADAR fields
     */
    private String getXRADAR(MessageType fm) {
        String message;

        message = XRADAR + "period: " + fm.getPeriod() + "\n";
        message += XRADAR + "group: " + fm.getGroup() + "\n";
        message += XRADAR + "outcome: " + fm.getOutcome() + "\n";
        message += XRADAR + "thread: " + fm.getThread() + "\n";
        
        if (fm instanceof NormalFakeMessage)
        {
            message += XRADAR + "VIP: " + ((NormalFakeMessage)fm).getVIP() + "\n";
        } else
        {
            message += XRADAR + "VIP: false\n";
        }
        message += XRADAR + "noise: " + fm.getNoise() + "\n";
        message += XRADAR + "response: " + fm.getResponse() + "\n";

        message = getLabels(fm, message);
        return message;
    }

    /**
     * @param fm The message to get the labels from
     * @param message The current string representation of the message
     * @return The message now containing the message
     */
    private String getLabels(MessageType fm, String message) {
        Iterator labels = fm.getLabels().iterator();
        while (labels.hasNext())
        {
            System.out.println("labels going out");
            message += XRADAR + "label: " + labels.next() + "\n";
        }
        return message;
    }

    /**
     * @param recipientType The type of recipients that is being processed
     * @param recipients The list of recipients to process
     * @return A string containing the recipients
     */
    private String getRecipientsString(String recipientType, ArrayList recipients) {
        String message = "";
        String recipientString = "";
        Recipient recipient;

        if (recipients != null)
        {
            for (int i = 0; i < recipients.size(); i++)
            {
                if (recipients.get(i) instanceof Recipient)
                {
                    recipient = ((Recipient)recipients.get(i));
                    //System.out.println(recipient.getIsVariable());
                    if (Boolean.getBoolean(recipient.getIsVariable()))
                    {
                        // the recipient is a variable, get the variable
                        //System.out.println("umm... variable");
                        recipientString = (String)variables.get(recipient
                                .getRecipientId());
                    } else
                    {
                        // the recipient is a person, get the person's email address
                        List l = session.createCriteria(Person.class).add(
                                Restrictions.eq("id", recipient.getRecipientId()))
                                .list();
                        recipientString = ((Person)l.get(0)).getEmail();
                    }
                    message += recipientType + recipientString + "\n";
                } else
                {
                    return (String)recipients.get(i);
                }
            }
            return message;
        } 
        return null;
    }

    /**
     * This creates a date string for the emails in the correct format
     * @param fm the message to read the time out of
     * @param format The format in which the time string should be
     * @return The string representation of the time in the specified format
     */
    private String dateString(MessageType fm, String format) {
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        // outputs the date into the format specified by the format string
        if (!fm.getBackstory())
        {
            GregorianCalendar calendar = new GregorianCalendar();
            calendar.roll(GregorianCalendar.MINUTE, new Integer(fm.getTime()
                    .substring(fm.getTime().lastIndexOf(" ") + 1)).intValue());
            calendar.roll(GregorianCalendar.HOUR, sessionOffset);
            calendar.roll(GregorianCalendar.DAY_OF_MONTH, dayOffset);

            return sdf.format(calendar.getTime());
        }
        return null;
    }

    private String getCurrentTime(String format) {
        DateFormat dateFormat = new SimpleDateFormat(format);
        Date date = new Date();
        return dateFormat.format(date);
    }

    private void getMessages() {
        if (normalMessages == null)
        {
            this.normalMessages = new ArrayList();
            List normalMessages = session.createCriteria(NormalFakeMessage.class)
                    .addOrder(Order.asc("id")).list();

            List variableMessages = session.createCriteria(
                    VariableSenderFakeMessage.class).addOrder(Order.asc("id"))
                    .list();
            
            for(int i = 0; i < normalMessages.size(); i++ )
            {
                this.normalMessages.add(new CorpusGUIMessage((NormalFakeMessage)normalMessages.get(i)));
            }
            
            for(int i = 0; i < variableMessages.size(); i++ )
            {
                this.normalMessages.add(new CorpusGUIMessage((VariableSenderFakeMessage)variableMessages.get(i)));
            }
            
        }
    }
    
    private void getVariables() {
        List variablesList = session.createCriteria(Variable.class).addOrder(
                Order.asc("id")).list();
        Variable workingOn;
        for(int i = 0; i < variablesList.size(); i++)
        {
            workingOn = ((Variable)variablesList.get(i));
            this.variables.put(workingOn.getName(), workingOn.getValue());
            this.variables.put(workingOn.getId(), workingOn.getValue());
        }
        
    }

    private void createEML() {
        // example of a message in eml format
        // Received: (injector v1.31); 19 Jan 2007 03:26:15 -0000
        // Message-Id: <1169177116.1169177175.injector.11239@CURUNIR>
        // X-RADAR-replyto: 0
        // Date: Fri, 19 Jan 2007 03:26:15 -0000 (GMT)
        // From: afurr@cmu.edu
        // To: bor@cs.cmu.edu
        // Subject: More Rangos updates
        // X-RADAR-time: Jun 7 1
        // X-RADAR-period: 0
        // X-RADAR-group: Query
        // X-RADAR-outcome: blank
        // X-RADAR-thread: 0
        // X-RADAR-VIP: false
        // X-RADAR-noise: false
        // X-RADAR-response: false
        // X-RADAR-label: CHANGE-ROOM
        // Sender: afurr@cmu.edu
        //
        // Hey, we just switched around Rangos' setup, and in a banquet
        // configuration of one section of Rangos, it can only hold about 110
        // people. Sorry about the overestimate.
        getMessages();
        int messageNumber = 1;
        String directory = System.getProperty("user.dir") + "/msgs/";
        (new File(directory)).mkdir();

        //System.out.println(directory);
        /*
         * execute this command set when this is attached to a GUI JFileChooser
         * selectDirectory = new JFileChooser();
         * selectDirectory.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
         * selectDirectory.showDialog(, arg1)
         */

        for (int i = 0; i < normalMessages.size(); i++)
        {
            try
            {
                FileWriter messagesOut = new FileWriter(directory + "msg" + i
                        + ".eml");
                messagesOut
                        .write(printEMLMessage((MessageType)normalMessages
                                .get(i)));
                messagesOut.close();
                System.out.println("message " + i + " eml");
            } catch (IOException e)
            {
                e.printStackTrace();
            }
        }
    }

    private String printEMLMessage(MessageType fm) {
        String message;

        message = "Recieved: (crossinjector v.0.1); "
                + dateString(fm, "dd MMM yyyy hh:mm:ss Z") + "\n";
        message += "Message-Id: <" + startTime + "."
                + (int)(System.currentTimeMillis() * -1) + ".injector."
                + fm.getId() + "@" + getHostname() + ">\n";
        message += XRADAR + "replyto: " + fm.getReplyTo() + "\n";
        message += "Date: "
                + dateString(fm, "EEE dd MMM yyyy hh:mm:ss Z (zzz)") + "\n";
        message += "From: " + fm.getFrom()/*.getEmail()*/ + "\n";
        message += getRecipientsString("To: ", fm.getTo());
        message += getRecipientsString("CC: ", fm.getCC());
        message += getRecipientsString("BCC: ", fm.getBCC());
        message += "Subject: " + fillInVariable(fm.getSubject()) + "\n";
        message += XRADAR + "time: " + fm.getTime() + "\n";

        if (all_radar)
        {
            message += getXRADAR(fm);
        } else if (labels)
        {
            message = getLabels(fm, message);
        }

        message += "Sender: " + fm.getFrom()/*.getEmail()*/ + "\n\n";
        message = writeBody(fm, message);

        return message;
    }

    
    private String fillInVariable(String variableString)
    {
        String value = "";
        String varParameter = "var\\s*=\\s*\"(\\w+)\"";
        
        Matcher full = Pattern.compile(airspaceGUI.base.corpus.Variable.VARIABLEVERIFICATION)
                    .matcher(variableString);
        Matcher partial = Pattern.compile(varParameter).matcher(variableString);
    //may want to move this processing into fillInVariable to make 
    //the overhead for calling that function less
        //System.out.println(variableString);
        while (full.find() && partial.find())
        {
            value = (String)variables.get(partial.group(1));
            variableString = variableString.replaceAll(full.group(), value);
        }
        //please dynamically resolve and return the value of the variable 
        //in the String
        return variableString;
    }
    
    private String fillInVariable(Variable variable)
    {
        return variable.getValue();
    }
    
    private void sendMessages(String recipient) {
        // construct a socket to send these emails over
        // make emails into a format where they can be sent
        // send them.
        getMessages();
        String time;
        Map normalMessages = orderMessagesByTime();
        
        //System.out.println("these are " + normalMessages);
        //System.out.println("The keys are " + normalMessages.keySet());
//      System.out.println("About to send email.");
        //for (int i = 0; i < /*normalMessages.size()*/1; i++)
        while(!normalMessages.isEmpty())
        {
            time = getCurrentTime("EEE dd MMM yyyy hh:mm:ss Z (zzz)");
            MessageType nfm = (MessageType)normalMessages
                            .get(time);
            // go through all the messages and send them
            //... maybe we need to put them into a different representation 
            // so that we can send them atLabel the correct time.
            if (nfm != null)
            {
                String name;
                String email;
                System.out.println("Sending...");
                if (nfm.getFrom().contains("<"))
                {
                    name = nfm.getFrom().substring(0, nfm.getFrom().indexOf("<"));
                    email = nfm.getFrom().substring(nfm.getFrom().indexOf("<"), nfm.getFrom().indexOf(">"));
                } else
                {
                    name = nfm.getFrom();
                    email = name;
                }
                sendMail(nfm, name, email);
                normalMessages.remove(time);
            } else
            {
                System.out.println("Not sending..." + time);
            }
            try
            {
                Thread.sleep(1000);
            } catch (InterruptedException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }
    
    
    private Map orderMessagesByTime() {
        Map sorted = new HashMap();
        for(int i = 0; i < normalMessages.size(); i++)
        {
            sorted.put(dateString((MessageType)normalMessages.get(i), 
                            "EEE dd MMM yyyy hh:mm:ss Z (zzz)"), 
                       normalMessages.get(i));
        }  
        return sorted;
    }

    private void sendMail(MessageType fm, String fromNameString, String fromEmailString)
    {
        String mailFromString = "MAIL FROM: <" + fromEmailString + ">";
        String fromString = "From: " + fromNameString + " <" + fromEmailString + ">";
        
        try {
            /*
             * Uncomment if you want to use jmail instead of socket code
             * 
             * Properties p = new Properties();
            p.setProperty("mail.user", recipient);
            p.setProperty("mail.from", fromEmailString);
            p.setProperty("mail.host", mailServer
                    .substring(0, mailServer.lastIndexOf(":")));
            p.setProperty("mail.port", mailServer
                    .substring(mailServer.lastIndexOf(":")+1, mailServer.length()));
            p.setProperty("mail.transport.protocol", "smtp");
            javax.mail.Session mailSession = javax.mail.Session.getDefaultInstance(p);
            mailSession.setDebug(true);
            //mailSession.setPasswordAuthentication(new URLName("smtp." + mailServer), 
            ///       new PasswordAuthentication("cmuwork@steinsquared.com", "4tesT!ng"));
            //Transport t = mailSession.getTransport();
            SMTPTransport t = (SMTPTransport)mailSession.getTransport();//new SMTPTransport(mailSession);
            System.out.println("got transport");*/
            
            /*
             * The commented out stuff here is socket code, only here if we 
             * want to stop using javamail.*/
            Socket s;
            if (mailServer.contains(":"))
            {
                int index = mailServer.indexOf(":");
                s = new Socket(mailServer.substring(0, index), 
                        Integer.valueOf(mailServer.substring(index +1)).intValue());
            } else
            {
                s = new Socket(mailServer, 25);
            }
            s.setKeepAlive(true);
            s.setTcpNoDelay(true);
            s.setReuseAddress(true);
            BufferedReader in = new BufferedReader
            (new InputStreamReader(s.getInputStream(), "8859_1"));
            BufferedWriter out = new BufferedWriter
            (new OutputStreamWriter(s.getOutputStream(), "8859_1"));
            //SMTPMessage message = new SMTPMessage(mailSession/*, s.getInputStream()*/);
            
            //System.out.println("socket set");
            
            while(!s.isConnected())
             {
                  Thread.sleep(1000);
              }
                      
            sendln(in, out, "HELO world");
            sendln(in, out, mailFromString);
            sendln(in, out, "RCPT TO: <" + recipient + ">" );
            sendln(in, out, "DATA");
            if (in.ready())
            {
               String fudge = in.readLine();
               System.out.println("fudge<< " + fudge);
            }
            sendln(in, out, "MIME-Version: 1.0");
            sendln(out, "DATE: " + dateString(fm, "EEE dd MMM yyyy hh:mm:ss Z (zzz)"));
            System.out.println("subject");
            //message.setSubject(fm.getSubject());
            sendln(out, "Subject: "+ fm.getSubject());
            System.out.println("from");
            sendln(out, fromString);

            //message.setFrom(new InternetAddress (fromEmailString, fromNameString));
            
            sendln(out, "To: " + recipient);
            System.out.println("DEBUG: sendEmail.sendMail: trying to set Received\n");
            sendln(out, "Date:" + dateString(fm, "EEE dd MMM yyyy hh:mm:ss Z (zzz)"));
            //message.setSentDate(new Date(dateString(fm, "EEE dd MMM yyyy hh:mm:ss Z (zzz)")));
            sendln(out, "Content-Type: text/plain; charset=\"us-ascii\"");
            sendln(out, "\r");
            //message.setText(writeBody(fm, null));
            sendln(out, writeBody(fm, null));
            sendln(out, "\r\n");
                  
            /*message.setRecipients(Message.RecipientType.TO, recipient);
            message.setRecipients(Message.RecipientType.TO, getAddressArray(fm.getTo()));
            message.setRecipients(Message.RecipientType.CC, getAddressArray(fm.getCC()));
            message.setRecipients(Message.RecipientType.BCC, getAddressArray(fm.getBCC()));
            */
            // done
            sendln(in, out,".");
            sendln(in, out, "QUIT");
            
            s.close();
            
           /* t.connect();
            t.sendMessage(message, message.getAllRecipients());
            t.close();*/


            //Thread.sleep(10);
        }
        catch (Exception e) {
            e.printStackTrace();
            System.out.println("\n" + e.getMessage());
        }
    }

    
    private Address[] getAddressArray(ArrayList list)
    {
        Address[] addresses = new Address[list.size()];
        
        for(int i = 0; i < list.size(); i++)
        {
            //System.out.println(list);
            //System.out.println(list.get(i).getClass());
            try
            {
                addresses[i] = new InternetAddress(((Recipient)list.get(i)).getRecipientMessage(session));
            } catch (AddressException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        
        return addresses;
    }

    /*This goes with the socket code above, both are still here in the case that
     * we don't want to use javamail anymore*/
     public void sendln(BufferedReader in, BufferedWriter out, String s) {
        try {
            out.write(s + "\r\n");
            out.flush();
          System.out.println(">> " + s);
          if (in.ready())
          {
              s = in.readLine();
              System.out.println("<< " + s);
          }
          //Thread.sleep(10);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    } // end sendln(3 args)

    public void sendln(BufferedWriter out, String s) {
        try {
            out.write(s + "\r\n");
            out.flush();
            //System.out.println(">> " + s);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    } // end sendln(2 args)
    
}
