package openinjector;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.JOptionPane;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import airspaceGUI.base.corpus.MessageType;
import airspaceGUI.base.corpus.Variable;

import edu.cmu.radar.base.corpus.Label;

/**
 * A class that models an xml e-mail message
 * 
 * @author srbennet
 * 
 */
public class RMessage implements MessageType
{
    private static final String LABEL_KEY = "Labels";

    //constant to deal with bcc recipients
    private static final String BCCSTRING = "bcc";

    //constant not to hard code the time end tag
    private static final String TIMEENDTAG = "</time>\n";

    /**
     * variable that contains all the different fields contained in a message
     * 
     */
    private final transient HashMap messageMap = new HashMap();

    //private boolean[] labels = new boolean[8]; // Hold labels

    /*protected String subject;

    protected String body;*/

    /**
     * Default constructor, initializes all the variables
     * 
     */
    public RMessage()
    {
        /*this.backstory = false;
        //bccRecipients = new ArrayList();
        //this.body = new String();
        ccRecipients = new ArrayList();
        this.from = new String();
        this.group = new String();
        this.messageid = 0;
        this.noise = false;
        this.outcome = new String();
        this.period = 0;
        this.replyto = 0;
        this.response = false;
        this.subject = new String();
        this.thread = 0;
        this.time = new String();
        toRecipients = new ArrayList();*/
        /*for (int i = 0; i < 8; i++)
        {
            labels[i] = false;
        }*/
        messageMap.put("backstory", Boolean.valueOf(false).toString());
    }

    /**
     * 
     * @return the array of labels the message can have
     */
    public Set getLabels() {
        return ((Set)messageMap.get(LABEL_KEY));
    }

    /**
     * 
     * @return a String containing the time of the message
     */
    public String getTime() {
        return (String)messageMap.get("time");//time;
    }

    /**
     * 
     * @return the messages' period
     */
    public int getPeriod() {
        return Integer.valueOf((String)messageMap.get("period")).intValue();
    }

    /**
     * 
     * @return the period of the message
     */
    public String getGroup() {
        return (String) messageMap.get("group");
    }

    /**
     * 
     * @return the messages' outcome
     */
    public String getOutcome() {
        return (String)messageMap.get("outcome");
    }

    /**
     * 
     * @return the value of this messages' thread
     */
    public int getThread() {
        return Integer.valueOf((String)messageMap.get("thread")).intValue();
    }

    /**
     * 
     * @return true if this message was written by a VIP, false otherwise
     */
    public boolean getVIP() {
        return Boolean.getBoolean((String)messageMap.get("VIP"));
    }

    /**
     * 
     * @return true if this message is noise, false otherwise
     */
    public boolean getNoise() {
        return Boolean.getBoolean((String)messageMap.get("noise"));
    }

    /**
     * 
     * @return true if this message has a response, false otherwise
     */
    public boolean getResponse() {
        return Boolean.getBoolean((String)messageMap.get("response"));
    }

    /**
     * 
     * @return true if this message is backstory, false otherwise
     */
    public boolean getBackstory() {
        return Boolean.getBoolean((String)messageMap.get("backstory"));
    }

    /**
     * 
     * @return this message's message id
     */
    public int getId() {
        Integer id = Integer.valueOf((String)messageMap.get("messageid"));
        return id.intValue();
    }

    /**
     * 
     * @return the id of the message that replies to this message, 0 if there
     *         are no replies
     */
    public int getReplyTo() {
        if (messageMap.get("replyto") == null)
        {
            return 0;
        }else
        {
            return Integer.valueOf((String)messageMap.get("replyto")).intValue();
        }
    }

    /**
     * 
     * @return the e-mail address in the from field of this message
     */
    public String getFrom() {
        return (String) messageMap.get("from");
    }
    
    /**
     * 
     * @return the e-mail address in the from field of this message
     */
    public int getIntFrom() {
        return ((Integer) messageMap.get("fromInt")).intValue();
    }

    /**
     * 
     * @return the e-mail address(es) in the to field of this message
     */
    public ArrayList getTo() {
        return (ArrayList) messageMap.get("to");
    }

    /**
     * 
     * @return the e-mail address(es) in the cc field of this message
     */
    public ArrayList getCC() {
        return (ArrayList) messageMap.get("cc");
    }

    /**
     * 
     * @return the e-mail address(es) in the bcc field of this message
     */
    public ArrayList getBCC() {
        return (ArrayList) messageMap.get(BCCSTRING);
    }

    /**
     * 
     * @return the subject of this message
     */
    public String getSubject() {
        return (String)messageMap.get("subject");
    }

    /**
     * 
     * @return the body of this message
     */
    public String getBody() {
        return (String) messageMap.get("body");
    }

    
    public void setMessageClass(int messageClass)
    {
        messageMap.put("messageClass", new Integer(messageClass));
    }
    
    /**
     * 
     * @param time -
     *            the time this messages time should be set to
     */
    public void setTime(final String time) {
        messageMap.put("time", time);
    }

    /**
     * 
     * @param period -
     *            the period this messages period should be set to
     */
    public void setPeriod(final int period) {
        messageMap.put("period", new Integer(period).toString());
    }

    /**
     * 
     * @param group -
     *            the group this messages group should be set to
     */
    public void setGroup(final String group) {
        messageMap.put("group", group);
    }

    /**
     * 
     * @param outcome -
     *            the outcome that this messages outcome should be set to
     */
    public void setOutcome(final String outcome) {
        messageMap.put("outcome", outcome);
    }

    /**
     * 
     * @param thread -
     *            the thread this messages' thread should be set to
     */
    public void setThread(final int thread) {
        messageMap.put("thread", new Integer(thread).toString());
    }

    /**
     * 
     * @param VIP -
     *            the vip value this messages' VIP field should be set to
     */
    public void setVIP(final boolean VIP) {
        messageMap.put("VIP", Boolean.valueOf(VIP).toString());
    }

    /**
     * 
     * @param noise -
     *            the noise value this messages' noise field should be set to
     */
    public void setNoise(final boolean noise) {
        messageMap.put("noise", Boolean.valueOf(noise).toString());
    }

    /**
     * 
     * @param response -
     *            the response value this messages' response field should be set
     *            to
     */
    public void setResponse(final boolean response) {
        messageMap.put("response", Boolean.valueOf(response).toString());
    }

    /**
     * 
     * @param backstory -
     *            the backstory value this messages' backstory field should be
     *            set to
     */
    public void setBackstory(final boolean backstory) {
        messageMap.put("backstory", Boolean.valueOf(backstory).toString());
    }

    /**
     * 
     * @param id -
     *            the message id this message should be set to
     */
    public void setId(final int id) {
        messageMap.put("messageid", new Integer(id).toString());
    }

    /**
     * 
     * @param replyto -
     *            the messageID of the message this message is a reply to
     */
    public void setReplyTo(final int replyto) {
        messageMap.put("replyto", new Integer(replyto).toString());
    }

    /**
     * 
     * @param from -
     *            the e-mail address this message is from
     */
    public void setFrom(final String from) {
        messageMap.put("from", from);
    }
    
    /**
     * 
     * @param from -
     *            the e-mail address this message is from
     */
    public void setFrom(final int from) {
        messageMap.put("fromInt", new Integer(from));
    }

    /**
     * 
     * @param to -
     *            the e-mail address(es) this message is to
     */
    public void setTo(final ArrayList to) {
        if (messageMap.containsKey("to"))
        {
            final ArrayList toRecipients = (ArrayList)messageMap.get("to");
            toRecipients.addAll(to);
            messageMap.put("to", toRecipients);
        } else
        {
            messageMap.put("to", to);
        }
    }

    /**
     * 
     * @param cc -
     *            the e-mail address(es) this message is cc'ed to
     */
    public void setCC(final ArrayList cc) {
        if (messageMap.containsKey("cc"))
        {
            final ArrayList ccRecipients = (ArrayList)messageMap.get("cc");
            ccRecipients.addAll(cc);
            messageMap.put("cc", ccRecipients);
        } else
        {
            messageMap.put("cc", cc);
        }
    }

    /**
     * 
     * @param bcc -
     *            the e-mail address(es) this message is bcc'ed to
     */
    public void setBCC(final ArrayList bcc) {
        if (messageMap.containsKey(BCCSTRING))
        {
            final ArrayList bccRecipients = (ArrayList)messageMap.get(BCCSTRING);
            bccRecipients.addAll(bcc);
            messageMap.put(BCCSTRING, bccRecipients);
        } else
        {
            messageMap.put(BCCSTRING, bcc);
        }
    }

    /**
     * 
     * @param labels -
     *            the classifications of this message
     */
    public void setLabels(final Set labels) {
        messageMap.put(LABEL_KEY, labels);
    }
    
    /**
     * This adds the given label to the set of labels that are 
     * attached to this message
     * @param label the label to add
     */
    public void addLabel(String label)
    {
        Set labels = (Set)messageMap.get(LABEL_KEY);
        Label newLabel = new Label(label);
        
        if (!labels.contains(newLabel))
        {
            labels.add(newLabel);
        }
        
        /*if (label.compareTo("CHANGE-SPEAKER") == 0)
        {
            labels[CHANGESPEAKER] = true;
        }
        else if (label.compareTo("CHANGE-ROOM") == 0)
        {
            labels[CHANGEROOM] = true;
        }
        else if (label.compareTo("CHANGE-SESSION") == 0)
        {
            labels[CHANGESESSION] = true;
        }
        else if (label.compareTo("BRIEFING") == 0)
        {
            labels[BRIEFING] = true;
        }
        else if (label.compareTo("WEB-VIO") == 0)
        {
            labels[WEBVIO] = true;
        }
        else if (label.compareTo("WEB-WBE") == 0)
        {
            labels[WEBWBE] = true;
        }
        else if (label.compareTo("INFO-REQ") == 0)
        {
            labels[INFOREQ] = true;
        }
        else if (label.compareTo("MISC-ACTION") == 0)
        {
            labels[MISCACTION] = true;
        }*/
    }

    /**
     * 
     * @param subject -
     *            the subject this messages subject should be set to
     */
    public void setSubject(final String subject) {
        messageMap.put("subject", subject);
    }

    /**
     * 
     * @param body -
     *            the body this message should have
     */
    public void setBody(final String body) {
        messageMap.put("body", body);
    }

    /**
     * A function to convert an RMessage to xml format
     * 
     * @param message -
     *            the message to be encoded into xml
     * @return the message encoded in xml
     */
    public String toXML(final RMessage message) {
        String asString = getXMLBeforeEmails(message);
        asString += ("\t<from>" + (revalidate(message.getFrom())) + "</from>\n");
        asString += ("\t<recipients>\n");
        
        final ArrayList toRecipients = getTo();
        final ArrayList ccRecipients = getCC();
        final ArrayList bccRecipients = getBCC();
        
        for (int i = 0; i < toRecipients.size(); i++)
        {
            asString += ("\t\t<to>" + revalidate((String)toRecipients.get(i)) + "</to>\n");
        }
        if (ccRecipients != null)
        {
            for (int j = 0; j < ccRecipients.size(); j++)
            {
                asString += ("\t\t<cc>" + revalidate((String)ccRecipients.get(j))
                        + "</cc>\n");
            }
        }
        if (bccRecipients != null)
        {
            for (int k = 0; k < bccRecipients.size(); k++)
            {
                asString += ("\t\t<bcc>" + revalidate((String)bccRecipients.get(k))
                        + "</bcc>\n");
            }
        }

        asString += ("\t</recipients>\n");
        asString += ("\t<subject>" + revalidate(message.getSubject()) + "</subject>\n");
        asString += ("\t<body>" + revalidate(message.getBody()) + "\n\t</body>\n");

        asString += ("</email>\n");
        asString += ("\n");

        return asString;
    }

    /**
     * transforms some of the header data in the message to a xml header
     * @param message the message to get the data out of
     * @return  a string containing the xml header information
     */
    private String getXMLBeforeEmails(final RMessage message) {
        String asString = "<email>\n";

        if (message.getBackstory()){ // add a special parameter in the time tag if this
                        // message is backstory
            asString += ("\t<time mode=\"backstory\">" + message.getTime() + TIMEENDTAG);
        }
        else
        {
            asString += ("\t<time>" + message.getTime() + TIMEENDTAG);
        }
            
        asString += ("\t<period>" + message.getPeriod() + "</period>\n");
        asString += ("\t<group>" + message.getGroup() + "</group>\n");
        asString += ("\t<outcome>" + message.getOutcome() + "</outcome>\n");
        asString += ("\t<thread>" + message.getThread() + "</thread>\n");

        asString = getLabelXML(asString);

        asString += ("\t<VIP>" + message.getVIP() + "</VIP>\n");
        asString += ("\t<noise>" + message.getNoise() + "</noise>\n");
        asString += ("\t<response>" + message.getResponse() + "</response>\n");
        asString += ("\t<messageid>" + message.getId() + "</messageid>\n");
        asString += ("\t<replyto>" + message.getReplyTo() + "</replyto>\n");
        return asString;
    }

    /**
     * transforms all characters that xml doesn't like into their equivelant character codes
     * @param s the string to transform
     * @return the transformed string
     */
    private String revalidate(final String s) {
        String string;
        string = s.replaceAll("&", "&amp;");
        string = string.replaceAll("<", "&lt;");
        string = string.replaceAll(">", "&gt;");
        return string;
    }
    
    /**
     * A function to convert an RMessage to injector xml format
     * 
     * @param message -
     *            the message to be encoded into xml
     * @return the message encoded in xml
     */
    public String toBrokenXML(final RMessage message) {
        String asString = getXMLBeforeEmails(message);
        asString += ("\t<from>" + message.getFrom() + "</from>\n");
        asString += ("\t<recipients>\n");
        
        final ArrayList toRecipients = getTo();
        final ArrayList ccRecipients = getCC();
        final ArrayList bccRecipients = getBCC();
        
        for (int i = 0; i < toRecipients.size(); i++)
        {
            asString += ("\t\t<to>" + (String)toRecipients.get(i) + "</to>\n");
        }
        if (ccRecipients != null)
        {
            for (int j = 0; j < ccRecipients.size(); j++)
            {
                asString += ("\t\t<cc>" + (String)ccRecipients.get(j)
                        + "</cc>\n");
            }
        }
        if (bccRecipients != null)
        {
            for (int k = 0; k < bccRecipients.size(); k++)
            {
                asString += ("\t\t<bcc>" + (String)bccRecipients.get(k)
                        + "</bcc>\n");
            }
        }
        asString += ("\t</recipients>\n");
        asString += ("\t<subject>" + message.getSubject() + "</subject>\n");
        asString += ("\t<body>" + message.getBody() + "\n\t</body>\n");

        asString += ("</email>\n");
        asString += ("\n");

        return asString;
    }

    private String labelXML(String labelName)
    {
        return "\t<label>" + labelName + "</label>\n";
    }
    
    /**
     * Trsndoems the labels into xml
     * @param asString the string to attach the label codes to
     * @return the string with labels attached
     */
    private String getLabelXML(String asString) {
        StringBuffer temp = new StringBuffer();
        Set labels = ((Set)messageMap.get(LABEL_KEY));
        //labels might be null if they don't exist or this message doesn't have any
        if (labels != null)
        {
            Iterator labelsItr = labels.iterator();
        
            while (labelsItr.hasNext())
            {
                temp.append(labelXML((String)labelsItr.next()));
            }
        }
        
        return asString;
    }
    

    /**
     * returns a string of this message in xml format
     * 
     * @return this message in xml string format
     */
    public String toString() {
        return toXML(this);
    }

    /**
     * Creates a copy of the calling RMessage object
     * 
     * @return the copy
     */
    public RMessage copy() {
        final RMessage copied = new RMessage();

        copied.setTime(getTime());
        copied.setPeriod(getPeriod());
        copied.setGroup(getGroup());
        copied.setOutcome(getOutcome());
        copied.setThread(getThread());
        copied.setVIP(getVIP());
        copied.setNoise(getNoise());
        copied.setResponse(getResponse());
        copied.setBackstory(getBackstory());
        copied.setBCC(getBCC());
        copied.setBody(getBody());
        copied.setCC(getCC());
        copied.setFrom(getFrom());
        copied.setLabels(getLabels());
        copied.setId(getId());
        copied.setReplyTo(getReplyTo());
        copied.setSubject(getSubject());
        copied.setTo(getTo());

        return copied;
    }

    /**
     * get all the recipients addresses, and store in a Vector
     * @param l the list of nodes
     * @return a vector of all the recipients addresses
     * 
     */ 
    public static ArrayList getAll(final NodeList l) {
        final ArrayList newVect = new ArrayList();

        for (int i = (l.getLength() - 1); i >= 0; i--)
        {
            newVect.add(l.item(i).getNodeValue());
        }

        return newVect;
    };
    
    /**
     * This function consolidates how to set a field in the message into a function, because
     * of the hashMap implementation, this is needed... also separates implemtation from api
     * @param field the name of the field to set
     * @param value the value to set the field to
     */
    private void setField(final String field, final String value)
    {
        if (field.equals("to") || field.equals("cc") || field.equals(BCCSTRING))
        {
            ArrayList list;
            if (messageMap.containsKey(field))
            {
                list = (ArrayList)messageMap.get(field);
            } else
            {
                list = new ArrayList();
            }
            list.add(value);
            messageMap.put(field, list);
        }else
        {
           messageMap.put(field, value);
        }
    }
    
    
    /**
     * 
     * @param emailTree the list of nodes to break this email off of 
     * @return the RMessage that was created from this node
     */
    public static RMessage getEmail(final Node emailTree) {
       
        RMessage mess = new RMessage();
        Node temp;
        Node temp2;

        //parse the xml tree that was generated above into rMessages
        temp = emailTree.getFirstChild();

            
        while (temp != null)
        {
            if (temp.getNodeType() == Node.ELEMENT_NODE)
            {
                final String LOADINGERROR = "Cannot load messages. \n\n ";
                if (temp.getNodeName().compareTo("time") == 0) //time comes next
                {
                    if (temp.toString().indexOf("mode") >= 0)
                    {
                        mess.setBackstory(true);
                    }
                    else
                    {
                        mess.setBackstory(false);
                    }
                    mess.setField(temp.getNodeName(), 
                            temp.getFirstChild().getNodeValue());

                } else if (temp.getNodeName().compareTo("label") == 0)
                {
                    parseLabels(mess, temp);
                } else if (temp.getNodeName().compareTo("from") == 0)
                {
                    //TODO fix me so that I don't error with the new sender format
                    String from = parseSenderOrRecipient(temp);
                    if (from== null)
                    {
                        if ((mess = parseEmail(mess, temp, LOADINGERROR)) == null)
                        {
                            return mess;
                        }
                    } else
                    {
                        mess.setFrom(from);
                    }
                } else if (temp.getNodeName().compareTo("recipients") == 0)
                {
                    temp2 = temp.getFirstChild();
                    final int numChild = temp.getChildNodes().getLength();

                    for (int k = 0; k < numChild; k++)
                    {
                        if ((mess = parseRecipients(mess, temp2, LOADINGERROR)) == null)
                        {
                            return mess;
                        }

                        temp2 = temp2.getNextSibling();
                    }
                } else if (temp.getNodeName().compareTo("subject") == 0)
                {
                    if (temp.getFirstChild().getNodeType() != Node.ELEMENT_NODE)
                    { // text node
                        String subject = temp.getFirstChild().getNodeValue();
                        subject = MessageLoader.revalidate(subject);
                        mess.setSubject(subject);
                    } else
                    {
                        temp2 = temp.getFirstChild();
                        // if the chunk contains a variable, insert
                        // the variable name so
                        // it could be edited if the user so chose
                        String varValue = temp2.getAttributes().getNamedItem("var").getNodeValue();
                        String otherAttributes = new String();
                        
                        varValue = MessageLoader.revalidate(varValue);
                        if (temp2.getAttributes().getNamedItem("fmt") != null)
                        {
                            otherAttributes += (" fmt = \"" + temp2.getAttributes().getNamedItem("fmt").getNodeValue()
                                    + "\" ");
                        }
                        
                        if (temp2.getAttributes().getNamedItem("offset") != null)
                        {
                            otherAttributes += (" offset = \"" + temp2.getAttributes().getNamedItem("offset").getNodeValue()
                                    + "\" ");
                        }
                        
                        
                       mess.setSubject("<echo var = \"" + varValue
                                    + "\" " + otherAttributes + "/>");
                    }
                } else if (temp.getNodeName().compareTo("body") == 0)
                {
                    String wholeBody;
                    wholeBody = new String();
                    temp2 = temp.getFirstChild();
                    while (temp2 != null)
                    {
                        if (temp2.getNodeType() != Node.ELEMENT_NODE)
                        { // text node
                            String body = temp2.getNodeValue();
                            body = MessageLoader.revalidate(body);
                            wholeBody += body;
                        } else
                        {
                            // if the chunk contains a variable, insert
                            // the variable name so
                            // it could be edited if the user so chose
                            String varValue = temp2.getAttributes().getNamedItem("var").getNodeValue();
                            String otherAttributes = new String();
                            
                            varValue = MessageLoader.revalidate(varValue);
                            //System.out.println(mess.getId() + " fmt " + temp2.getAttributes().getNamedItem("fmt"));
                            if (temp2.getAttributes().getNamedItem("fmt") != null)
                            {
                                otherAttributes += (" fmt = \"" + temp2.getAttributes().getNamedItem("fmt").getNodeValue()
                                        + "\" ");
                            }
                            
                            if (temp2.getAttributes().getNamedItem("offset") != null)
                            {
                                otherAttributes += (" offset = \"" + temp2.getAttributes().getNamedItem("offset").getNodeValue()
                                        + "\" ");
                            }
                            
                            
                           wholeBody += ("<echo var = \"" + varValue
                                        + "\" " + otherAttributes + "/>");
                        }
                        temp2 = temp2.getNextSibling();
                    }
                    mess.setBody(wholeBody);
                } else
                {
                    mess.setField(temp.getNodeName(), 
                            temp.getFirstChild().getNodeValue());
                }
            }
            temp = temp.getNextSibling();
        }

        return mess;
    }

    private static RMessage parseEmail(final RMessage mess, final Node temp, final String LOADINGERROR) {
        if (temp.getFirstChild().getNodeType() != Node.ELEMENT_NODE)
        {
            final String email = parseSenderOrRecipient(temp);//temp.getFirstChild().getNodeValue();
            // the node is not a variable
            
            if (email != null 
                    && (email.matches(RMessage.EMAILVERIFICATION) 
                            || email.matches(Variable.VARIABLEVERIFICATION))) {
                mess.setField(temp.getNodeName(), email);
            }
            else
            {
                //System.out.println(email);
                JOptionPane.showMessageDialog(null,
                        LOADINGERROR
                        +"The recipient field appears to be invalid in message " 
                        + mess.getId() + "\n" +  MessageLoader.ENTER_VALID_VARIABLE, 
                        MessageLoader.ERROR_BOX_TITLE,
                        JOptionPane.ERROR_MESSAGE);
                return null;
            }
                
        } else
        {
            // this is a variable
            dealWithVariables(mess, temp);
        }
        return mess;
    }
    
    private static String parseSenderOrRecipient(final Node temp)
    {
        String result = null;
        Node thisChild = temp.getFirstChild();
        //Node thisChild = child.getFirstChild();
        
        while(thisChild != null)
        {
            if (thisChild.getNodeName().equals("personal"))
            {
                if (result != null){
                    result = thisChild.getFirstChild().getNodeValue() + result;
                } else
                {
                    result = thisChild.getFirstChild().getNodeValue();
                }
            } else if (thisChild.getNodeName().equals("addr"))
            {
                if (result != null)
                {
                    result += "<" + thisChild.getFirstChild().getNodeValue() + ">";
                } else
                {
                    result = "<" + thisChild.getFirstChild().getNodeValue() + ">";
                }
            } else {
                //maintain backwards compatability
                if (thisChild != null && thisChild.getNodeValue() != null &&
                        thisChild.getNodeValue().matches(MessageType.EMAILVERIFICATION))
                {
                    result = thisChild.getNodeValue();
                }
            }
            thisChild = thisChild.getNextSibling();
        }
        
        return result;
    }
    
    
    private static RMessage parseRecipients(final RMessage mess, final Node temp2, final String error) {
        if (temp2.getNodeType() == Node.ELEMENT_NODE)
        {
            return parseEmail(mess, temp2, error);
        }
            
        return mess;
    }

    private static void parseLabels(final RMessage mess, final Node temp) {
        final ArrayList labels = getAll(temp.getChildNodes());
        boolean[] label = new boolean[8];
        Set labelSet = new HashSet();
        
        for (int j = 0; j < labels.size(); j++)
        {
            labelSet.add(labels.get(j));
            /*if (((String)labels.get(j))
                    .compareTo("CHANGE-SPEAKER") == 0)
            {
                label[0] = true;
            }
            else if (((String)labels.get(j))
                    .compareTo("CHANGE-ROOM") == 0)
            {
                label[1] = true;
            }
            else if (((String)labels.get(j))
                    .compareTo("CHANGE-SESSION") == 0)
            {
                label[2] = true;
            }
            else if (((String)labels.get(j))
                    .compareTo("BRIEFING") == 0)
            {
                label[3] = true;
            }
            else if (((String)labels.get(j))
                    .compareTo("WEB-VIO") == 0)
            {
                label[4] = true;
            }
            else if (((String)labels.get(j))
                    .compareTo("WEB-WBE") == 0)
            {
                label[5] = true;
            }
            else if (((String)labels.get(j))
                    .compareTo("INFO-REQ") == 0)
            {
                label[6] = true;
            }
            else if (((String)labels.get(j))
                    .compareTo("MISC-ACTION") == 0)
            {
                label[7] = true;
            }*/
         }

        mess.setLabels(labelSet);
    }

    private static void dealWithVariables(final RMessage mess, final Node temp) {
        String value = temp.getFirstChild().getAttributes().getNamedItem("var").getNodeValue();
        value = MessageLoader.revalidate(value);
        
        mess.setField(temp.getNodeName(), "<echo var=\""
                    + value
                    + "\" />");
    }

}